@@ -1,4 +1,5 @@
 use super::{Array, ArrayRef, Splitable, new_empty_array, new_null_array};
+use crate::array::list::LIST_VALUES_NAME;
 use crate::bitmap::Bitmap;
 use crate::datatypes::{ArrowDataType, Field};
 
@@ -12,7 +13,8 @@ mod mutable;
 pub use mutable::*;
 use polars_error::{PolarsResult, polars_bail, polars_ensure};
 use polars_utils::format_tuple;
-use polars_utils::pl_str::PlSmallStr;
+#[cfg(feature = "proptest")]
+pub mod proptest;
 
 use crate::datatypes::reshape::{Dimension, ReshapeDimension};
 
@@ -227,32 +229,6 @@ impl FixedSizeListArray {
         }
         dims
     }
-
-    pub fn propagate_nulls(&self) -> Self {
-        let Some(validity) = self.validity() else {
-            return self.clone();
-        };
-
-        let propagated_validity = if self.size == 1 {
-            validity.clone()
-        } else {
-            Bitmap::from_trusted_len_iter(
-                (0..self.size * validity.len())
-                    .map(|i| unsafe { validity.get_bit_unchecked(i / self.size) }),
-            )
-        };
-
-        let propagated_validity = match self.values.validity() {
-            None => propagated_validity,
-            Some(val) => val & &propagated_validity,
-        };
-        Self::new(
-            self.dtype().clone(),
-            self.length,
-            self.values.with_validity(Some(propagated_validity)),
-            self.validity.clone(),
-        )
-    }
 }
 
 // must use
@@ -358,7 +334,7 @@ impl FixedSizeListArray {
 
     /// Returns a [`ArrowDataType`] consistent with [`FixedSizeListArray`].
     pub fn default_datatype(dtype: ArrowDataType, size: usize) -> ArrowDataType {
-        let field = Box::new(Field::new(PlSmallStr::from_static("item"), dtype, true));
+        let field = Box::new(Field::new(LIST_VALUES_NAME, dtype, true));
         ArrowDataType::FixedSizeList(field, size)
     }
 }
