@@ -15,12 +15,13 @@ mod field;
 mod into_scalar;
 #[cfg(feature = "object")]
 mod static_array_collect;
-mod time_unit;
+mod temporal;
 
 use std::cmp::Ordering;
 use std::fmt::{Display, Formatter};
 use std::hash::{Hash, Hasher};
 use std::ops::{Add, AddAssign, Div, Mul, Rem, Sub, SubAssign};
+use std::sync::Arc;
 
 mod schema;
 pub use aliases::*;
@@ -38,6 +39,11 @@ pub use into_scalar::*;
 use num_traits::{AsPrimitive, Bounded, FromPrimitive, Num, NumCast, One, Zero};
 use polars_compute::arithmetic::HasPrimitiveArithmeticKernel;
 use polars_compute::float_sum::FloatSum;
+#[cfg(feature = "dtype-categorical")]
+pub use polars_dtype::categorical::{
+    CatNative, CatSize, CategoricalMapping, CategoricalPhysical, Categories, FrozenCategories,
+    ensure_same_categories, ensure_same_frozen_categories,
+};
 use polars_utils::abs_diff::AbsDiff;
 use polars_utils::float::IsFloat;
 use polars_utils::min_max::MinMax;
@@ -45,12 +51,12 @@ use polars_utils::nulls::IsNull;
 use polars_utils::total_ord::TotalHash;
 pub use schema::SchemaExtPl;
 #[cfg(feature = "serde")]
-use serde::de::{EnumAccess, VariantAccess, Visitor};
+use serde::de::Visitor;
 #[cfg(any(feature = "serde", feature = "serde-lazy"))]
 use serde::{Deserialize, Serialize};
 #[cfg(any(feature = "serde", feature = "serde-lazy"))]
 use serde::{Deserializer, Serializer};
-pub use time_unit::*;
+pub use temporal::*;
 
 pub use crate::chunked_array::logical::*;
 #[cfg(feature = "object")]
@@ -58,6 +64,7 @@ use crate::chunked_array::object::ObjectArray;
 #[cfg(feature = "object")]
 use crate::chunked_array::object::PolarsObjectSafe;
 use crate::prelude::*;
+use crate::series::implementations::SeriesWrap;
 use crate::utils::Wrap;
 
 pub struct TrueT;
@@ -78,14 +85,21 @@ pub unsafe trait PolarsDataType: Send + Sync + Sized + 'static {
     type HasViews;
     type IsStruct;
     type IsObject;
-    type IsLogical;
 
-    fn get_dtype() -> DataType
+    /// Returns the DataType variant associated with this PolarsDataType.
+    /// Not implemented for types whose DataTypes have parameters.
+    fn get_static_dtype() -> DataType
     where
         Self: Sized;
 }
 
-pub trait PolarsNumericType: 'static
+pub trait PolarsPhysicalType: PolarsDataType {
+    // A physical type is one backed by a ChunkedArray directly, as opposed to
+    // logical types which wrap physical data.
+    fn ca_into_series(ca: ChunkedArray<Self>) -> Series;
+}
+
+pub trait PolarsNumericType: PolarsPhysicalType + 'static
 where
     Self: for<'a> PolarsDataType<
             OwnedPhysical = Self::Native,
@@ -96,7 +110,6 @@ where
             HasViews = FalseT,
             IsStruct = FalseT,
             IsObject = FalseT,
-            IsLogical = FalseT,
         >,
 {
     type Native: NumericNative;
@@ -105,12 +118,22 @@ where
 pub trait PolarsIntegerType: PolarsNumericType {}
 pub trait PolarsFloatType: PolarsNumericType {}
 
-macro_rules! impl_polars_num_datatype {
-    ($trait: ident, $ca:ident, $variant:ident, $physical:ty, $owned_phys:ty) => {
-        #[derive(Clone, Copy)]
-        pub struct $ca {}
+/// # Safety
+/// The physical() return type must be correct for Native.
+#[cfg(feature = "dtype-categorical")]
+pub unsafe trait PolarsCategoricalType: PolarsDataType {
+    type Native: NumericNative + CatNative + DictionaryKey + PartialEq + Eq + Hash;
+    type PolarsPhysical: PolarsIntegerType<Native = Self::Native>;
 
-        unsafe impl PolarsDataType for $ca {
+    fn physical() -> CategoricalPhysical;
+}
+
+macro_rules! impl_polars_num_datatype {
+    ($trait: ident, $pdt:ident, $variant:ident, $physical:ty, $owned_phys:ty) => {
+        #[derive(Clone, Copy)]
+        pub struct $pdt {}
+
+        unsafe impl PolarsDataType for $pdt {
             type Physical<'a> = $physical;
             type OwnedPhysical = $owned_phys;
             type ZeroablePhysical<'a> = $physical;
@@ -119,28 +142,27 @@ macro_rules! impl_polars_num_datatype {
             type HasViews = FalseT;
             type IsStruct = FalseT;
             type IsObject = FalseT;
-            type IsLogical = FalseT;
 
             #[inline]
-            fn get_dtype() -> DataType {
+            fn get_static_dtype() -> DataType {
                 DataType::$variant
             }
         }
 
-        impl PolarsNumericType for $ca {
+        impl PolarsNumericType for $pdt {
             type Native = $physical;
         }
 
-        impl $trait for $ca {}
+        impl $trait for $pdt {}
     };
 }
 
-macro_rules! impl_polars_datatype_pass_dtype {
-    ($ca:ident, $dtype:expr, $arr:ty, $lt:lifetime, $phys:ty, $zerophys:ty, $owned_phys:ty, $has_views:ident, $is_logical:ident) => {
+macro_rules! impl_polars_datatype {
+    ($pdt:ident, $dtype:expr, $arr:ty, $lt:lifetime, $phys:ty, $zerophys:ty, $owned_phys:ty, $has_views:ident) => {
         #[derive(Clone, Copy)]
-        pub struct $ca {}
+        pub struct $pdt {}
 
-        unsafe impl PolarsDataType for $ca {
+        unsafe impl PolarsDataType for $pdt {
             type Physical<$lt> = $phys;
             type OwnedPhysical = $owned_phys;
             type ZeroablePhysical<$lt> = $zerophys;
@@ -149,45 +171,38 @@ macro_rules! impl_polars_datatype_pass_dtype {
             type HasViews = $has_views;
             type IsStruct = FalseT;
             type IsObject = FalseT;
-            type IsLogical = $is_logical;
 
             #[inline]
-            fn get_dtype() -> DataType {
+            fn get_static_dtype() -> DataType {
                 $dtype
             }
         }
     };
 }
-macro_rules! impl_polars_binview_datatype {
-    ($ca:ident, $variant:ident, $arr:ty, $lt:lifetime, $phys:ty, $zerophys:ty, $owned_phys:ty) => {
-        impl_polars_datatype_pass_dtype!(
-            $ca,
-            DataType::$variant,
-            $arr,
-            $lt,
-            $phys,
-            $zerophys,
-            $owned_phys,
-            TrueT,
+
+macro_rules! impl_polars_categorical_datatype {
+    ($pdt:ident, $phys:ty, $native:ty, $phys_variant:ident) => {
+        impl_polars_datatype!(
+            $pdt,
+            unimplemented!(),
+            PrimitiveArray<$native>,
+            'a,
+            $native,
+            $native,
+            $native,
             FalseT
         );
-    };
-}
 
-macro_rules! impl_polars_datatype {
-    ($ca:ident, $variant:ident, $arr:ty, $lt:lifetime, $phys:ty, $zerophys:ty, $owned_phys:ty, $is_logical:ident) => {
-        impl_polars_datatype_pass_dtype!(
-            $ca,
-            DataType::$variant,
-            $arr,
-            $lt,
-            $phys,
-            $zerophys,
-            $owned_phys,
-            FalseT,
-            $is_logical
-        );
-    };
+        #[cfg(feature = "dtype-categorical")]
+        unsafe impl PolarsCategoricalType for $pdt {
+            type Native = $native;
+            type PolarsPhysical = $phys;
+
+            fn physical() -> CategoricalPhysical {
+                CategoricalPhysical::$phys_variant
+            }
+        }
+    }
 }
 
 impl_polars_num_datatype!(PolarsIntegerType, UInt8Type, UInt8, u8, u8);
@@ -198,23 +213,27 @@ impl_polars_num_datatype!(PolarsIntegerType, Int8Type, Int8, i8, i8);
 impl_polars_num_datatype!(PolarsIntegerType, Int16Type, Int16, i16, i16);
 impl_polars_num_datatype!(PolarsIntegerType, Int32Type, Int32, i32, i32);
 impl_polars_num_datatype!(PolarsIntegerType, Int64Type, Int64, i64, i64);
-
 #[cfg(feature = "dtype-i128")]
 impl_polars_num_datatype!(PolarsIntegerType, Int128Type, Int128, i128, i128);
 impl_polars_num_datatype!(PolarsFloatType, Float32Type, Float32, f32, f32);
 impl_polars_num_datatype!(PolarsFloatType, Float64Type, Float64, f64, f64);
-impl_polars_datatype!(DateType, Date, PrimitiveArray<i32>, 'a, i32, i32, i32, TrueT);
-impl_polars_datatype!(TimeType, Time, PrimitiveArray<i64>, 'a, i64, i64, i64, TrueT);
-impl_polars_binview_datatype!(StringType, String, Utf8ViewArray, 'a, &'a str, Option<&'a str>, String);
-impl_polars_binview_datatype!(BinaryType, Binary, BinaryViewArray, 'a, &'a [u8], Option<&'a [u8]>, Box<[u8]>);
-impl_polars_datatype!(BinaryOffsetType, BinaryOffset, BinaryArray<i64>, 'a, &'a [u8], Option<&'a [u8]>, Box<[u8]>, FalseT);
-impl_polars_datatype!(BooleanType, Boolean, BooleanArray, 'a, bool, bool, bool, FalseT);
+
+impl_polars_datatype!(StringType, DataType::String, Utf8ViewArray, 'a, &'a str, Option<&'a str>, String, TrueT);
+impl_polars_datatype!(BinaryType, DataType::Binary, BinaryViewArray, 'a, &'a [u8], Option<&'a [u8]>, Box<[u8]>, TrueT);
+impl_polars_datatype!(BinaryOffsetType, DataType::BinaryOffset, BinaryArray<i64>, 'a, &'a [u8], Option<&'a [u8]>, Box<[u8]>, FalseT);
+impl_polars_datatype!(BooleanType, DataType::Boolean, BooleanArray, 'a, bool, bool, bool, FalseT);
 
 #[cfg(feature = "dtype-decimal")]
-impl_polars_datatype_pass_dtype!(DecimalType, DataType::Unknown(UnknownKind::Any), PrimitiveArray<i128>, 'a, i128, i128, i128, FalseT, TrueT);
-impl_polars_datatype_pass_dtype!(DatetimeType, DataType::Unknown(UnknownKind::Any), PrimitiveArray<i64>, 'a, i64, i64, i64, FalseT, TrueT);
-impl_polars_datatype_pass_dtype!(DurationType, DataType::Unknown(UnknownKind::Any), PrimitiveArray<i64>, 'a, i64, i64, i64, FalseT, TrueT);
-impl_polars_datatype_pass_dtype!(CategoricalType, DataType::Unknown(UnknownKind::Any), PrimitiveArray<u32>, 'a, u32, u32, u32, FalseT, TrueT);
+impl_polars_datatype!(DecimalType, unimplemented!(), PrimitiveArray<i128>, 'a, i128, i128, i128, FalseT);
+impl_polars_datatype!(DatetimeType, unimplemented!(), PrimitiveArray<i64>, 'a, i64, i64, i64, FalseT);
+impl_polars_datatype!(DurationType, unimplemented!(), PrimitiveArray<i64>, 'a, i64, i64, i64, FalseT);
+impl_polars_datatype!(CategoricalType, unimplemented!(), PrimitiveArray<u32>, 'a, u32, u32, u32, FalseT);
+impl_polars_datatype!(DateType, DataType::Date, PrimitiveArray<i32>, 'a, i32, i32, i32, FalseT);
+impl_polars_datatype!(TimeType, DataType::Time, PrimitiveArray<i64>, 'a, i64, i64, i64, FalseT);
+
+impl_polars_categorical_datatype!(Categorical8Type, UInt8Type, u8, U8);
+impl_polars_categorical_datatype!(Categorical16Type, UInt16Type, u16, U16);
+impl_polars_categorical_datatype!(Categorical32Type, UInt32Type, u32, U32);
 
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 pub struct ListType {}
@@ -227,14 +246,19 @@ unsafe impl PolarsDataType for ListType {
     type HasViews = FalseT;
     type IsStruct = FalseT;
     type IsObject = FalseT;
-    type IsLogical = FalseT;
 
-    fn get_dtype() -> DataType {
+    fn get_static_dtype() -> DataType {
         // Null as we cannot know anything without self.
         DataType::List(Box::new(DataType::Null))
     }
 }
 
+impl PolarsPhysicalType for ListType {
+    fn ca_into_series(ca: ChunkedArray<Self>) -> Series {
+        Series(Arc::new(SeriesWrap(ca)))
+    }
+}
+
 #[cfg(feature = "dtype-struct")]
 pub struct StructType {}
 #[cfg(feature = "dtype-struct")]
@@ -252,9 +276,8 @@ unsafe impl PolarsDataType for StructType {
     type HasViews = FalseT;
     type IsStruct = TrueT;
     type IsObject = FalseT;
-    type IsLogical = FalseT;
 
-    fn get_dtype() -> DataType
+    fn get_static_dtype() -> DataType
     where
         Self: Sized,
     {
@@ -274,9 +297,8 @@ unsafe impl PolarsDataType for FixedSizeListType {
     type HasViews = FalseT;
     type IsStruct = FalseT;
     type IsObject = FalseT;
-    type IsLogical = FalseT;
 
-    fn get_dtype() -> DataType {
+    fn get_static_dtype() -> DataType {
         // Null as we cannot know anything without self.
         DataType::Array(Box::new(DataType::Null), 0)
     }
@@ -294,13 +316,74 @@ unsafe impl<T: PolarsObject> PolarsDataType for ObjectType<T> {
     type HasViews = FalseT;
     type IsStruct = FalseT;
     type IsObject = TrueT;
-    type IsLogical = FalseT;
 
-    fn get_dtype() -> DataType {
+    fn get_static_dtype() -> DataType {
         DataType::Object(T::type_name())
     }
 }
 
+macro_rules! impl_phys_dtype {
+    ($pdt:ty) => {
+        impl PolarsPhysicalType for $pdt {
+            fn ca_into_series(ca: ChunkedArray<Self>) -> Series {
+                Series(Arc::new(SeriesWrap(ca)))
+            }
+        }
+    };
+}
+
+macro_rules! impl_cond_phys_dtype {
+    ($pdt:ty, $feat:literal) => {
+        impl PolarsPhysicalType for $pdt {
+            fn ca_into_series(ca: ChunkedArray<Self>) -> Series {
+                #[cfg(feature = $feat)]
+                {
+                    Series(Arc::new(SeriesWrap(ca)))
+                }
+
+                #[cfg(not(feature = $feat))]
+                {
+                    unimplemented!()
+                }
+            }
+        }
+    };
+}
+
+// Annoyingly these types always exist but may not have an implementation to refer to.
+impl_cond_phys_dtype!(UInt8Type, "dtype-u8");
+impl_cond_phys_dtype!(UInt16Type, "dtype-u16");
+impl_cond_phys_dtype!(Int8Type, "dtype-i8");
+impl_cond_phys_dtype!(Int16Type, "dtype-i16");
+
+impl_phys_dtype!(Int32Type);
+impl_phys_dtype!(Int64Type);
+impl_phys_dtype!(UInt32Type);
+impl_phys_dtype!(UInt64Type);
+impl_phys_dtype!(Float32Type);
+impl_phys_dtype!(Float64Type);
+
+impl_phys_dtype!(StringType);
+impl_phys_dtype!(BinaryType);
+impl_phys_dtype!(BinaryOffsetType);
+impl_phys_dtype!(BooleanType);
+
+#[cfg(feature = "dtype-i128")]
+impl_phys_dtype!(Int128Type);
+
+#[cfg(feature = "dtype-array")]
+impl_phys_dtype!(FixedSizeListType);
+
+#[cfg(feature = "dtype-struct")]
+impl_phys_dtype!(StructType);
+
+#[cfg(feature = "object")]
+impl<T: PolarsObject> PolarsPhysicalType for ObjectType<T> {
+    fn ca_into_series(ca: ChunkedArray<Self>) -> Series {
+        Series(Arc::new(SeriesWrap(ca)))
+    }
+}
+
 #[cfg(feature = "dtype-array")]
 pub type ArrayChunked = ChunkedArray<FixedSizeListType>;
 pub type ListChunked = ChunkedArray<ListType>;
