@@ -67,10 +67,11 @@ fn finish_as_iters<'a>(
         // Exploded list should be equal to groups length.
         list_vals_len == ac_truthy.groups.len()
     {
-        out = out.explode()?
+        out = out.explode(false)?
     }
 
     ac_truthy.with_agg_state(AggState::AggregatedList(out));
+    ac_truthy.with_update_groups(UpdateGroups::WithSeriesLen);
 
     Ok(ac_truthy)
 }
@@ -179,7 +180,6 @@ impl PhysicalExpr for TernaryExpr {
                     return Ok(AggregationContext {
                         state: NotAggregated(out),
                         groups: ac_target.groups.clone(),
-                        sorted: ac_target.sorted,
                         update_groups: ac_target.update_groups,
                         original_len: ac_target.original_len,
                     });
@@ -322,7 +322,6 @@ impl PhysicalExpr for TernaryExpr {
         Ok(AggregationContext {
             state: agg_state_out,
             groups: ac_target.groups.clone(),
-            sorted: ac_target.sorted,
             update_groups: ac_target.update_groups,
             original_len: ac_target.original_len,
         })
