@@ -1,12 +1,14 @@
 use std::borrow::Cow;
-use std::sync::atomic::{AtomicBool, AtomicI64, AtomicU8, Ordering};
-use std::sync::{Mutex, RwLock};
+use std::sync::atomic::AtomicI64;
+use std::sync::{Mutex, OnceLock, RwLock};
+use std::time::Duration;
 
 use bitflags::bitflags;
-use once_cell::sync::OnceCell;
 use polars_core::config::verbose;
 use polars_core::prelude::*;
 use polars_ops::prelude::ChunkJoinOptIds;
+use polars_utils::relaxed_cell::RelaxedCell;
+use polars_utils::unique_id::UniqueId;
 
 use super::NodeTimer;
 
@@ -68,9 +70,6 @@ bitflags! {
         const CACHE_WINDOW_EXPR = 0x02;
         /// Indicates the expression has a window function
         const HAS_WINDOW = 0x04;
-        /// If set, the expression is evaluated in the
-        /// streaming engine.
-        const IN_STREAMING = 0x08;
     }
 }
 
@@ -100,21 +99,22 @@ impl From<u8> for StateFlags {
     }
 }
 
-type CachedValue = Arc<(AtomicI64, OnceCell<DataFrame>)>;
+type CachedValue = Arc<(AtomicI64, OnceLock<DataFrame>)>;
 
 /// State/ cache that is maintained during the Execution of the physical plan.
+#[derive(Clone)]
 pub struct ExecutionState {
     // cached by a `.cache` call and kept in memory for the duration of the plan.
-    df_cache: Arc<Mutex<PlHashMap<usize, CachedValue>>>,
-    pub schema_cache: RwLock<Option<SchemaRef>>,
+    df_cache: Arc<RwLock<PlHashMap<UniqueId, CachedValue>>>,
+    pub schema_cache: Arc<RwLock<Option<SchemaRef>>>,
     /// Used by Window Expressions to cache intermediate state
     pub window_cache: Arc<WindowCache>,
     // every join/union split gets an increment to distinguish between schema state
     pub branch_idx: usize,
-    pub flags: AtomicU8,
+    pub flags: RelaxedCell<u8>,
     pub ext_contexts: Arc<Vec<DataFrame>>,
     node_timer: Option<NodeTimer>,
-    stop: Arc<AtomicBool>,
+    stop: Arc<RelaxedCell<bool>>,
 }
 
 impl ExecutionState {
@@ -128,16 +128,16 @@ impl ExecutionState {
             schema_cache: Default::default(),
             window_cache: Default::default(),
             branch_idx: 0,
-            flags: AtomicU8::new(StateFlags::init().as_u8()),
+            flags: RelaxedCell::from(StateFlags::init().as_u8()),
             ext_contexts: Default::default(),
             node_timer: None,
-            stop: Arc::new(AtomicBool::new(false)),
+            stop: Arc::new(RelaxedCell::from(false)),
         }
     }
 
     /// Toggle this to measure execution times.
-    pub fn time_nodes(&mut self) {
-        self.node_timer = Some(NodeTimer::new())
+    pub fn time_nodes(&mut self, start: std::time::Instant) {
+        self.node_timer = Some(NodeTimer::new(start))
     }
     pub fn has_node_timer(&self) -> bool {
         self.node_timer.is_some()
@@ -147,14 +147,26 @@ impl ExecutionState {
         self.node_timer.unwrap().finish()
     }
 
+    // Timings should be a list of (start, end, name) where the start
+    // and end are raw durations since the query start as nanoseconds.
+    pub fn record_raw_timings(&self, timings: &[(u64, u64, String)]) {
+        for &(start, end, ref name) in timings {
+            self.node_timer.as_ref().unwrap().store_duration(
+                Duration::from_nanos(start),
+                Duration::from_nanos(end),
+                name.to_string(),
+            );
+        }
+    }
+
     // This is wrong when the U64 overflows which will never happen.
     pub fn should_stop(&self) -> PolarsResult<()> {
         try_raise_keyboard_interrupt();
-        polars_ensure!(!self.stop.load(Ordering::Relaxed), ComputeError: "query interrupted");
+        polars_ensure!(!self.stop.load(), ComputeError: "query interrupted");
         Ok(())
     }
 
-    pub fn cancel_token(&self) -> Arc<AtomicBool> {
+    pub fn cancel_token(&self) -> Arc<RelaxedCell<bool>> {
         self.stop.clone()
     }
 
@@ -180,7 +192,7 @@ impl ExecutionState {
             schema_cache: Default::default(),
             window_cache: Default::default(),
             branch_idx: self.branch_idx,
-            flags: AtomicU8::new(self.flags.load(Ordering::Relaxed)),
+            flags: self.flags.clone(),
             ext_contexts: self.ext_contexts.clone(),
             node_timer: self.node_timer.clone(),
             stop: self.stop.clone(),
@@ -204,17 +216,28 @@ impl ExecutionState {
         lock.clone()
     }
 
-    pub fn get_df_cache(&self, key: usize, cache_hits: u32) -> CachedValue {
-        let mut guard = self.df_cache.lock().unwrap();
-        guard
-            .entry(key)
-            .or_insert_with(|| Arc::new((AtomicI64::new(cache_hits as i64), OnceCell::new())))
-            .clone()
+    pub fn get_df_cache(&self, key: &UniqueId, cache_hits: u32) -> CachedValue {
+        let guard = self.df_cache.read().unwrap();
+
+        match guard.get(key) {
+            Some(v) => v.clone(),
+            None => {
+                drop(guard);
+                let mut guard = self.df_cache.write().unwrap();
+
+                guard
+                    .entry(*key)
+                    .or_insert_with(|| {
+                        Arc::new((AtomicI64::new(cache_hits as i64), OnceLock::new()))
+                    })
+                    .clone()
+            },
+        }
     }
 
-    pub fn remove_df_cache(&self, key: usize) {
-        let mut guard = self.df_cache.lock().unwrap();
-        let _ = guard.remove(&key).unwrap();
+    pub fn remove_df_cache(&self, key: &UniqueId) {
+        let mut guard = self.df_cache.write().unwrap();
+        let _ = guard.remove(key).unwrap();
     }
 
     /// Clear the cache used by the Window expressions
@@ -223,26 +246,26 @@ impl ExecutionState {
     }
 
     fn set_flags(&self, f: &dyn Fn(StateFlags) -> StateFlags) {
-        let flags: StateFlags = self.flags.load(Ordering::Relaxed).into();
+        let flags: StateFlags = self.flags.load().into();
         let flags = f(flags);
-        self.flags.store(flags.as_u8(), Ordering::Relaxed);
+        self.flags.store(flags.as_u8());
     }
 
     /// Indicates that window expression's [`GroupTuples`] may be cached.
     pub fn cache_window(&self) -> bool {
-        let flags: StateFlags = self.flags.load(Ordering::Relaxed).into();
+        let flags: StateFlags = self.flags.load().into();
         flags.contains(StateFlags::CACHE_WINDOW_EXPR)
     }
 
     /// Indicates that window expression's [`GroupTuples`] may be cached.
     pub fn has_window(&self) -> bool {
-        let flags: StateFlags = self.flags.load(Ordering::Relaxed).into();
+        let flags: StateFlags = self.flags.load().into();
         flags.contains(StateFlags::HAS_WINDOW)
     }
 
     /// More verbose logging
     pub fn verbose(&self) -> bool {
-        let flags: StateFlags = self.flags.load(Ordering::Relaxed).into();
+        let flags: StateFlags = self.flags.load().into();
         flags.contains(StateFlags::VERBOSE)
     }
 
@@ -273,19 +296,3 @@ impl Default for ExecutionState {
         ExecutionState::new()
     }
 }
-
-impl Clone for ExecutionState {
-    /// clones, but clears no state.
-    fn clone(&self) -> Self {
-        Self {
-            df_cache: self.df_cache.clone(),
-            schema_cache: self.schema_cache.read().unwrap().clone().into(),
-            window_cache: self.window_cache.clone(),
-            branch_idx: self.branch_idx,
-            flags: AtomicU8::new(self.flags.load(Ordering::Relaxed)),
-            ext_contexts: self.ext_contexts.clone(),
-            node_timer: self.node_timer.clone(),
-            stop: self.stop.clone(),
-        }
-    }
-}
