@@ -149,7 +149,7 @@ pub(crate) fn init_buffers(
     schema: &Schema,
     capacity: usize,
     ignore_errors: bool,
-) -> PolarsResult<PlIndexMap<BufferKey, Buffer>> {
+) -> PolarsResult<PlIndexMap<BufferKey<'_>, Buffer<'_>>> {
     schema
         .iter()
         .map(|(name, dtype)| {
@@ -341,7 +341,7 @@ fn deserialize_all<'a>(
                 );
             }
         },
-        val => AnyValue::StringOwned(format!("{:#?}", val).into()),
+        val => AnyValue::StringOwned(format!("{val:#?}").into()),
     };
     Ok(out)
 }
