@@ -1,10 +1,10 @@
 use std::fmt;
 
 use arrow::array::Array;
-use arrow::bitmap::{Bitmap, MutableBitmap};
+use arrow::bitmap::{Bitmap, BitmapBuilder};
 use polars_core::prelude::*;
 #[cfg(feature = "parquet")]
-use polars_parquet::read::expr::{ParquetColumnExpr, ParquetScalar, ParquetScalarRange};
+use polars_parquet::read::expr::{ParquetColumnExpr, ParquetScalar, SpecializedParquetColumnExpr};
 use polars_utils::format_pl_smallstr;
 #[cfg(feature = "serde")]
 use serde::{Deserialize, Serialize};
@@ -13,26 +13,27 @@ pub trait PhysicalIoExpr: Send + Sync {
     /// Take a [`DataFrame`] and produces a boolean [`Series`] that serves
     /// as a predicate mask
     fn evaluate_io(&self, df: &DataFrame) -> PolarsResult<Series>;
-
-    /// Can take &dyn Statistics and determine of a file should be
-    /// read -> `true`
-    /// or not -> `false`
-    fn as_stats_evaluator(&self) -> Option<&dyn StatsEvaluator> {
-        None
-    }
 }
 
 #[derive(Debug, Clone)]
-pub enum SpecializedColumnPredicateExpr {
-    Eq(Scalar),
-    EqMissing(Scalar),
+pub enum SpecializedColumnPredicate {
+    Equal(Scalar),
+
+    Between(Scalar, Scalar),
+
+    EqualOneOf(Box<[Scalar]>),
+
+    StartsWith(Box<[u8]>),
+    EndsWith(Box<[u8]>),
+    StartEndsWith(Box<[u8]>, Box<[u8]>),
 }
 
 #[derive(Clone)]
 pub struct ColumnPredicateExpr {
     column_name: PlSmallStr,
     dtype: DataType,
-    specialized: Option<SpecializedColumnPredicateExpr>,
+    #[cfg(feature = "parquet")]
+    specialized: Option<SpecializedParquetColumnExpr>,
     expr: Arc<dyn PhysicalIoExpr>,
 }
 
@@ -41,37 +42,51 @@ impl ColumnPredicateExpr {
         column_name: PlSmallStr,
         dtype: DataType,
         expr: Arc<dyn PhysicalIoExpr>,
-        specialized: Option<SpecializedColumnPredicateExpr>,
+        specialized: Option<SpecializedColumnPredicate>,
     ) -> Self {
+        use SpecializedColumnPredicate as S;
+        #[cfg(feature = "parquet")]
+        use SpecializedParquetColumnExpr as P;
+        #[cfg(feature = "parquet")]
+        let specialized = specialized.and_then(|s| {
+            Some(match s {
+                S::Equal(s) => P::Equal(cast_to_parquet_scalar(s)?),
+                S::Between(low, high) => {
+                    P::Between(cast_to_parquet_scalar(low)?, cast_to_parquet_scalar(high)?)
+                },
+                S::EqualOneOf(scalars) => P::EqualOneOf(
+                    scalars
+                        .into_iter()
+                        .map(|s| cast_to_parquet_scalar(s).ok_or(()))
+                        .collect::<Result<Box<_>, ()>>()
+                        .ok()?,
+                ),
+                S::StartsWith(s) => P::StartsWith(s),
+                S::EndsWith(s) => P::EndsWith(s),
+                S::StartEndsWith(start, end) => P::StartEndsWith(start, end),
+            })
+        });
+
         Self {
             column_name,
             dtype,
+            #[cfg(feature = "parquet")]
             specialized,
             expr,
         }
     }
-
-    pub fn is_eq_scalar(&self) -> bool {
-        self.to_eq_scalar().is_some()
-    }
-    pub fn to_eq_scalar(&self) -> Option<&Scalar> {
-        match &self.specialized {
-            Some(SpecializedColumnPredicateExpr::Eq(sc)) if !sc.is_null() => Some(sc),
-            Some(SpecializedColumnPredicateExpr::EqMissing(sc)) => Some(sc),
-            _ => None,
-        }
-    }
 }
 
 #[cfg(feature = "parquet")]
 impl ParquetColumnExpr for ColumnPredicateExpr {
-    fn evaluate_mut(&self, values: &dyn Array, bm: &mut MutableBitmap) {
+    fn evaluate_mut(&self, values: &dyn Array, bm: &mut BitmapBuilder) {
         // We should never evaluate nulls with this.
         assert!(values.validity().is_none_or(|v| v.set_bits() == 0));
 
-        // @NOTE: This is okay because we don't have Enums, Categoricals, or Decimals
+        // @TODO: Probably these unwraps should be removed.
         let series =
-            Series::from_arrow_chunks(self.column_name.clone(), vec![values.to_boxed()]).unwrap();
+            Series::from_chunk_and_dtype(self.column_name.clone(), values.to_boxed(), &self.dtype)
+                .unwrap();
         let column = series.into_column();
         let df = unsafe { DataFrame::new_no_checks(values.len(), vec![column]) };
 
@@ -82,8 +97,8 @@ impl ParquetColumnExpr for ColumnPredicateExpr {
         bm.reserve(true_mask.len());
         for chunk in true_mask.downcast_iter() {
             match chunk.validity() {
-                None => bm.extend(chunk.values()),
-                Some(v) => bm.extend(chunk.values() & v),
+                None => bm.extend_from_bitmap(chunk.values()),
+                Some(v) => bm.extend_from_bitmap(&(chunk.values() & v)),
             }
         }
     }
@@ -98,13 +113,8 @@ impl ParquetColumnExpr for ColumnPredicateExpr {
         true_mask.get(0).unwrap_or(false)
     }
 
-    fn to_equals_scalar(&self) -> Option<ParquetScalar> {
-        self.to_eq_scalar()
-            .and_then(|s| cast_to_parquet_scalar(s.clone()))
-    }
-
-    fn to_range_scalar(&self) -> Option<ParquetScalarRange> {
-        None
+    fn as_specialized(&self) -> Option<&SpecializedParquetColumnExpr> {
+        self.specialized.as_ref()
     }
 }
 
@@ -140,10 +150,9 @@ fn cast_to_parquet_scalar(scalar: Scalar) -> Option<ParquetScalar> {
 
         // @TODO: Cast to string
         #[cfg(feature = "dtype-categorical")]
-        A::Categorical(_, _, _)
-        | A::CategoricalOwned(_, _, _)
-        | A::Enum(_, _, _)
-        | A::EnumOwned(_, _, _) => return None,
+        A::Categorical(_, _) | A::CategoricalOwned(_, _) | A::Enum(_, _) | A::EnumOwned(_, _) => {
+            return None;
+        },
 
         A::String(v) => P::String(v.into()),
         A::StringOwned(v) => P::String(v.as_str().into()),
@@ -153,10 +162,6 @@ fn cast_to_parquet_scalar(scalar: Scalar) -> Option<ParquetScalar> {
     })
 }
 
-pub trait StatsEvaluator {
-    fn should_read(&self, stats: &BatchStats) -> PolarsResult<bool>;
-}
-
 #[cfg(any(feature = "parquet", feature = "ipc"))]
 pub fn apply_predicate(
     df: &mut DataFrame,
@@ -397,13 +402,8 @@ pub trait SkipBatchPredicate: Send + Sync {
 
 #[derive(Clone)]
 pub struct ColumnPredicates {
-    pub predicates: PlHashMap<
-        PlSmallStr,
-        (
-            Arc<dyn PhysicalIoExpr>,
-            Option<SpecializedColumnPredicateExpr>,
-        ),
-    >,
+    pub predicates:
+        PlHashMap<PlSmallStr, (Arc<dyn PhysicalIoExpr>, Option<SpecializedColumnPredicate>)>,
     pub is_sumwise_complete: bool,
 }
 
@@ -468,21 +468,37 @@ pub struct ScanIOPredicate {
 
     /// A predicate that gets given statistics and evaluates whether a batch can be skipped.
     pub column_predicates: Arc<ColumnPredicates>,
+
+    /// Predicate parts only referring to hive columns.
+    pub hive_predicate: Option<Arc<dyn PhysicalIoExpr>>,
+
+    pub hive_predicate_is_full_predicate: bool,
 }
+
 impl ScanIOPredicate {
     pub fn set_external_constant_columns(&mut self, constant_columns: Vec<(PlSmallStr, Scalar)>) {
+        if constant_columns.is_empty() {
+            return;
+        }
+
         let mut live_columns = self.live_columns.as_ref().clone();
         for (c, _) in constant_columns.iter() {
             live_columns.swap_remove(c);
         }
         self.live_columns = Arc::new(live_columns);
 
+        let mut predicate_on_all_null_inserted_column = false;
+
         if let Some(skip_batch_predicate) = self.skip_batch_predicate.take() {
             let mut sbp_constant_columns = Vec::with_capacity(constant_columns.len() * 3);
             for (c, v) in constant_columns.iter() {
                 sbp_constant_columns.push((format_pl_smallstr!("{c}_min"), v.clone()));
                 sbp_constant_columns.push((format_pl_smallstr!("{c}_max"), v.clone()));
                 let nc = if v.is_null() {
+                    if self.column_predicates.predicates.contains_key(c) {
+                        predicate_on_all_null_inserted_column = true;
+                    }
+
                     AnyValue::Null
                 } else {
                     (0 as IdxSize).into()
@@ -502,6 +518,13 @@ impl ScanIOPredicate {
         }
         self.column_predicates = Arc::new(column_predicates);
 
+        if predicate_on_all_null_inserted_column {
+            // TODO:
+            // We currently switch this to false because don't skip the batch properly on all-NULL inserted columns.
+            // It can be removed once the batch is being skipped properly.
+            Arc::make_mut(&mut self.column_predicates).is_sumwise_complete = false;
+        }
+
         self.predicate = Arc::new(PhysicalExprWithConstCols {
             constants: constant_columns,
             child: self.predicate.clone(),
@@ -514,68 +537,3 @@ impl fmt::Debug for ScanIOPredicate {
         f.write_str("scan_io_predicate")
     }
 }
-
-/// A collection of column stats with a known schema.
-#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
-#[derive(Debug, Clone)]
-pub struct BatchStats {
-    schema: SchemaRef,
-    stats: Vec<ColumnStats>,
-    // This might not be available, as when pruning hive partitions.
-    num_rows: Option<usize>,
-}
-
-impl Default for BatchStats {
-    fn default() -> Self {
-        Self {
-            schema: Arc::new(Schema::default()),
-            stats: Vec::new(),
-            num_rows: None,
-        }
-    }
-}
-
-impl BatchStats {
-    /// Constructs a new [`BatchStats`].
-    ///
-    /// The `stats` should match the order of the `schema`.
-    pub fn new(schema: SchemaRef, stats: Vec<ColumnStats>, num_rows: Option<usize>) -> Self {
-        Self {
-            schema,
-            stats,
-            num_rows,
-        }
-    }
-
-    /// Returns the [`Schema`] of the batch.
-    pub fn schema(&self) -> &SchemaRef {
-        &self.schema
-    }
-
-    /// Returns the [`ColumnStats`] of all columns in the batch, if known.
-    pub fn column_stats(&self) -> &[ColumnStats] {
-        self.stats.as_ref()
-    }
-
-    /// Returns the [`ColumnStats`] of a single column in the batch.
-    ///
-    /// Returns an `Err` if no statistics are available for the given column.
-    pub fn get_stats(&self, column: &str) -> PolarsResult<&ColumnStats> {
-        self.schema.try_index_of(column).map(|i| &self.stats[i])
-    }
-
-    /// Returns the number of rows in the batch.
-    ///
-    /// Returns `None` if the number of rows is unknown.
-    pub fn num_rows(&self) -> Option<usize> {
-        self.num_rows
-    }
-
-    pub fn with_schema(&mut self, schema: SchemaRef) {
-        self.schema = schema;
-    }
-
-    pub fn take_indices(&mut self, indices: &[usize]) {
-        self.stats = indices.iter().map(|&i| self.stats[i].clone()).collect();
-    }
-}
