@@ -1,9 +1,8 @@
-use std::path::{Path, PathBuf};
-
 use polars_core::prelude::*;
 use polars_io::cloud::CloudOptions;
 use polars_io::ipc::IpcScanOptions;
 use polars_io::{HiveOptions, RowIndex};
+use polars_utils::plpath::PlPath;
 use polars_utils::slice_enum::Slice;
 
 use crate::prelude::*;
@@ -75,9 +74,12 @@ impl LazyFileListReader for LazyIpcReader {
                 projection: None,
                 row_index,
                 pre_slice,
-                cast_columns_policy: CastColumnsPolicy::ErrorOnMismatch,
+                cast_columns_policy: CastColumnsPolicy::ERROR_ON_MISMATCH,
                 missing_columns_policy: MissingColumnsPolicy::Raise,
+                extra_columns_policy: ExtraColumnsPolicy::Raise,
                 include_file_paths,
+                column_mapping: None,
+                deletion_files: None,
             },
         )?
         .build()
@@ -134,14 +136,11 @@ impl LazyFileListReader for LazyIpcReader {
 
 impl LazyFrame {
     /// Create a LazyFrame directly from a ipc scan.
-    pub fn scan_ipc(path: impl AsRef<Path>, args: ScanArgsIpc) -> PolarsResult<Self> {
-        Self::scan_ipc_sources(
-            ScanSources::Paths([path.as_ref().to_path_buf()].into()),
-            args,
-        )
+    pub fn scan_ipc(path: PlPath, args: ScanArgsIpc) -> PolarsResult<Self> {
+        Self::scan_ipc_sources(ScanSources::Paths([path].into()), args)
     }
 
-    pub fn scan_ipc_files(paths: Arc<[PathBuf]>, args: ScanArgsIpc) -> PolarsResult<Self> {
+    pub fn scan_ipc_files(paths: Arc<[PlPath]>, args: ScanArgsIpc) -> PolarsResult<Self> {
         Self::scan_ipc_sources(ScanSources::Paths(paths), args)
     }
 
