@@ -9,7 +9,7 @@ fn test_agg_list_type() -> PolarsResult<()> {
     let s = Series::new("foo".into(), &[1, 2, 3]);
     let s = s.cast(&DataType::Datetime(TimeUnit::Nanoseconds, None))?;
 
-    let l = unsafe { s.agg_list(&GroupsProxy::Idx(vec![(0, unitvec![0, 1, 2])].into())) };
+    let l = unsafe { s.agg_list(&GroupsType::Idx(vec![(0, unitvec![0, 1, 2])].into())) };
 
     let result = match l.dtype() {
         DataType::List(inner) => {
@@ -249,7 +249,7 @@ fn test_binary_agg_context_0() -> PolarsResult<()> {
         .unwrap();
 
     let out = out.column("foo")?;
-    let out = out.explode()?;
+    let out = out.explode(false)?;
     let out = out.str()?;
     assert_eq!(
         Vec::from(out),
@@ -293,7 +293,7 @@ fn test_binary_agg_context_1() -> PolarsResult<()> {
     // [90, 90]
     // [7, 90]
     let out = out.column("vals")?;
-    let out = out.explode()?;
+    let out = out.explode(false)?;
     let out = out.i32()?;
     assert_eq!(
         Vec::from(out),
@@ -314,7 +314,7 @@ fn test_binary_agg_context_1() -> PolarsResult<()> {
     // [90, 90]
     // [90, 7]
     let out = out.column("vals")?;
-    let out = out.explode()?;
+    let out = out.explode(false)?;
     let out = out.i32()?;
     assert_eq!(
         Vec::from(out),
@@ -344,7 +344,7 @@ fn test_binary_agg_context_2() -> PolarsResult<()> {
     // 3 - [3, 4] = [0, -1]
     // 5 - [5, 6] = [0, -1]
     let out = out.column("vals")?;
-    let out = out.explode()?;
+    let out = out.explode(false)?;
     let out = out.i32()?;
     assert_eq!(
         Vec::from(out),
@@ -362,7 +362,7 @@ fn test_binary_agg_context_2() -> PolarsResult<()> {
     // [3, 4] - 3 = [0, 1]
     // [5, 6] - 5 = [0, 1]
     let out = out.column("vals")?;
-    let out = out.explode()?;
+    let out = out.explode(false)?;
     let out = out.i32()?;
     assert_eq!(
         Vec::from(out),
@@ -443,22 +443,13 @@ fn take_aggregations() -> PolarsResult<()> {
         .agg([
             // keep the head as it test slice correctness
             col("book")
-                .gather(
-                    col("count")
-                        .arg_sort(SortOptions {
-                            descending: true,
-                            nulls_last: false,
-                            multithreaded: true,
-                            maintain_order: false,
-                        })
-                        .head(Some(2)),
-                )
+                .gather(col("count").arg_sort(true, false).head(Some(2)))
                 .alias("ordered"),
         ])
         .sort(["user"], Default::default())
         .collect()?;
     let s = out.column("ordered")?;
-    let flat = s.explode()?;
+    let flat = s.explode(false)?;
     let flat = flat.str()?;
     let vals = flat.into_no_null_iter().collect::<Vec<_>>();
     assert_eq!(vals, ["a", "b", "c", "a", "a"]);
@@ -483,14 +474,7 @@ fn test_take_consistency() -> PolarsResult<()> {
     let out = df
         .clone()
         .lazy()
-        .select([col("A")
-            .arg_sort(SortOptions {
-                descending: true,
-                nulls_last: false,
-                multithreaded: true,
-                maintain_order: false,
-            })
-            .get(lit(0))])
+        .select([col("A").arg_sort(true, false).get(lit(0))])
         .collect()?;
 
     let a = out.column("A")?;
@@ -501,14 +485,7 @@ fn test_take_consistency() -> PolarsResult<()> {
         .clone()
         .lazy()
         .group_by_stable([col("cars")])
-        .agg([col("A")
-            .arg_sort(SortOptions {
-                descending: true,
-                nulls_last: false,
-                multithreaded: true,
-                maintain_order: false,
-            })
-            .get(lit(0))])
+        .agg([col("A").arg_sort(true, false).get(lit(0))])
         .collect()?;
 
     let out = out.column("A")?;
@@ -520,26 +497,9 @@ fn test_take_consistency() -> PolarsResult<()> {
         .group_by_stable([col("cars")])
         .agg([
             col("A"),
+            col("A").arg_sort(true, false).get(lit(0)).alias("1"),
             col("A")
-                .arg_sort(SortOptions {
-                    descending: true,
-                    nulls_last: false,
-                    multithreaded: true,
-                    maintain_order: false,
-                })
-                .get(lit(0))
-                .alias("1"),
-            col("A")
-                .get(
-                    col("A")
-                        .arg_sort(SortOptions {
-                            descending: true,
-                            nulls_last: false,
-                            multithreaded: true,
-                            maintain_order: false,
-                        })
-                        .get(lit(0)),
-                )
+                .get(col("A").arg_sort(true, false).get(lit(0)))
                 .alias("2"),
         ])
         .collect()?;
@@ -571,3 +531,38 @@ fn test_take_in_groups() -> PolarsResult<()> {
     );
     Ok(())
 }
+
+#[test]
+fn test_anonymous_function_returns_scalar_all_null_20679() {
+    use std::sync::Arc;
+
+    fn reduction_function(column: Column) -> PolarsResult<Option<Column>> {
+        let val = column.get(0)?.into_static();
+        let col = Column::new_scalar("".into(), Scalar::new(column.dtype().clone(), val), 1);
+        Ok(Some(col))
+    }
+
+    let a = Column::new("a".into(), &[0, 0, 1]);
+    let dtype = DataType::Null;
+    let b = Column::new_scalar("b".into(), Scalar::new(dtype, AnyValue::Null), 3);
+    let df = DataFrame::new(vec![a, b]).unwrap();
+
+    let f = move |c: &mut [Column]| reduction_function(std::mem::take(&mut c[0]));
+
+    let expr = Expr::AnonymousFunction {
+        input: vec![col("b")],
+        function: LazySerde::Deserialized(SpecialEq::new(Arc::new(f))),
+        output_type: Default::default(),
+        options: FunctionOptions::aggregation(),
+        fmt_str: Box::new(PlSmallStr::EMPTY),
+    };
+
+    let grouped_df = df
+        .lazy()
+        .group_by([col("a")])
+        .agg([expr])
+        .collect()
+        .unwrap();
+
+    assert_eq!(grouped_df.get_columns()[1].dtype(), &DataType::Null);
+}
