@@ -51,9 +51,10 @@ fn test_lazy_unpivot() {
     let df = get_df();
 
     let args = UnpivotArgsDSL {
-        on: vec!["sepal_length".into(), "sepal_width".into()],
-        index: vec!["petal_width".into(), "petal_length".into()],
-        ..Default::default()
+        on: by_name(["sepal_length", "sepal_width"], true),
+        index: by_name(["petal_width", "petal_length"], true),
+        variable_name: None,
+        value_name: None,
     };
 
     let out = df
@@ -289,7 +290,7 @@ fn test_lazy_query_4() -> PolarsResult<()> {
                 )
                 .alias("diff_cases"),
         ])
-        .explode([col("day"), col("diff_cases")])
+        .explode(by_name(["day", "diff_cases"], true))
         .join(
             base_df,
             [col("uid"), col("day")],
@@ -585,10 +586,11 @@ fn test_simplify_expr() {
         &mut OptFlags::SIMPLIFY_EXPR,
     )
     .unwrap();
-    let plan = node_to_lp(lp_top, &expr_arena, &mut lp_arena);
-    assert!(
-        matches!(plan, DslPlan::Select{ expr, ..} if matches!(&expr[0], Expr::BinaryExpr{left, ..} if **left == Expr::Literal(LiteralValue::Dyn(DynLiteralValue::Float(2.0)))))
-    );
+
+    assert!(matches!(
+        lp_arena.get(lp_top),
+        IR::Select { expr, .. }  if matches!(expr_arena.get(expr[0].node()), AExpr::BinaryExpr{ left, ..} if matches!(expr_arena.get(*left), &AExpr::Literal(LiteralValue::Dyn(DynLiteralValue::Float(2.0)))))
+    ));
 }
 
 #[test]
@@ -665,13 +667,12 @@ fn test_type_coercion() {
     let mut expr_arena = Arena::new();
     let mut lp_arena = Arena::new();
     let lp_top = to_alp(lp, &mut expr_arena, &mut lp_arena, &mut OptFlags::default()).unwrap();
-    let lp = node_to_lp(lp_top, &expr_arena, &mut lp_arena);
 
-    if let DslPlan::Select { expr, .. } = lp {
-        if let Expr::BinaryExpr { left, right, .. } = &expr[0] {
-            assert!(matches!(&**left, Expr::Cast { .. }));
+    if let IR::Select { expr, .. } = lp_arena.get(lp_top) {
+        if let AExpr::BinaryExpr { left, right, .. } = expr_arena.get(expr[0].node()) {
+            assert!(matches!(expr_arena.get(*left), AExpr::Cast { .. }));
             // bar is already float, does not have to be coerced
-            assert!(matches!(&**right, Expr::Column { .. }));
+            assert!(matches!(expr_arena.get(*right), AExpr::Column { .. }));
         } else {
             panic!()
         }
@@ -1041,7 +1042,7 @@ fn test_group_by_cum_sum() -> PolarsResult<()> {
         .collect()?;
 
     assert_eq!(
-        Vec::from(out.column("vals")?.explode()?.i32()?),
+        Vec::from(out.column("vals")?.explode(false)?.i32()?),
         [1, 5, 11, 3, 12, 20]
             .iter()
             .copied()
@@ -1102,7 +1103,7 @@ fn test_multiple_explode() -> PolarsResult<()> {
         .lazy()
         .group_by([col("a")])
         .agg([col("b").alias("b_list"), col("c").alias("c_list")])
-        .explode([col("c_list"), col("b_list")])
+        .explode(by_name(["c_list", "b_list"], true))
         .collect()?;
     assert_eq!(out.shape(), (5, 3));
 
@@ -1125,8 +1126,6 @@ fn test_filter_and_alias() -> PolarsResult<()> {
         "a" => [2, 2],
         "a_squared" => [4.0, 4.0]
     ]?;
-    println!("{:?}", out);
-    println!("{:?}", expected);
     assert!(out.equals(&expected));
     Ok(())
 }
@@ -1175,7 +1174,7 @@ fn test_fill_forward() -> PolarsResult<()> {
         .lazy()
         .select([col("b")
             .fill_null_with_strategy(FillNullStrategy::Forward(FillNullLimit::None))
-            .over_with_options([col("a")], None, WindowMapping::Join)])
+            .over_with_options(Some([col("a")]), None, WindowMapping::Join)?])
         .collect()?;
     let agg = out.column("b")?.list()?;
 
@@ -1253,7 +1252,10 @@ fn test_exclude() -> PolarsResult<()> {
     "c" => [1, 2, 3]
     ]?;
 
-    let out = df.lazy().select([col("*").exclude(["b"])]).collect()?;
+    let out = df
+        .lazy()
+        .select([all().exclude_cols(["b"]).as_expr()])
+        .collect()?;
 
     assert_eq!(out.get_column_names(), &["a", "c"]);
     Ok(())
@@ -1302,7 +1304,7 @@ fn test_sort_by() -> PolarsResult<()> {
         .group_by_stable([col("b")])
         .agg([col("a").sort_by([col("b"), col("c")], SortMultipleOptions::default())])
         .collect()?;
-    let a = out.column("a")?.explode()?;
+    let a = out.column("a")?.explode(false)?;
     assert_eq!(
         Vec::from(a.i32().unwrap()),
         &[Some(3), Some(1), Some(2), Some(5), Some(4)]
@@ -1315,7 +1317,7 @@ fn test_sort_by() -> PolarsResult<()> {
         .agg([col("a").sort_by([col("b"), col("c")], SortMultipleOptions::default())])
         .collect()?;
 
-    let a = out.column("a")?.explode()?;
+    let a = out.column("a")?.explode(false)?;
     assert_eq!(
         Vec::from(a.i32().unwrap()),
         &[Some(3), Some(1), Some(2), Some(5), Some(4)]
@@ -1335,7 +1337,7 @@ fn test_filter_after_shift_in_groups() -> PolarsResult<()> {
             col("B")
                 .shift(lit(1))
                 .filter(col("B").shift(lit(1)).gt(lit(4)))
-                .over_with_options([col("fruits")], None, WindowMapping::Join)
+                .over_with_options(Some([col("fruits")]), None, WindowMapping::Join)?
                 .alias("filtered"),
         ])
         .collect()?;
@@ -1403,8 +1405,8 @@ fn test_categorical_addition() -> PolarsResult<()> {
     let out = df
         .lazy()
         .select([
-            col("fruits").cast(DataType::Categorical(None, Default::default())),
-            col("cars").cast(DataType::Categorical(None, Default::default())),
+            col("fruits").cast(DataType::from_categories(Categories::global())),
+            col("cars").cast(DataType::from_categories(Categories::global())),
         ])
         .select([(col("fruits") + lit(" ") + col("cars")).alias("foo")])
         .collect()?;
@@ -1580,7 +1582,7 @@ fn test_exclude_regex() -> PolarsResult<()> {
     let df = fruits_cars();
     let out = df
         .lazy()
-        .select([col("*").exclude(["^(fruits|cars)$"])])
+        .select([(all() - Selector::Matches("^(fruits|cars)$".into())).as_expr()])
         .collect()?;
 
     assert_eq!(out.get_column_names(), &["A", "B"]);
@@ -1621,7 +1623,9 @@ pub fn test_select_by_dtypes() -> PolarsResult<()> {
     ]?;
     let out = df
         .lazy()
-        .select([dtype_cols([DataType::Float32, DataType::String])])
+        .select([dtype_cols([DataType::Float32, DataType::String])
+            .as_selector()
+            .as_expr()])
         .collect()?;
     assert_eq!(out.dtypes(), &[DataType::String, DataType::Float32]);
 
@@ -1660,15 +1664,7 @@ fn test_single_group_result() -> PolarsResult<()> {
 
     let out = df
         .lazy()
-        .select([col("a")
-            .arg_sort(SortOptions {
-                descending: false,
-                nulls_last: false,
-                multithreaded: true,
-                maintain_order: false,
-                limit: None,
-            })
-            .over([col("a")])])
+        .select([col("a").arg_sort(false, false).over([col("a")])])
         .collect()?;
 
     let a = out.column("a")?.idx()?;
@@ -1695,10 +1691,10 @@ fn test_single_ranked_group() -> PolarsResult<()> {
                 },
                 None,
             )
-            .over_with_options([col("group")], None, WindowMapping::Join)])
+            .over_with_options(Some([col("group")]), None, WindowMapping::Join)?])
         .collect()?;
 
-    let out = out.column("value")?.explode()?;
+    let out = out.column("value")?.explode(false)?;
     let out = out.f64()?;
     assert_eq!(
         Vec::from(out),
@@ -1767,7 +1763,7 @@ fn test_is_in() -> PolarsResult<()> {
         )])
         .collect()?;
     let out = out.column("cars").unwrap();
-    let out = out.explode()?;
+    let out = out.explode(false)?;
     let out = out.bool().unwrap();
     assert_eq!(
         Vec::from(out),
@@ -1785,7 +1781,7 @@ fn test_is_in() -> PolarsResult<()> {
         .collect()?;
 
     let out = out.column("cars").unwrap();
-    let out = out.explode()?;
+    let out = out.explode(false)?;
     let out = out.bool().unwrap();
     assert_eq!(
         Vec::from(out),
@@ -1947,7 +1943,6 @@ fn test_sort_maintain_order_true() -> PolarsResult<()> {
         )
         .slice(0, 3)
         .collect()?;
-    println!("{:?}", res);
     assert!(res.equals(&df![
         "A" => [1, 1, 1],
         "B" => ["A", "B", "C"],
@@ -1964,7 +1959,11 @@ fn test_over_with_options_empty_join() -> PolarsResult<()> {
 
     let empty_df_out = empty_df
         .lazy()
-        .select([col("b").over_with_options([col("a")], Option::None, WindowMapping::Join)])
+        .select([col("b").over_with_options(
+            Some([col("a")]),
+            Option::None,
+            WindowMapping::Join,
+        )?])
         .collect()?;
 
     let f1: Field = Field::new("b".into(), DataType::List(Box::new(DataType::Int32)));
