@@ -65,6 +65,17 @@ pub trait SeriesJoin: SeriesSealed + Sized {
                     build_null_count,
                 )
             },
+            T::List(_) => {
+                let lhs = &encode_rows_unordered(&[lhs.into_owned().into()])?.into_series();
+                let rhs = &encode_rows_unordered(&[rhs.into_owned().into()])?.into_series();
+                lhs.hash_join_left(rhs, validate, nulls_equal)
+            },
+            #[cfg(feature = "dtype-array")]
+            T::Array(_, _) => {
+                let lhs = &encode_rows_unordered(&[lhs.into_owned().into()])?.into_series();
+                let rhs = &encode_rows_unordered(&[rhs.into_owned().into()])?.into_series();
+                lhs.hash_join_left(rhs, validate, nulls_equal)
+            },
             #[cfg(feature = "dtype-struct")]
             T::Struct(_) => {
                 let lhs = &encode_rows_unordered(&[lhs.into_owned().into()])?.into_series();
@@ -88,18 +99,23 @@ pub trait SeriesJoin: SeriesSealed + Sized {
 
                 use BitRepr as B;
                 match (lhs, rhs) {
-                    (B::Small(lhs), B::Small(rhs)) => {
+                    (B::U32(lhs), B::U32(rhs)) => {
                         // Turbofish: see #17137.
                         num_group_join_left::<UInt32Type>(&lhs, &rhs, validate, nulls_equal)
                     },
-                    (B::Large(lhs), B::Large(rhs)) => {
+                    (B::U64(lhs), B::U64(rhs)) => {
                         // Turbofish: see #17137.
                         num_group_join_left::<UInt64Type>(&lhs, &rhs, validate, nulls_equal)
                     },
+                    #[cfg(feature = "dtype-i128")]
+                    (B::I128(lhs), B::I128(rhs)) => {
+                        // Turbofish: see #17137.
+                        num_group_join_left::<Int128Type>(&lhs, &rhs, validate, nulls_equal)
+                    },
                     _ => {
                         polars_bail!(
-                        nyi = "Mismatch bit repr Hash Left Join between {lhs_dtype} and {rhs_dtype}",
-                    );
+                            nyi = "Mismatch bit repr Hash Left Join between {lhs_dtype} and {rhs_dtype}",
+                        );
                     },
                 }
             },
@@ -149,6 +165,17 @@ pub trait SeriesJoin: SeriesSealed + Sized {
                     hash_join_tuples_left_semi(lhs, rhs, nulls_equal)
                 }
             },
+            T::List(_) => {
+                let lhs = &encode_rows_unordered(&[lhs.into_owned().into()])?.into_series();
+                let rhs = &encode_rows_unordered(&[rhs.into_owned().into()])?.into_series();
+                lhs.hash_join_semi_anti(rhs, anti, nulls_equal)?
+            },
+            #[cfg(feature = "dtype-array")]
+            T::Array(_, _) => {
+                let lhs = &encode_rows_unordered(&[lhs.into_owned().into()])?.into_series();
+                let rhs = &encode_rows_unordered(&[rhs.into_owned().into()])?.into_series();
+                lhs.hash_join_semi_anti(rhs, anti, nulls_equal)?
+            },
             #[cfg(feature = "dtype-struct")]
             T::Struct(_) => {
                 let lhs = &encode_rows_unordered(&[lhs.into_owned().into()])?.into_series();
@@ -172,14 +199,19 @@ pub trait SeriesJoin: SeriesSealed + Sized {
 
                 use BitRepr as B;
                 match (lhs, rhs) {
-                    (B::Small(lhs), B::Small(rhs)) => {
+                    (B::U32(lhs), B::U32(rhs)) => {
                         // Turbofish: see #17137.
                         num_group_join_anti_semi::<UInt32Type>(&lhs, &rhs, anti, nulls_equal)
                     },
-                    (B::Large(lhs), B::Large(rhs)) => {
+                    (B::U64(lhs), B::U64(rhs)) => {
                         // Turbofish: see #17137.
                         num_group_join_anti_semi::<UInt64Type>(&lhs, &rhs, anti, nulls_equal)
                     },
+                    #[cfg(feature = "dtype-i128")]
+                    (B::I128(lhs), B::I128(rhs)) => {
+                        // Turbofish: see #17137.
+                        num_group_join_anti_semi::<Int128Type>(&lhs, &rhs, anti, nulls_equal)
+                    },
                     _ => {
                         polars_bail!(
                             nyi = "Mismatch bit repr Hash Semi-Anti Join between {lhs_dtype} and {rhs_dtype}",
@@ -256,6 +288,17 @@ pub trait SeriesJoin: SeriesSealed + Sized {
                     !swapped,
                 ))
             },
+            T::List(_) => {
+                let lhs = &encode_rows_unordered(&[lhs.into_owned().into()])?.into_series();
+                let rhs = &encode_rows_unordered(&[rhs.into_owned().into()])?.into_series();
+                lhs.hash_join_inner(rhs, validate, nulls_equal)
+            },
+            #[cfg(feature = "dtype-array")]
+            T::Array(_, _) => {
+                let lhs = &encode_rows_unordered(&[lhs.into_owned().into()])?.into_series();
+                let rhs = &encode_rows_unordered(&[rhs.into_owned().into()])?.into_series();
+                lhs.hash_join_inner(rhs, validate, nulls_equal)
+            },
             #[cfg(feature = "dtype-struct")]
             T::Struct(_) => {
                 let lhs = &encode_rows_unordered(&[lhs.into_owned().into()])?.into_series();
@@ -279,14 +322,19 @@ pub trait SeriesJoin: SeriesSealed + Sized {
 
                 use BitRepr as B;
                 match (lhs, rhs) {
-                    (B::Small(lhs), B::Small(rhs)) => {
+                    (B::U32(lhs), B::U32(rhs)) => {
                         // Turbofish: see #17137.
                         group_join_inner::<UInt32Type>(&lhs, &rhs, validate, nulls_equal)
                     },
-                    (B::Large(lhs), BitRepr::Large(rhs)) => {
+                    (B::U64(lhs), BitRepr::U64(rhs)) => {
                         // Turbofish: see #17137.
                         group_join_inner::<UInt64Type>(&lhs, &rhs, validate, nulls_equal)
                     },
+                    #[cfg(feature = "dtype-i128")]
+                    (B::I128(lhs), BitRepr::I128(rhs)) => {
+                        // Turbofish: see #17137.
+                        group_join_inner::<Int128Type>(&lhs, &rhs, validate, nulls_equal)
+                    },
                     _ => {
                         polars_bail!(
                             nyi = "Mismatch bit repr Hash Inner Join between {lhs_dtype} and {rhs_dtype}"
@@ -332,6 +380,17 @@ pub trait SeriesJoin: SeriesSealed + Sized {
                 let rhs = rhs.iter().map(|k| k.as_slice()).collect::<Vec<_>>();
                 hash_join_tuples_outer(lhs, rhs, swapped, validate, nulls_equal)
             },
+            T::List(_) => {
+                let lhs = &encode_rows_unordered(&[lhs.into_owned().into()])?.into_series();
+                let rhs = &encode_rows_unordered(&[rhs.into_owned().into()])?.into_series();
+                lhs.hash_join_outer(rhs, validate, nulls_equal)
+            },
+            #[cfg(feature = "dtype-array")]
+            T::Array(_, _) => {
+                let lhs = &encode_rows_unordered(&[lhs.into_owned().into()])?.into_series();
+                let rhs = &encode_rows_unordered(&[rhs.into_owned().into()])?.into_series();
+                lhs.hash_join_outer(rhs, validate, nulls_equal)
+            },
             #[cfg(feature = "dtype-struct")]
             T::Struct(_) => {
                 let lhs = &encode_rows_unordered(&[lhs.into_owned().into()])?.into_series();
@@ -352,16 +411,23 @@ pub trait SeriesJoin: SeriesSealed + Sized {
 
                 use BitRepr as B;
                 match (lhs, rhs) {
-                    (B::Small(lhs), B::Small(rhs)) => {
+                    (B::U32(lhs), B::U32(rhs)) => {
                         // Turbofish: see #17137.
                         hash_join_outer::<UInt32Type>(&lhs, &rhs, validate, nulls_equal)
                     },
-                    (B::Large(lhs), B::Large(rhs)) => {
+                    (B::U64(lhs), B::U64(rhs)) => {
                         // Turbofish: see #17137.
                         hash_join_outer::<UInt64Type>(&lhs, &rhs, validate, nulls_equal)
                     },
+                    #[cfg(feature = "dtype-i128")]
+                    (B::I128(lhs), B::I128(rhs)) => {
+                        // Turbofish: see #17137.
+                        hash_join_outer::<Int128Type>(&lhs, &rhs, validate, nulls_equal)
+                    },
                     _ => {
-                        polars_bail!(nyi = "Mismatch bit repr Hash Join Outer between {lhs_dtype} and {rhs_dtype}");
+                        polars_bail!(
+                            nyi = "Mismatch bit repr Hash Join Outer between {lhs_dtype} and {rhs_dtype}"
+                        );
                     },
                 }
             },
@@ -627,8 +693,8 @@ where
         (ca, other, false)
     };
     let hb = PlRandomState::default();
-    let bh_a = a.to_bytes_hashes(true, hb.clone());
-    let bh_b = b.to_bytes_hashes(true, hb.clone());
+    let bh_a = a.to_bytes_hashes(true, hb);
+    let bh_b = b.to_bytes_hashes(true, hb);
 
     (bh_a, bh_b, swapped, hb)
 }
