@@ -1,23 +1,28 @@
-use arrow::array::{DictionaryArray, MutableBinaryViewArray, PrimitiveArray};
+use std::marker::PhantomData;
+
+use arrow::array::{DictionaryArray, DictionaryKey, MutableBinaryViewArray, PrimitiveArray};
 use arrow::bitmap::{Bitmap, BitmapBuilder};
 use arrow::datatypes::ArrowDataType;
 use arrow::types::{AlignedBytes, NativeType};
 
-use super::binview::BinViewDecoder;
-use super::utils::{self, dict_indices_decoder, freeze_validity, Decoder, StateTranslation};
 use super::PredicateFilter;
-use crate::parquet::encoding::hybrid_rle::HybridRleDecoder;
+use super::binview::BinViewDecoder;
+use super::utils::{self, Decoder, StateTranslation, dict_indices_decoder, freeze_validity};
 use crate::parquet::encoding::Encoding;
+use crate::parquet::encoding::hybrid_rle::HybridRleDecoder;
 use crate::parquet::error::ParquetResult;
 use crate::parquet::page::{DataPage, DictPage};
+use crate::read::deserialize::dictionary_encoded::IndexMapping;
 
-impl<'a> StateTranslation<'a, CategoricalDecoder> for HybridRleDecoder<'a> {
+impl<'a, T: DictionaryKey + IndexMapping<Output = T::AlignedBytes>>
+    StateTranslation<'a, CategoricalDecoder<T>> for HybridRleDecoder<'a>
+{
     type PlainDecoder = HybridRleDecoder<'a>;
 
     fn new(
-        _decoder: &CategoricalDecoder,
+        _decoder: &CategoricalDecoder<T>,
         page: &'a DataPage,
-        _dict: Option<&'a <CategoricalDecoder as Decoder>::Dict>,
+        _dict: Option<&'a <CategoricalDecoder<T> as Decoder>::Dict>,
         page_validity: Option<&Bitmap>,
     ) -> ParquetResult<Self> {
         if !matches!(
@@ -39,29 +44,33 @@ impl<'a> StateTranslation<'a, CategoricalDecoder> for HybridRleDecoder<'a> {
 /// These are marked as special in the Arrow Field Metadata and they have the properly that for a
 /// given row group all the values are in the dictionary page and all data pages are dictionary
 /// encoded. This makes the job of decoding them extremely simple and fast.
-pub struct CategoricalDecoder {
+pub struct CategoricalDecoder<T> {
     dict_size: usize,
     decoder: BinViewDecoder,
+    key_type: PhantomData<T>,
 }
 
-impl CategoricalDecoder {
+impl<T> CategoricalDecoder<T> {
     pub fn new() -> Self {
         Self {
             dict_size: usize::MAX,
             decoder: BinViewDecoder::new_string(),
+            key_type: PhantomData,
         }
     }
 }
 
-impl utils::Decoder for CategoricalDecoder {
+impl<T: DictionaryKey + IndexMapping<Output = T::AlignedBytes>> utils::Decoder
+    for CategoricalDecoder<T>
+{
     type Translation<'a> = HybridRleDecoder<'a>;
     type Dict = <BinViewDecoder as utils::Decoder>::Dict;
-    type DecodedState = (Vec<u32>, BitmapBuilder);
-    type Output = DictionaryArray<u32>;
+    type DecodedState = (Vec<T>, BitmapBuilder);
+    type Output = DictionaryArray<T>;
 
     fn with_capacity(&self, capacity: usize) -> Self::DecodedState {
         (
-            Vec::<u32>::with_capacity(capacity),
+            Vec::<T>::with_capacity(capacity),
             BitmapBuilder::with_capacity(capacity),
         )
     }
@@ -88,7 +97,7 @@ impl utils::Decoder for CategoricalDecoder {
     ) -> ParquetResult<()> {
         let additional = additional
             .as_any()
-            .downcast_ref::<DictionaryArray<u32>>()
+            .downcast_ref::<DictionaryArray<T>>()
             .unwrap();
         decoded.0.extend(additional.keys().values().iter().copied());
         match additional.validity() {
@@ -105,10 +114,10 @@ impl utils::Decoder for CategoricalDecoder {
         dtype: ArrowDataType,
         dict: Option<Self::Dict>,
         (values, validity): Self::DecodedState,
-    ) -> ParquetResult<DictionaryArray<u32>> {
+    ) -> ParquetResult<DictionaryArray<T>> {
         let validity = freeze_validity(validity);
         let dict = dict.unwrap();
-        let keys = PrimitiveArray::new(ArrowDataType::UInt32, values.into(), validity);
+        let keys = PrimitiveArray::new(T::PRIMITIVE.into(), values.into(), validity);
 
         let mut view_dict = MutableBinaryViewArray::with_capacity(dict.len());
         let (views, buffers, _, _, _) = dict.into_inner();
@@ -136,13 +145,13 @@ impl utils::Decoder for CategoricalDecoder {
     ) -> ParquetResult<()> {
         super::dictionary_encoded::decode_dict_dispatch(
             state.translation,
-            self.dict_size,
+            T::try_from(self.dict_size).ok().unwrap(),
             state.dict_mask,
             state.is_optional,
             state.page_validity.as_ref(),
             filter,
             &mut decoded.1,
-            <<u32 as NativeType>::AlignedBytes as AlignedBytes>::cast_vec_ref_mut(&mut decoded.0),
+            <<T as NativeType>::AlignedBytes as AlignedBytes>::cast_vec_ref_mut(&mut decoded.0),
             pred_true_mask,
         )
     }
