@@ -17,6 +17,7 @@ mod range;
 mod repeat;
 mod selectors;
 mod syntactic_sugar;
+#[cfg(feature = "temporal")]
 mod temporal;
 
 pub use arity::*;
@@ -30,8 +31,6 @@ pub use correlation::*;
 pub use horizontal::*;
 #[cfg(any(feature = "range", feature = "arg_where"))]
 pub use index::*;
-#[cfg(feature = "dtype-struct")]
-use polars_core::utils::get_supertype;
 #[cfg(all(feature = "range", feature = "temporal"))]
 pub use range::date_range; // This shouldn't be necessary, but clippy complains about dead code
 #[cfg(all(feature = "range", feature = "dtype-time"))]
@@ -41,6 +40,7 @@ pub use range::*;
 pub use repeat::*;
 pub use selectors::*;
 pub use syntactic_sugar::*;
+#[cfg(feature = "temporal")]
 pub use temporal::*;
 
 #[cfg(feature = "arg_where")]
