@@ -1,33 +1,34 @@
 use std::io::Cursor;
-use std::sync::Arc;
+use std::sync::{Arc, OnceLock};
 
 use polars_core::datatypes::{DataType, Field};
 use polars_core::error::*;
 use polars_core::frame::DataFrame;
 use polars_core::frame::column::Column;
+use polars_core::prelude::UnknownKind;
 use polars_core::schema::Schema;
+use polars_utils::pl_str::PlSmallStr;
 use pyo3::prelude::*;
-use pyo3::pybacked::PyBackedBytes;
-use pyo3::types::PyBytes;
 
-use crate::constants::MAP_LIST_NAME;
 use crate::prelude::*;
 
 // Will be overwritten on Python Polars start up.
+#[allow(clippy::type_complexity)]
 pub static mut CALL_COLUMNS_UDF_PYTHON: Option<
-    fn(s: Column, lambda: &PyObject) -> PolarsResult<Column>,
+    fn(s: Column, output_dtype: Option<DataType>, lambda: &PyObject) -> PolarsResult<Column>,
 > = None;
 pub static mut CALL_DF_UDF_PYTHON: Option<
     fn(s: DataFrame, lambda: &PyObject) -> PolarsResult<DataFrame>,
 > = None;
 
-pub use polars_utils::python_function::{
-    PYTHON_SERDE_MAGIC_BYTE_MARK, PYTHON3_VERSION, PythonFunction,
-};
+pub use polars_utils::python_function::PythonFunction;
+#[cfg(feature = "serde")]
+pub use polars_utils::python_function::{PYTHON_SERDE_MAGIC_BYTE_MARK, PYTHON3_VERSION};
 
 pub struct PythonUdfExpression {
     python_function: PyObject,
-    output_type: Option<DataType>,
+    output_type: Option<DataTypeExpr>,
+    materialized_output_type: OnceLock<DataType>,
     is_elementwise: bool,
     returns_scalar: bool,
 }
@@ -35,13 +36,15 @@ pub struct PythonUdfExpression {
 impl PythonUdfExpression {
     pub fn new(
         lambda: PyObject,
-        output_type: Option<DataType>,
+        output_type: Option<impl Into<DataTypeExpr>>,
         is_elementwise: bool,
         returns_scalar: bool,
     ) -> Self {
+        let output_type = output_type.map(Into::into);
         Self {
             python_function: lambda,
             output_type,
+            materialized_output_type: OnceLock::new(),
             is_elementwise,
             returns_scalar,
         }
@@ -49,56 +52,30 @@ impl PythonUdfExpression {
 
     #[cfg(feature = "serde")]
     pub(crate) fn try_deserialize(buf: &[u8]) -> PolarsResult<Arc<dyn ColumnsUdf>> {
-        // Handle byte mark
-
         use polars_utils::pl_serialize;
-        debug_assert!(buf.starts_with(PYTHON_SERDE_MAGIC_BYTE_MARK));
-        let buf = &buf[PYTHON_SERDE_MAGIC_BYTE_MARK.len()..];
 
-        // Handle pickle metadata
-        let use_cloudpickle = buf[0];
-        if use_cloudpickle != 0 {
-            let ser_py_version = &buf[1..3];
-            let cur_py_version = *PYTHON3_VERSION;
-            polars_ensure!(
-                ser_py_version == cur_py_version,
-                InvalidOperation:
-                "current Python version {:?} does not match the Python version used to serialize the UDF {:?}",
-                (3, cur_py_version[0], cur_py_version[1]),
-                (3, ser_py_version[0], ser_py_version[1] )
-            );
+        if !buf.starts_with(PYTHON_SERDE_MAGIC_BYTE_MARK) {
+            polars_bail!(InvalidOperation: "serialization expected python magic byte mark");
         }
-        let buf = &buf[3..];
+        let buf = &buf[PYTHON_SERDE_MAGIC_BYTE_MARK.len()..];
 
         // Load UDF metadata
         let mut reader = Cursor::new(buf);
-        let (output_type, is_elementwise, returns_scalar): (Option<DataType>, bool, bool) =
+        let (output_type, is_elementwise, returns_scalar): (Option<DataTypeExpr>, bool, bool) =
             pl_serialize::deserialize_from_reader::<_, _, true>(&mut reader)?;
 
-        let remainder = &buf[reader.position() as usize..];
+        let buf = &buf[reader.position() as usize..];
+        let python_function = pl_serialize::python_object_deserialize(buf)?;
 
-        // Load UDF
-        Python::with_gil(|py| {
-            let pickle = PyModule::import(py, "pickle")
-                .expect("unable to import 'pickle'")
-                .getattr("loads")
-                .unwrap();
-            let arg = (PyBytes::new(py, remainder),);
-            let python_function = pickle.call1(arg).map_err(from_pyerr)?;
-            Ok(Arc::new(Self::new(
-                python_function.into(),
-                output_type,
-                is_elementwise,
-                returns_scalar,
-            )) as Arc<dyn ColumnsUdf>)
-        })
+        Ok(Arc::new(Self::new(
+            python_function,
+            output_type,
+            is_elementwise,
+            returns_scalar,
+        )))
     }
 }
 
-fn from_pyerr(e: PyErr) -> PolarsError {
-    PolarsError::ComputeError(format!("error raised in python: {e}").into())
-}
-
 impl DataFrameUdf for polars_utils::python_function::PythonFunction {
     fn call_udf(&self, df: DataFrame) -> PolarsResult<DataFrame> {
         let func = unsafe { CALL_DF_UDF_PYTHON.unwrap() };
@@ -107,14 +84,32 @@ impl DataFrameUdf for polars_utils::python_function::PythonFunction {
 }
 
 impl ColumnsUdf for PythonUdfExpression {
+    fn resolve_dsl(
+        &self,
+        input_schema: &Schema,
+        self_dtype: Option<&DataType>,
+    ) -> PolarsResult<()> {
+        if let Some(output_type) = self.output_type.as_ref() {
+            let dtype = output_type
+                .clone()
+                .into_datatype_with_opt_self(input_schema, self_dtype)?;
+            self.materialized_output_type.get_or_init(|| dtype);
+        }
+        Ok(())
+    }
+
     fn call_udf(&self, s: &mut [Column]) -> PolarsResult<Option<Column>> {
         let func = unsafe { CALL_COLUMNS_UDF_PYTHON.unwrap() };
 
         let output_type = self
-            .output_type
-            .clone()
-            .unwrap_or_else(|| DataType::Unknown(Default::default()));
-        let mut out = func(s[0].clone(), &self.python_function)?;
+            .materialized_output_type
+            .get()
+            .map_or_else(|| DataType::Unknown(Default::default()), |dt| dt.clone());
+        let mut out = func(
+            s[0].clone(),
+            self.materialized_output_type.get().cloned(),
+            &self.python_function,
+        )?;
         if !matches!(output_type, DataType::Unknown(_)) {
             let must_cast = out.dtype().matches_schema_type(&output_type).map_err(|_| {
                 polars_err!(
@@ -132,62 +127,38 @@ impl ColumnsUdf for PythonUdfExpression {
 
     #[cfg(feature = "serde")]
     fn try_serialize(&self, buf: &mut Vec<u8>) -> PolarsResult<()> {
-        // Write byte marks
-
         use polars_utils::pl_serialize;
+
+        // Write byte marks
         buf.extend_from_slice(PYTHON_SERDE_MAGIC_BYTE_MARK);
 
-        Python::with_gil(|py| {
-            // Try pickle to serialize the UDF, otherwise fall back to cloudpickle.
-            let pickle = PyModule::import(py, "pickle")
-                .expect("unable to import 'pickle'")
-                .getattr("dumps")
-                .unwrap();
-            let pickle_result = pickle.call1((self.python_function.clone_ref(py),));
-            let (dumped, use_cloudpickle) = match pickle_result {
-                Ok(dumped) => (dumped, false),
-                Err(_) => {
-                    let cloudpickle = PyModule::import(py, "cloudpickle")
-                        .map_err(from_pyerr)?
-                        .getattr("dumps")
-                        .unwrap();
-                    let dumped = cloudpickle
-                        .call1((self.python_function.clone_ref(py),))
-                        .map_err(from_pyerr)?;
-                    (dumped, true)
-                },
-            };
+        // Write UDF metadata
+        pl_serialize::serialize_into_writer::<_, _, true>(
+            &mut *buf,
+            &(
+                self.output_type.clone(),
+                self.is_elementwise,
+                self.returns_scalar,
+            ),
+        )?;
 
-            // Write pickle metadata
-            buf.push(use_cloudpickle as u8);
-            buf.extend_from_slice(&*PYTHON3_VERSION);
-
-            // Write UDF metadata
-            pl_serialize::serialize_into_writer::<_, _, true>(
-                &mut *buf,
-                &(
-                    self.output_type.clone(),
-                    self.is_elementwise,
-                    self.returns_scalar,
-                ),
-            )?;
-
-            // Write UDF
-            let dumped = dumped.extract::<PyBackedBytes>().unwrap();
-            buf.extend_from_slice(&dumped);
-            Ok(())
-        })
+        pl_serialize::python_object_serialize(&self.python_function, buf)?;
+        Ok(())
     }
 }
 
 /// Serializable version of [`GetOutput`] for Python UDFs.
 pub struct PythonGetOutput {
-    return_dtype: Option<DataType>,
+    return_dtype: Option<DataTypeExpr>,
+    materialized_output_type: OnceLock<DataType>,
 }
 
 impl PythonGetOutput {
-    pub fn new(return_dtype: Option<DataType>) -> Self {
-        Self { return_dtype }
+    pub fn new(return_dtype: Option<impl Into<DataTypeExpr>>) -> Self {
+        Self {
+            return_dtype: return_dtype.map(Into::into),
+            materialized_output_type: OnceLock::new(),
+        }
     }
 
     #[cfg(feature = "serde")]
@@ -199,7 +170,7 @@ impl PythonGetOutput {
         let buf = &buf[PYTHON_SERDE_MAGIC_BYTE_MARK.len()..];
 
         let mut reader = Cursor::new(buf);
-        let return_dtype: Option<DataType> =
+        let return_dtype: Option<DataTypeExpr> =
             pl_serialize::deserialize_from_reader::<_, _, true>(&mut reader)?;
 
         Ok(Arc::new(Self::new(return_dtype)) as Arc<dyn FunctionOutputField>)
@@ -209,15 +180,26 @@ impl PythonGetOutput {
 impl FunctionOutputField for PythonGetOutput {
     fn get_field(
         &self,
-        _input_schema: &Schema,
+        input_schema: &Schema,
         _cntxt: Context,
         fields: &[Field],
     ) -> PolarsResult<Field> {
         // Take the name of first field, just like [`GetOutput::map_field`].
         let name = fields[0].name();
-        let return_dtype = match self.return_dtype {
-            Some(ref dtype) => dtype.clone(),
-            None => DataType::Unknown(Default::default()),
+        let return_dtype = match self.materialized_output_type.get() {
+            Some(dtype) => dtype.clone(),
+            None => {
+                let dtype = if let Some(output_type) = self.return_dtype.as_ref() {
+                    output_type
+                        .clone()
+                        .into_datatype_with_self(input_schema, fields[0].dtype())?
+                } else {
+                    DataType::Unknown(UnknownKind::Any)
+                };
+
+                self.materialized_output_type.get_or_init(|| dtype.clone());
+                dtype
+            },
         };
         Ok(Field::new(name.clone(), return_dtype))
     }
@@ -232,22 +214,21 @@ impl FunctionOutputField for PythonGetOutput {
 }
 
 impl Expr {
-    pub fn map_python(self, func: PythonUdfExpression, agg_list: bool) -> Expr {
-        let (collect_groups, name) = if agg_list {
-            (ApplyOptions::ApplyList, MAP_LIST_NAME)
-        } else if func.is_elementwise {
-            (ApplyOptions::ElementWise, "python_udf")
-        } else {
-            (ApplyOptions::GroupWise, "python_udf")
-        };
+    pub fn map_python(self, func: PythonUdfExpression) -> Expr {
+        const NAME: &str = "python_udf";
 
         let returns_scalar = func.returns_scalar;
         let return_dtype = func.output_type.clone();
 
         let output_field = PythonGetOutput::new(return_dtype);
-        let output_type = SpecialEq::new(Arc::new(output_field) as Arc<dyn FunctionOutputField>);
+        let output_type = LazySerde::Deserialized(SpecialEq::new(
+            Arc::new(output_field) as Arc<dyn FunctionOutputField>
+        ));
 
         let mut flags = FunctionFlags::default() | FunctionFlags::OPTIONAL_RE_ENTRANT;
+        if func.is_elementwise {
+            flags.set_elementwise();
+        }
         if returns_scalar {
             flags |= FunctionFlags::RETURNS_SCALAR;
         }
@@ -257,11 +238,10 @@ impl Expr {
             function: new_column_udf(func),
             output_type,
             options: FunctionOptions {
-                collect_groups,
-                fmt_str: name,
                 flags,
                 ..Default::default()
             },
+            fmt_str: Box::new(PlSmallStr::from(NAME)),
         }
     }
 }
