@@ -1,40 +1,37 @@
 use super::*;
 
 impl AExpr {
-    /// Push nodes at this level to a pre-allocated stack.
-    pub(crate) fn nodes<C: PushNode>(&self, container: &mut C) {
+    /// Push the inputs of this node to the given container, in reverse order.
+    /// This ensures the primary node responsible for the name is pushed last.
+    ///
+    /// This is subtlely different from `children_rev` as this only includes the input expressions,
+    /// not expressions used during evaluation.
+    pub fn inputs_rev<E>(&self, container: &mut E)
+    where
+        E: Extend<Node>,
+    {
         use AExpr::*;
 
         match self {
             Column(_) | Literal(_) | Len => {},
-            Alias(e, _) => container.push_node(*e),
             BinaryExpr { left, op: _, right } => {
-                // reverse order so that left is popped first
-                container.push_node(*right);
-                container.push_node(*left);
+                container.extend([*right, *left]);
             },
-            Cast { expr, .. } => container.push_node(*expr),
-            Sort { expr, .. } => container.push_node(*expr),
+            Cast { expr, .. } => container.extend([*expr]),
+            Sort { expr, .. } => container.extend([*expr]),
             Gather { expr, idx, .. } => {
-                container.push_node(*idx);
-                // latest, so that it is popped first
-                container.push_node(*expr);
+                container.extend([*idx, *expr]);
             },
             SortBy { expr, by, .. } => {
-                for node in by {
-                    container.push_node(*node)
-                }
-                // latest, so that it is popped first
-                container.push_node(*expr);
+                container.extend(by.iter().cloned().rev());
+                container.extend([*expr]);
             },
             Filter { input, by } => {
-                container.push_node(*by);
-                // latest, so that it is popped first
-                container.push_node(*input);
+                container.extend([*by, *input]);
             },
             Agg(agg_e) => match agg_e.get_input() {
-                NodeInputs::Single(node) => container.push_node(node),
-                NodeInputs::Many(nodes) => container.extend_from_slice(&nodes),
+                NodeInputs::Single(node) => container.extend([node]),
+                NodeInputs::Many(nodes) => container.extend(nodes.into_iter().rev()),
                 NodeInputs::Leaf => {},
             },
             Ternary {
@@ -42,21 +39,12 @@ impl AExpr {
                 falsy,
                 predicate,
             } => {
-                container.push_node(*predicate);
-                container.push_node(*falsy);
-                // latest, so that it is popped first
-                container.push_node(*truthy);
+                container.extend([*predicate, *falsy, *truthy]);
             },
-            AnonymousFunction { input, .. } | Function { input, .. } =>
-            // we iterate in reverse order, so that the lhs is popped first and will be found
-            // as the root columns/ input columns by `_suffix` and `_keep_name` etc.
-            {
-                input
-                    .iter()
-                    .rev()
-                    .for_each(|e| container.push_node(e.node()))
+            AnonymousFunction { input, .. } | Function { input, .. } => {
+                container.extend(input.iter().rev().map(|e| e.node()))
             },
-            Explode(e) => container.push_node(*e),
+            Explode { expr: e, .. } => container.extend([*e]),
             Window {
                 function,
                 partition_by,
@@ -64,23 +52,94 @@ impl AExpr {
                 options: _,
             } => {
                 if let Some((n, _)) = order_by {
-                    container.push_node(*n);
+                    container.extend([*n]);
                 }
-                for e in partition_by.iter().rev() {
-                    container.push_node(*e);
-                }
-                // latest so that it is popped first
-                container.push_node(*function);
+                container.extend(partition_by.iter().rev().cloned());
+                container.extend([*function]);
+            },
+            Eval {
+                expr,
+                evaluation,
+                variant: _,
+            } => {
+                // We don't use the evaluation here because it does not contain inputs.
+                _ = evaluation;
+                container.extend([*expr]);
             },
             Slice {
                 input,
                 offset,
                 length,
             } => {
-                container.push_node(*length);
-                container.push_node(*offset);
-                // latest so that it is popped first
-                container.push_node(*input);
+                container.extend([*length, *offset, *input]);
+            },
+        }
+    }
+
+    /// Push the children of this node to the given container, in reverse order.
+    /// This ensures the primary node responsible for the name is pushed last.
+    ///
+    /// This is subtlely different from `input_rev` as this only all expressions included in the
+    /// expression not only the input expressions,
+    pub fn children_rev<E: Extend<Node>>(&self, container: &mut E) {
+        use AExpr::*;
+
+        match self {
+            Column(_) | Literal(_) | Len => {},
+            BinaryExpr { left, op: _, right } => {
+                container.extend([*right, *left]);
+            },
+            Cast { expr, .. } => container.extend([*expr]),
+            Sort { expr, .. } => container.extend([*expr]),
+            Gather { expr, idx, .. } => {
+                container.extend([*idx, *expr]);
+            },
+            SortBy { expr, by, .. } => {
+                container.extend(by.iter().cloned().rev());
+                container.extend([*expr]);
+            },
+            Filter { input, by } => {
+                container.extend([*by, *input]);
+            },
+            Agg(agg_e) => match agg_e.get_input() {
+                NodeInputs::Single(node) => container.extend([node]),
+                NodeInputs::Many(nodes) => container.extend(nodes.into_iter().rev()),
+                NodeInputs::Leaf => {},
+            },
+            Ternary {
+                truthy,
+                falsy,
+                predicate,
+            } => {
+                container.extend([*predicate, *falsy, *truthy]);
+            },
+            AnonymousFunction { input, .. } | Function { input, .. } => {
+                container.extend(input.iter().rev().map(|e| e.node()))
+            },
+            Explode { expr: e, .. } => container.extend([*e]),
+            Window {
+                function,
+                partition_by,
+                order_by,
+                options: _,
+            } => {
+                if let Some((n, _)) = order_by {
+                    container.extend([*n]);
+                }
+                container.extend(partition_by.iter().rev().cloned());
+                container.extend([*function]);
+            },
+            Eval {
+                expr,
+                evaluation,
+                variant: _,
+            } => container.extend([*evaluation, *expr]),
+            Slice {
+                input,
+                offset,
+                length,
+            } => {
+                container.extend([*length, *offset, *input]);
             },
         }
     }
@@ -89,29 +148,28 @@ impl AExpr {
         use AExpr::*;
         let input = match &mut self {
             Column(_) | Literal(_) | Len => return self,
-            Alias(input, _) => input,
             Cast { expr, .. } => expr,
-            Explode(input) => input,
+            Explode { expr, .. } => expr,
             BinaryExpr { left, right, .. } => {
-                *right = inputs[0];
-                *left = inputs[1];
+                *left = inputs[0];
+                *right = inputs[1];
                 return self;
             },
             Gather { expr, idx, .. } => {
-                *idx = inputs[0];
-                *expr = inputs[1];
+                *expr = inputs[0];
+                *idx = inputs[1];
                 return self;
             },
             Sort { expr, .. } => expr,
             SortBy { expr, by, .. } => {
-                *expr = *inputs.last().unwrap();
+                *expr = inputs[0];
                 by.clear();
-                by.extend_from_slice(&inputs[..inputs.len() - 1]);
+                by.extend_from_slice(&inputs[1..]);
                 return self;
             },
             Filter { input, by, .. } => {
-                *by = inputs[0];
-                *input = inputs[1];
+                *input = inputs[0];
+                *by = inputs[1];
                 return self;
             },
             Agg(a) => {
@@ -131,28 +189,35 @@ impl AExpr {
                 falsy,
                 predicate,
             } => {
-                *predicate = inputs[0];
+                *truthy = inputs[0];
                 *falsy = inputs[1];
-                *truthy = inputs[2];
+                *predicate = inputs[2];
                 return self;
             },
             AnonymousFunction { input, .. } | Function { input, .. } => {
-                debug_assert_eq!(input.len(), inputs.len());
-
-                // Assign in reverse order as that was the order in which nodes were extracted.
-                for (e, node) in input.iter_mut().zip(inputs.iter().rev()) {
+                assert_eq!(input.len(), inputs.len());
+                for (e, node) in input.iter_mut().zip(inputs.iter()) {
                     e.set_node(*node);
                 }
                 return self;
             },
+            Eval {
+                expr,
+                evaluation,
+                variant: _,
+            } => {
+                *expr = inputs[0];
+                _ = evaluation; // Intentional.
+                return self;
+            },
             Slice {
                 input,
                 offset,
                 length,
             } => {
-                *length = inputs[0];
+                *input = inputs[0];
                 *offset = inputs[1];
-                *input = inputs[2];
+                *length = inputs[2];
                 return self;
             },
             Window {
@@ -162,14 +227,12 @@ impl AExpr {
                 ..
             } => {
                 let offset = order_by.is_some() as usize;
-                *function = *inputs.last().unwrap();
+                *function = inputs[0];
                 partition_by.clear();
-                partition_by.extend_from_slice(&inputs[offset..inputs.len() - 1]);
-
+                partition_by.extend_from_slice(&inputs[1..inputs.len() - offset]);
                 if let Some((_, options)) = order_by {
-                    *order_by = Some((inputs[0], *options));
+                    *order_by = Some((*inputs.last().unwrap(), *options));
                 }
-
                 return self;
             },
         };
@@ -197,8 +260,6 @@ impl IRAggExpr {
             Std(input, _) => Single(*input),
             Var(input, _) => Single(*input),
             AggGroups(input) => Single(*input),
-            #[cfg(feature = "bitwise")]
-            Bitwise(input, _) => Single(*input),
         }
     }
     pub fn set_input(&mut self, input: Node) {
@@ -218,8 +279,6 @@ impl IRAggExpr {
             Std(input, _) => input,
             Var(input, _) => input,
             AggGroups(input) => input,
-            #[cfg(feature = "bitwise")]
-            Bitwise(input, _) => input,
         };
         *node = input;
     }
