@@ -43,9 +43,9 @@ impl<'a> IRBuilder<'a> {
 
         // Run the optimizer
         let mut conversion_optimizer = ConversionOptimizer::new(true, true, true);
-        conversion_optimizer.fill_scratch(&b.lp_arena.get(b.root).get_exprs(), b.expr_arena);
+        conversion_optimizer.fill_scratch(b.lp_arena.get(b.root).exprs(), b.expr_arena);
         conversion_optimizer
-            .optimize_exprs(b.expr_arena, b.lp_arena, b.root)
+            .optimize_exprs(b.expr_arena, b.lp_arena, b.root, false)
             .map_err(|e| e.context(format!("optimizing '{ir_name}' failed").into()))?;
 
         Ok(b)
@@ -53,7 +53,9 @@ impl<'a> IRBuilder<'a> {
 
     /// An escape hatch to add an `Expr`. Working with IR is preferred.
     pub fn add_expr(&mut self, expr: Expr) -> PolarsResult<ExprIR> {
-        to_expr_ir(expr, self.expr_arena)
+        let schema = self.lp_arena.get(self.root).schema(self.lp_arena);
+        let mut ctx = ExprToIRContext::new(self.expr_arena, &schema);
+        to_expr_ir(expr, &mut ctx)
     }
 
     pub fn project(self, exprs: Vec<ExprIR>, options: ProjectionOptions) -> Self {
@@ -146,6 +148,49 @@ impl<'a> IRBuilder<'a> {
         }
     }
 
+    pub fn drop<I, S>(self, names: I) -> Self
+    where
+        I: IntoIterator<Item = S>,
+        I::IntoIter: ExactSizeIterator,
+        S: Into<PlSmallStr>,
+    {
+        let names = names.into_iter();
+        // if len == 0, no projection has to be done. This is a select all operation.
+        if names.size_hint().0 == 0 {
+            self
+        } else {
+            let mut schema = self.schema().as_ref().as_ref().clone();
+
+            for name in names {
+                let name: PlSmallStr = name.into();
+                schema.remove(&name);
+            }
+
+            let lp = IR::SimpleProjection {
+                input: self.root,
+                columns: Arc::new(schema),
+            };
+            let node = self.lp_arena.add(lp);
+            IRBuilder::new(node, self.expr_arena, self.lp_arena)
+        }
+    }
+
+    pub fn sort(
+        self,
+        by_column: Vec<ExprIR>,
+        slice: Option<(i64, usize)>,
+        sort_options: SortMultipleOptions,
+    ) -> Self {
+        let ir = IR::Sort {
+            input: self.root,
+            by_column,
+            slice,
+            sort_options,
+        };
+        let node = self.lp_arena.add(ir);
+        IRBuilder::new(node, self.expr_arena, self.lp_arena)
+    }
+
     pub fn node(self) -> Node {
         self.root
     }
@@ -277,7 +322,7 @@ impl<'a> IRBuilder<'a> {
         other: Node,
         left_on: Vec<ExprIR>,
         right_on: Vec<ExprIR>,
-        options: Arc<JoinOptions>,
+        options: Arc<JoinOptionsIR>,
     ) -> Self {
         let schema_left = self.schema();
         let schema_right = self.lp_arena.get(other).schema(self.lp_arena);
