@@ -44,12 +44,23 @@ impl OutputName {
         self.get().expect("no output name set")
     }
 
+    pub fn into_inner(self) -> Option<PlSmallStr> {
+        match self {
+            OutputName::Alias(name) => Some(name),
+            OutputName::ColumnLhs(name) => Some(name),
+            OutputName::LiteralLhs(name) => Some(name),
+            #[cfg(feature = "dtype-struct")]
+            OutputName::Field(name) => Some(name),
+            OutputName::None => None,
+        }
+    }
+
     pub(crate) fn is_none(&self) -> bool {
         matches!(self, OutputName::None)
     }
 }
 
-#[derive(Debug)]
+#[derive(Clone, Debug)]
 #[cfg_attr(feature = "ir_serde", derive(Serialize, Deserialize))]
 pub struct ExprIR {
     /// Output name of this expression.
@@ -70,22 +81,13 @@ impl PartialEq for ExprIR {
     }
 }
 
-impl Clone for ExprIR {
-    fn clone(&self) -> Self {
-        let output_dtype = OnceLock::new();
-        if let Some(dt) = self.output_dtype.get() {
-            output_dtype.set(dt.clone()).unwrap()
-        }
-
-        ExprIR {
-            output_name: self.output_name.clone(),
-            node: self.node,
-            output_dtype,
-        }
+impl Borrow<Node> for ExprIR {
+    fn borrow(&self) -> &Node {
+        &self.node
     }
 }
 
-impl Borrow<Node> for ExprIR {
+impl Borrow<Node> for &ExprIR {
     fn borrow(&self) -> &Node {
         &self.node
     }
@@ -136,7 +138,7 @@ impl ExprIR {
                 } => {
                     match function {
                         #[cfg(feature = "dtype-struct")]
-                        FunctionExpr::StructExpr(StructFunction::FieldByName(name)) => {
+                        IRFunctionExpr::StructExpr(IRStructFunction::FieldByName(name)) => {
                             out.output_name = OutputName::Field(name.clone());
                         },
                         _ => {
@@ -150,10 +152,9 @@ impl ExprIR {
                     }
                     break;
                 },
-                AExpr::AnonymousFunction { input, options, .. } => {
+                AExpr::AnonymousFunction { input, fmt_str, .. } => {
                     if input.is_empty() {
-                        out.output_name =
-                            OutputName::LiteralLhs(PlSmallStr::from_static(options.fmt_str));
+                        out.output_name = OutputName::LiteralLhs(fmt_str.as_ref().clone());
                     } else {
                         out.output_name = input[0].output_name.clone();
                     }
@@ -163,13 +164,6 @@ impl ExprIR {
                     out.output_name = OutputName::LiteralLhs(get_len_name());
                     break;
                 },
-                AExpr::Alias(_, _) => {
-                    // Should be removed during conversion.
-                    #[cfg(debug_assertions)]
-                    {
-                        unreachable!()
-                    }
-                },
                 _ => {},
             }
         }
@@ -228,7 +222,9 @@ impl ExprIR {
         let out = node_to_expr(self.node, expr_arena);
 
         match &self.output_name {
-            OutputName::Alias(name) => out.alias(name.clone()),
+            OutputName::Alias(name) if expr_arena.get(self.node).to_name(expr_arena) != name => {
+                out.alias(name.clone())
+            },
             _ => out,
         }
     }
@@ -291,6 +287,10 @@ impl ExprIR {
         let name = self.output_name();
         Ok(Field::new(name.clone(), dtype.clone()))
     }
+
+    pub fn into_inner(self) -> (Node, OutputName) {
+        (self.node, self.output_name)
+    }
 }
 
 impl AsRef<ExprIR> for ExprIR {
