@@ -8,10 +8,10 @@ use std::borrow::Cow;
 use std::fmt;
 
 pub use dot::{EscapeLabel, IRDotDisplay, PathsDisplay, ScanSourcesDisplay};
-pub use format::{ExprIRDisplay, IRDisplay};
+pub use format::{ExprIRDisplay, IRDisplay, write_group_by, write_ir_non_recursive};
 use polars_core::prelude::*;
 use polars_utils::idx_vec::UnitVec;
-use polars_utils::unitvec;
+use polars_utils::unique_id::UniqueId;
 #[cfg(feature = "ir_serde")]
 use serde::{Deserialize, Serialize};
 use strum_macros::IntoStaticStr;
@@ -19,7 +19,7 @@ use strum_macros::IntoStaticStr;
 use self::hive::HivePartitionsDf;
 use crate::prelude::*;
 
-#[cfg_attr(feature = "ir_serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "ir_serde", derive(serde::Serialize, serde::Deserialize))]
 pub struct IRPlan {
     pub lp_top: Node,
     pub lp_arena: Arena<IR>,
@@ -59,7 +59,7 @@ pub enum IR {
         predicate: Option<ExprIR>,
         /// schema of the projected file
         output_schema: Option<SchemaRef>,
-        scan_type: Box<FileScan>,
+        scan_type: Box<FileScanIR>,
         /// generic options that can be used for all file types.
         unified_scan_args: Box<UnifiedScanArgs>,
     },
@@ -91,8 +91,8 @@ pub enum IR {
     },
     Cache {
         input: Node,
-        // Unique ID.
-        id: usize,
+        /// This holds the `Arc<DslPlan>` to guarantee uniqueness.
+        id: UniqueId,
         /// How many hits the cache must be saved in memory.
         cache_hits: u32,
     },
@@ -112,7 +112,7 @@ pub enum IR {
         schema: SchemaRef,
         left_on: Vec<ExprIR>,
         right_on: Vec<ExprIR>,
-        options: Arc<JoinOptions>,
+        options: Arc<JoinOptionsIR>,
     },
     HStack {
         input: Node,
@@ -176,7 +176,7 @@ impl IRPlan {
         self.lp_arena.get(self.lp_top)
     }
 
-    pub fn as_ref(&self) -> IRPlanRef {
+    pub fn as_ref(&self) -> IRPlanRef<'_> {
         IRPlanRef {
             lp_top: self.lp_top,
             lp_arena: &self.lp_arena,
@@ -184,11 +184,6 @@ impl IRPlan {
         }
     }
 
-    /// Extract the original logical plan if the plan is for the Streaming Engine
-    pub fn extract_streaming_plan(&self) -> Option<IRPlanRef> {
-        self.as_ref().extract_streaming_plan()
-    }
-
     pub fn describe(&self) -> String {
         self.as_ref().describe()
     }
@@ -197,11 +192,11 @@ impl IRPlan {
         self.as_ref().describe_tree_format()
     }
 
-    pub fn display(&self) -> format::IRDisplay {
+    pub fn display(&self) -> format::IRDisplay<'_> {
         self.as_ref().display()
     }
 
-    pub fn display_dot(&self) -> dot::IRDotDisplay {
+    pub fn display_dot(&self) -> dot::IRDotDisplay<'_> {
         self.as_ref().display_dot()
     }
 }
@@ -219,22 +214,6 @@ impl<'a> IRPlanRef<'a> {
         }
     }
 
-    /// Extract the original logical plan if the plan is for the Streaming Engine
-    pub fn extract_streaming_plan(self) -> Option<IRPlanRef<'a>> {
-        // @NOTE: the streaming engine replaces the whole tree with a MapFunction { Pipeline, .. }
-        // and puts the original plan somewhere in there. This is how we extract it. Disgusting, I
-        // know.
-        let IR::MapFunction { input: _, function } = self.root() else {
-            return None;
-        };
-
-        let FunctionIR::Pipeline { original, .. } = function else {
-            return None;
-        };
-
-        Some(original.as_ref()?.as_ref().as_ref())
-    }
-
     pub fn display(self) -> format::IRDisplay<'a> {
         format::IRDisplay::new(self)
     }
