@@ -18,6 +18,8 @@ mod fused;
 mod join_utils;
 pub(crate) use join_utils::ExprOrigin;
 mod expand_datasets;
+#[cfg(feature = "python")]
+pub use expand_datasets::ExpandedPythonScan;
 mod predicate_pushdown;
 mod projection_pushdown;
 mod set_order;
@@ -37,7 +39,7 @@ pub use predicate_pushdown::PredicatePushDown;
 pub use projection_pushdown::ProjectionPushDown;
 pub use simplify_expr::{SimplifyBooleanRule, SimplifyExprRule};
 use slice_pushdown_lp::SlicePushDown;
-pub use stack_opt::{OptimizationRule, StackOptimizer};
+pub use stack_opt::{OptimizationRule, OptimizeExprContext, StackOptimizer};
 
 use self::flatten_union::FlattenUnionRule;
 use self::set_order::set_order_flags;
@@ -64,6 +66,10 @@ pub(crate) fn init_hashmap<K, V>(max_len: Option<usize>) -> PlHashMap<K, V> {
     PlHashMap::with_capacity(std::cmp::min(max_len.unwrap_or(HASHMAP_SIZE), HASHMAP_SIZE))
 }
 
+pub(crate) fn pushdown_maintain_errors() -> bool {
+    std::env::var("POLARS_PUSHDOWN_OPT_MAINTAIN_ERRORS").as_deref() == Ok("1")
+}
+
 pub fn optimize(
     logical_plan: DslPlan,
     mut opt_flags: OptFlags,
@@ -75,20 +81,6 @@ pub fn optimize(
     #[allow(dead_code)]
     let verbose = verbose();
 
-    #[cfg(feature = "python")]
-    if opt_flags.streaming() {
-        polars_warn!(
-            Deprecation,
-            "\
-The old streaming engine is being deprecated and will soon be replaced by the new streaming \
-engine. Starting Polars version 1.23.0 and until the new streaming engine is released, the old \
-streaming engine may become less usable. For people who rely on the old streaming engine, it is \
-suggested to pin your version to before 1.23.0.
-
-More information on the new streaming engine: https://github.com/pola-rs/polars/issues/20947"
-        )
-    }
-
     // Gradually fill the rules passed to the optimizer
     let opt = StackOptimizer {};
     let mut rules: Vec<Box<dyn OptimizationRule>> = Vec::with_capacity(8);
@@ -112,6 +104,9 @@ More information on the new streaming engine: https://github.com/pola-rs/polars/
     #[cfg(not(feature = "cse"))]
     let comm_subexpr_elim = false;
 
+    // Note: This can be in opt_flags in the future if needed.
+    let pushdown_maintain_errors = pushdown_maintain_errors();
+
     // During debug we check if the optimizations have not modified the final schema.
     #[cfg(debug_assertions)]
     let prev_schema = lp_arena.get(lp_top).schema(lp_arena).into_owned();
@@ -184,20 +179,19 @@ More information on the new streaming engine: https://github.com/pola-rs/polars/
     }
 
     if opt_flags.predicate_pushdown() {
-        let mut predicate_pushdown_opt =
-            PredicatePushDown::new(expr_eval, opt_flags.new_streaming());
+        let mut predicate_pushdown_opt = PredicatePushDown::new(
+            expr_eval,
+            pushdown_maintain_errors,
+            opt_flags.new_streaming(),
+        );
         let alp = lp_arena.take(lp_top);
         let alp = predicate_pushdown_opt.optimize(alp, lp_arena, expr_arena)?;
         lp_arena.replace(lp_top, alp);
     }
 
-    if opt_flags.cluster_with_columns() {
-        cluster_with_columns::optimize(lp_top, lp_arena, expr_arena)
-    }
-
     // Make sure it is after predicate pushdown
     if opt_flags.collapse_joins() && get_or_init_members!().has_filter_with_join_input {
-        collapse_joins::optimize(lp_top, lp_arena, expr_arena);
+        collapse_joins::optimize(lp_top, lp_arena, expr_arena, opt_flags.new_streaming());
     }
 
     // Make sure its before slice pushdown.
@@ -212,8 +206,14 @@ More information on the new streaming engine: https://github.com/pola-rs/polars/
     }
 
     if opt_flags.slice_pushdown() {
-        let mut slice_pushdown_opt =
-            SlicePushDown::new(opt_flags.streaming(), opt_flags.new_streaming());
+        let mut slice_pushdown_opt = SlicePushDown::new(
+            // We don't maintain errors on slice as the behavior is much more predictable that way.
+            //
+            // Even if we enable maintain_errors (thereby preventing the slice from being pushed),
+            // the new-streaming engine still may not error due to early-stopping.
+            false, // maintain_errors
+            opt_flags.new_streaming(),
+        );
         let alp = lp_arena.take(lp_top);
         let alp = slice_pushdown_opt.optimize(alp, lp_arena, expr_arena)?;
 
@@ -222,6 +222,7 @@ More information on the new streaming engine: https://github.com/pola-rs/polars/
         // Expressions use the stack optimizer.
         rules.push(Box::new(slice_pushdown_opt));
     }
+
     // This optimization removes branches, so we must do it when type coercion
     // is completed.
     if opt_flags.simplify_expr() {
@@ -237,9 +238,14 @@ More information on the new streaming engine: https://github.com/pola-rs/polars/
 
     lp_top = opt.optimize_loop(&mut rules, expr_arena, lp_arena, lp_top)?;
 
+    if opt_flags.cluster_with_columns() {
+        cluster_with_columns::optimize(lp_top, lp_arena, expr_arena)
+    }
+
     if _cse_plan_changed
-        && get_members_opt!()
-            .is_some_and(|members| members.has_joins_or_unions && members.has_cache)
+        && get_members_opt!().is_some_and(|members| {
+            (members.has_joins_or_unions | members.has_sink_multiple) && members.has_cache
+        })
     {
         // We only want to run this on cse inserted caches
         cache_states::set_cache_states(
@@ -249,6 +255,7 @@ More information on the new streaming engine: https://github.com/pola-rs/polars/
             scratch,
             expr_eval,
             verbose,
+            pushdown_maintain_errors,
             opt_flags.new_streaming(),
         )?;
     }
