@@ -44,10 +44,8 @@ impl TreeWalker for Expr {
         let ret = match self {
             Alias(l, r) => Alias(am(l, f)?, r),
             Column(_) => self,
-            Columns(_) => self,
-            DtypeColumn(_) => self,
-            IndexColumn(_) => self,
             Literal(_) => self,
+            DataTypeFunction(_) => self,
             #[cfg(feature = "dtype-struct")]
             Field(_) => self,
             BinaryExpr { left, op, right } => {
@@ -72,27 +70,23 @@ impl TreeWalker for Expr {
                 AggGroups(x) => AggGroups(am(x, f)?),
                 Std(x, ddf) => Std(am(x, f)?, ddf),
                 Var(x, ddf) => Var(am(x, f)?, ddf),
-                #[cfg(feature = "bitwise")]
-                Bitwise(x, t) => Bitwise(am(x, f)?, t),
             }),
             Ternary { predicate, truthy, falsy } => Ternary { predicate: am(predicate, &mut f)?, truthy: am(truthy, &mut f)?, falsy: am(falsy, f)? },
-            Function { input, function, options } => Function { input: input.into_iter().map(f).collect::<Result<_, _>>()?, function, options },
-            Explode(expr) => Explode(am(expr, f)?),
+            Function { input, function } => Function { input: input.into_iter().map(f).collect::<Result<_, _>>()?, function },
+            Explode { input, skip_empty } => Explode { input: am(input, f)?, skip_empty },
             Filter { input, by } => Filter { input: am(input, &mut f)?, by: am(by, f)? },
             Window { function, partition_by, order_by, options } => {
                 let partition_by = partition_by.into_iter().map(&mut f).collect::<Result<_, _>>()?;
                 Window { function: am(function, f)?, partition_by, order_by, options }
             },
-            Wildcard => Wildcard,
             Slice { input, offset, length } => Slice { input: am(input, &mut f)?, offset: am(offset, &mut f)?, length: am(length, f)? },
-            Exclude(expr, excluded) => Exclude(am(expr, f)?, excluded),
             KeepName(expr) => KeepName(am(expr, f)?),
             Len => Len,
-            Nth(_) => self,
             RenameAlias { function, expr } => RenameAlias { function, expr: am(expr, f)? },
-            AnonymousFunction { input, function, output_type, options } => {
-                AnonymousFunction { input: input.into_iter().map(f).collect::<Result<_, _>>()?, function, output_type, options }
+            AnonymousFunction { input, function, output_type, options, fmt_str } => {
+                AnonymousFunction { input: input.into_iter().map(f).collect::<Result<_, _>>()?, function, output_type, options, fmt_str }
             },
+            Eval { expr: input, evaluation, variant } => Eval { expr: am(input, &mut f)?, evaluation: am(evaluation, f)?, variant },
             SubPlan(_, _) => self,
             Selector(_) => self,
         };
@@ -165,7 +159,6 @@ impl AExpr {
     fn is_equal_node(&self, other: &Self) -> bool {
         use AExpr::*;
         match (self, other) {
-            (Alias(_, l), Alias(_, r)) => l == r,
             (Column(l), Column(r)) => l == r,
             (Literal(l), Literal(r)) => l == r,
             (Window { options: l, .. }, Window { options: r, .. }) => l == r,
@@ -186,8 +179,17 @@ impl AExpr {
             | (Filter { .. }, Filter { .. })
             | (Ternary { .. }, Ternary { .. })
             | (Len, Len)
-            | (Slice { .. }, Slice { .. })
-            | (Explode(_), Explode(_)) => true,
+            | (Slice { .. }, Slice { .. }) => true,
+            (
+                Explode {
+                    expr: _,
+                    skip_empty: l_skip_empty,
+                },
+                Explode {
+                    expr: _,
+                    skip_empty: r_skip_empty,
+                },
+            ) => l_skip_empty == r_skip_empty,
             (
                 SortBy {
                     sort_options: l_sort_options,
@@ -229,15 +231,15 @@ impl AExpr {
 
 #[cfg(feature = "cse")]
 impl<'a> AExprArena<'a> {
-    fn new(node: Node, arena: &'a Arena<AExpr>) -> Self {
+    pub fn new(node: Node, arena: &'a Arena<AExpr>) -> Self {
         Self { node, arena }
     }
-    fn to_aexpr(&self) -> &'a AExpr {
+    pub fn to_aexpr(&self) -> &'a AExpr {
         self.arena.get(self.node)
     }
 
     // Check single node on equality
-    fn is_equal_single(&self, other: &Self) -> bool {
+    pub fn is_equal_single(&self, other: &Self) -> bool {
         let self_ae = self.to_aexpr();
         let other_ae = other.to_aexpr();
         self_ae.is_equal_node(other_ae)
@@ -247,8 +249,8 @@ impl<'a> AExprArena<'a> {
 #[cfg(feature = "cse")]
 impl PartialEq for AExprArena<'_> {
     fn eq(&self, other: &Self) -> bool {
-        let mut scratch1 = vec![];
-        let mut scratch2 = vec![];
+        let mut scratch1 = unitvec![];
+        let mut scratch2 = unitvec![];
 
         scratch1.push(self.node);
         scratch2.push(other.node);
@@ -263,8 +265,8 @@ impl PartialEq for AExprArena<'_> {
                         return false;
                     }
 
-                    l.to_aexpr().nodes(&mut scratch1);
-                    r.to_aexpr().nodes(&mut scratch2);
+                    l.to_aexpr().inputs_rev(&mut scratch1);
+                    r.to_aexpr().inputs_rev(&mut scratch2);
                 },
                 (None, None) => return true,
                 _ => return false,
@@ -282,7 +284,7 @@ impl TreeWalker for AexprNode {
     ) -> PolarsResult<VisitRecursion> {
         let mut scratch = unitvec![];
 
-        self.to_aexpr(arena).nodes(&mut scratch);
+        self.to_aexpr(arena).inputs_rev(&mut scratch);
         for node in scratch.as_slice() {
             let aenode = AexprNode::new(*node);
             match op(&aenode, arena)? {
@@ -303,7 +305,7 @@ impl TreeWalker for AexprNode {
         let mut scratch = unitvec![];
 
         let ae = arena.get(self.node).clone();
-        ae.nodes(&mut scratch);
+        ae.inputs_rev(&mut scratch);
 
         // rewrite the nodes
         for node in scratch.as_mut_slice() {
@@ -311,6 +313,7 @@ impl TreeWalker for AexprNode {
             *node = op(aenode, arena)?.node;
         }
 
+        scratch.as_mut_slice().reverse();
         let ae = ae.replace_inputs(&scratch);
         self.node = arena.add(ae);
         Ok(self)
