@@ -84,7 +84,7 @@ impl PyCatalogClient {
     }
 
     #[pyo3(signature = (catalog_name))]
-    pub fn list_namespaces(&self, py: Python, catalog_name: &str) -> PyResult<PyObject> {
+    pub fn list_namespaces(&self, py: Python<'_>, catalog_name: &str) -> PyResult<PyObject> {
         let v = py.enter_polars(|| {
             pl_async::get_runtime().block_in_place_on(self.client().list_namespaces(catalog_name))
         })?;
@@ -113,7 +113,7 @@ impl PyCatalogClient {
     #[pyo3(signature = (catalog_name, namespace))]
     pub fn list_tables(
         &self,
-        py: Python,
+        py: Python<'_>,
         catalog_name: &str,
         namespace: &str,
     ) -> PyResult<PyObject> {
@@ -147,7 +147,7 @@ impl PyCatalogClient {
     #[pyo3(signature = (table_name, catalog_name, namespace))]
     pub fn get_table_info(
         &self,
-        py: Python,
+        py: Python<'_>,
         table_name: &str,
         catalog_name: &str,
         namespace: &str,
@@ -168,7 +168,7 @@ impl PyCatalogClient {
     #[pyo3(signature = (table_id, write))]
     pub fn get_table_credentials(
         &self,
-        py: Python,
+        py: Python<'_>,
         table_id: &str,
         write: bool,
     ) -> PyResult<PyObject> {
@@ -235,7 +235,7 @@ impl PyCatalogClient {
     #[pyo3(signature = (catalog_name, namespace, table_name, cloud_options, credential_provider, retries))]
     pub fn scan_table(
         &self,
-        py: Python,
+        py: Python<'_>,
         catalog_name: &str,
         namespace: &str,
         table_name: &str,
@@ -274,7 +274,7 @@ impl PyCatalogClient {
     #[pyo3(signature = (catalog_name, comment, storage_root))]
     pub fn create_catalog(
         &self,
-        py: Python,
+        py: Python<'_>,
         catalog_name: &str,
         comment: Option<&str>,
         storage_root: Option<&str>,
@@ -293,7 +293,7 @@ impl PyCatalogClient {
     }
 
     #[pyo3(signature = (catalog_name, force))]
-    pub fn delete_catalog(&self, py: Python, catalog_name: &str, force: bool) -> PyResult<()> {
+    pub fn delete_catalog(&self, py: Python<'_>, catalog_name: &str, force: bool) -> PyResult<()> {
         py.allow_threads(|| {
             pl_async::get_runtime()
                 .block_in_place_on(self.client().delete_catalog(catalog_name, force))
@@ -304,7 +304,7 @@ impl PyCatalogClient {
     #[pyo3(signature = (catalog_name, namespace, comment, storage_root))]
     pub fn create_namespace(
         &self,
-        py: Python,
+        py: Python<'_>,
         catalog_name: &str,
         namespace: &str,
         comment: Option<&str>,
@@ -327,7 +327,7 @@ impl PyCatalogClient {
     #[pyo3(signature = (catalog_name, namespace, force))]
     pub fn delete_namespace(
         &self,
-        py: Python,
+        py: Python<'_>,
         catalog_name: &str,
         namespace: &str,
         force: bool,
@@ -348,7 +348,7 @@ impl PyCatalogClient {
     ))]
     pub fn create_table(
         &self,
-        py: Python,
+        py: Python<'_>,
         catalog_name: &str,
         namespace: &str,
         table_name: &str,
@@ -388,7 +388,7 @@ impl PyCatalogClient {
     #[pyo3(signature = (catalog_name, namespace, table_name))]
     pub fn delete_table(
         &self,
-        py: Python,
+        py: Python<'_>,
         catalog_name: &str,
         namespace: &str,
         table_name: &str,
@@ -405,7 +405,7 @@ impl PyCatalogClient {
 
     #[pyo3(signature = (type_json))]
     #[staticmethod]
-    pub fn type_json_to_polars_type(py: Python, type_json: &str) -> PyResult<PyObject> {
+    pub fn type_json_to_polars_type(py: Python<'_>, type_json: &str) -> PyResult<PyObject> {
         Ok(Wrap(parse_type_json_str(type_json).map_err(to_py_err)?)
             .into_pyobject(py)?
             .unbind())
@@ -414,7 +414,7 @@ impl PyCatalogClient {
     #[pyo3(signature = (catalog_info_cls, namespace_info_cls, table_info_cls, column_info_cls))]
     #[staticmethod]
     pub fn init_classes(
-        py: Python,
+        py: Python<'_>,
         catalog_info_cls: Py<PyAny>,
         namespace_info_cls: Py<PyAny>,
         table_info_cls: Py<PyAny>,
@@ -434,7 +434,7 @@ impl PyCatalogClient {
 }
 
 fn catalog_info_to_pyobject(
-    py: Python,
+    py: Python<'_>,
     CatalogInfo {
         name,
         comment,
@@ -472,7 +472,7 @@ fn catalog_info_to_pyobject(
 }
 
 fn namespace_info_to_pyobject(
-    py: Python,
+    py: Python<'_>,
     NamespaceInfo {
         name,
         comment,
@@ -506,7 +506,7 @@ fn namespace_info_to_pyobject(
         .call((), Some(&dict))
 }
 
-fn table_info_to_pyobject(py: Python, table_info: TableInfo) -> PyResult<Bound<'_, PyAny>> {
+fn table_info_to_pyobject(py: Python<'_>, table_info: TableInfo) -> PyResult<Bound<'_, PyAny>> {
     let TableInfo {
         name,
         table_id,
@@ -590,7 +590,7 @@ fn table_info_to_pyobject(py: Python, table_info: TableInfo) -> PyResult<Bound<'
 }
 
 fn properties_to_pyobject(
-    py: Python,
+    py: Python<'_>,
     properties: PlHashMap<PlSmallStr, String>,
 ) -> Bound<'_, PyDict> {
     let dict = PyDict::new(py);
