@@ -47,12 +47,19 @@ impl PyExpr {
     }
 
     #[cfg(feature = "timezones")]
-    fn dt_convert_time_zone(&self, time_zone: String) -> Self {
-        self.inner
+    fn dt_convert_time_zone(&self, time_zone: String) -> PyResult<Self> {
+        use crate::utils::to_py_err;
+
+        Ok(self
+            .inner
             .clone()
             .dt()
-            .convert_time_zone(time_zone.into())
-            .into()
+            .convert_time_zone(
+                TimeZone::opt_try_new(Some(PlSmallStr::from(time_zone)))
+                    .map_err(to_py_err)?
+                    .unwrap_or(TimeZone::UTC),
+            )
+            .into())
     }
 
     fn dt_cast_time_unit(&self, time_unit: Wrap<TimeUnit>) -> Self {
@@ -66,12 +73,19 @@ impl PyExpr {
         time_zone: Option<String>,
         ambiguous: Self,
         non_existent: Wrap<NonExistent>,
-    ) -> Self {
-        self.inner
+    ) -> PyResult<Self> {
+        use crate::utils::to_py_err;
+
+        Ok(self
+            .inner
             .clone()
             .dt()
-            .replace_time_zone(time_zone.map(|x| x.into()), ambiguous.inner, non_existent.0)
-            .into()
+            .replace_time_zone(
+                TimeZone::opt_try_new(time_zone.map(PlSmallStr::from_string)).map_err(to_py_err)?,
+                ambiguous.inner,
+                non_existent.0,
+            )
+            .into())
     }
 
     fn dt_truncate(&self, every: Self) -> Self {
