@@ -6,13 +6,15 @@ use polars::series::ops::NullBehavior;
 use polars_core::chunked_array::cast::CastOptions;
 use polars_core::series::IsSorted;
 use polars_plan::plans::predicates::aexpr_to_skip_batch_predicate;
-use polars_plan::plans::{node_to_expr, to_aexpr};
+use polars_plan::plans::{ExprToIRContext, node_to_expr, to_expr_ir};
 use polars_utils::arena::Arena;
 use pyo3::class::basic::CompareOp;
 use pyo3::prelude::*;
 
+use super::datatype::PyDataTypeExpr;
+use super::selector::PySelector;
 use crate::PyExpr;
-use crate::conversion::{Wrap, parse_fill_null_strategy, vec_extract_wrapped};
+use crate::conversion::{Wrap, parse_fill_null_strategy};
 use crate::error::PyPolarsErr;
 use crate::map::lazy::map_single;
 use crate::utils::EnterPolarsExt;
@@ -242,9 +244,7 @@ impl PyExpr {
     fn null_count(&self) -> Self {
         self.inner.clone().null_count().into()
     }
-    fn cast(&self, dtype: Wrap<DataType>, strict: bool, wrap_numerical: bool) -> Self {
-        let dt = dtype.0;
-
+    fn cast(&self, dtype: PyDataTypeExpr, strict: bool, wrap_numerical: bool) -> Self {
         let options = if wrap_numerical {
             CastOptions::Overflowing
         } else if strict {
@@ -253,7 +253,7 @@ impl PyExpr {
             CastOptions::NonStrict
         };
 
-        let expr = self.inner.clone().cast_with_options(dt, options);
+        let expr = self.inner.clone().cast_with_options(dtype.inner, options);
         expr.into()
     }
     fn sort_with(&self, descending: bool, nulls_last: bool) -> Self {
@@ -270,16 +270,7 @@ impl PyExpr {
     }
 
     fn arg_sort(&self, descending: bool, nulls_last: bool) -> Self {
-        self.inner
-            .clone()
-            .arg_sort(SortOptions {
-                descending,
-                nulls_last,
-                multithreaded: true,
-                maintain_order: false,
-                limit: None,
-            })
-            .into()
+        self.inner.clone().arg_sort(descending, nulls_last).into()
     }
 
     #[cfg(feature = "top_k")]
@@ -328,10 +319,11 @@ impl PyExpr {
     }
 
     #[cfg(feature = "search_sorted")]
-    fn search_sorted(&self, element: Self, side: Wrap<SearchSortedSide>) -> Self {
+    #[pyo3(signature = (element, side, descending=false))]
+    fn search_sorted(&self, element: Self, side: Wrap<SearchSortedSide>, descending: bool) -> Self {
         self.inner
             .clone()
-            .search_sorted(element.inner, side.0)
+            .search_sorted(element.inner, side.0, descending)
             .into()
     }
 
@@ -425,6 +417,13 @@ impl PyExpr {
             .into()
     }
 
+    fn is_close(&self, other: Self, abs_tol: f64, rel_tol: f64, nans_equal: bool) -> Self {
+        self.inner
+            .clone()
+            .is_close(other.inner, abs_tol, rel_tol, nans_equal)
+            .into()
+    }
+
     #[cfg(feature = "approx_unique")]
     fn approx_n_unique(&self) -> Self {
         self.inner.clone().approx_n_unique().into()
@@ -585,16 +584,18 @@ impl PyExpr {
     #[pyo3(signature = (partition_by, order_by, order_by_descending, order_by_nulls_last, mapping_strategy))]
     fn over(
         &self,
-        partition_by: Vec<Self>,
+        partition_by: Option<Vec<Self>>,
         order_by: Option<Vec<Self>>,
         order_by_descending: bool,
         order_by_nulls_last: bool,
         mapping_strategy: Wrap<WindowMapping>,
-    ) -> Self {
-        let partition_by = partition_by
-            .into_iter()
-            .map(|e| e.inner)
-            .collect::<Vec<Expr>>();
+    ) -> PyResult<Self> {
+        let partition_by = partition_by.map(|partition_by| {
+            partition_by
+                .into_iter()
+                .map(|e| e.inner)
+                .collect::<Vec<Expr>>()
+        });
 
         let order_by = order_by.map(|order_by| {
             (
@@ -608,10 +609,12 @@ impl PyExpr {
             )
         });
 
-        self.inner
+        Ok(self
+            .inner
             .clone()
             .over_with_options(partition_by, order_by, mapping_strategy.0)
-            .into()
+            .map_err(PyPolarsErr::from)?
+            .into())
     }
 
     fn rolling(
@@ -681,10 +684,10 @@ impl PyExpr {
         self.inner.clone().cum_count(reverse).into()
     }
 
-    fn cumulative_eval(&self, expr: Self, min_periods: usize, parallel: bool) -> Self {
+    fn cumulative_eval(&self, expr: Self, min_samples: usize) -> Self {
         self.inner
             .clone()
-            .cumulative_eval(expr.inner, min_periods, parallel)
+            .cumulative_eval(expr.inner, min_samples)
             .into()
     }
 
@@ -696,23 +699,22 @@ impl PyExpr {
         self.inner.clone().shrink_dtype().into()
     }
 
-    #[pyo3(signature = (lambda, output_type, agg_list, is_elementwise, returns_scalar))]
+    #[pyo3(signature = (lambda, output_type, is_elementwise, returns_scalar, is_ufunc))]
     fn map_batches(
         &self,
         lambda: PyObject,
-        output_type: Option<Wrap<DataType>>,
-        agg_list: bool,
+        output_type: Option<PyDataTypeExpr>,
         is_elementwise: bool,
         returns_scalar: bool,
+        is_ufunc: bool,
     ) -> Self {
-        map_single(
-            self,
-            lambda,
-            output_type,
-            agg_list,
-            is_elementwise,
-            returns_scalar,
-        )
+        let output_type = if is_ufunc {
+            debug_assert!(output_type.is_none());
+            Some(DataTypeExpr::Literal(DataType::Unknown(UnknownKind::Ufunc)))
+        } else {
+            output_type.map(|v| v.inner)
+        };
+        map_single(self, lambda, output_type, is_elementwise, returns_scalar)
     }
 
     fn dot(&self, other: Self) -> Self {
@@ -725,13 +727,6 @@ impl PyExpr {
     fn mode(&self) -> Self {
         self.inner.clone().mode().into()
     }
-    fn exclude(&self, columns: Vec<String>) -> Self {
-        self.inner.clone().exclude(columns).into()
-    }
-    fn exclude_dtype(&self, dtypes: Vec<Wrap<DataType>>) -> Self {
-        let dtypes = vec_extract_wrapped(dtypes);
-        self.inner.clone().exclude_dtype(&dtypes).into()
-    }
     fn interpolate(&self, method: Wrap<InterpolationMethod>) -> Self {
         self.inner.clone().interpolate(method.0).into()
     }
@@ -912,7 +907,7 @@ impl PyExpr {
         old: PyExpr,
         new: PyExpr,
         default: Option<PyExpr>,
-        return_dtype: Option<Wrap<DataType>>,
+        return_dtype: Option<PyDataTypeExpr>,
     ) -> Self {
         self.inner
             .clone()
@@ -920,7 +915,7 @@ impl PyExpr {
                 old.inner,
                 new.inner,
                 default.map(|e| e.inner),
-                return_dtype.map(|dt| dt.0),
+                return_dtype.map(|dt| dt.inner),
             )
             .into()
     }
@@ -942,10 +937,12 @@ impl PyExpr {
     }
 
     #[pyo3(signature = (schema))]
-    fn skip_batch_predicate(&self, py: Python, schema: Wrap<Schema>) -> PyResult<Option<Self>> {
+    fn skip_batch_predicate(&self, py: Python<'_>, schema: Wrap<Schema>) -> PyResult<Option<Self>> {
         let mut aexpr_arena = Arena::new();
         py.enter_polars(|| {
-            let node = to_aexpr(self.inner.clone(), &mut aexpr_arena)?;
+            let mut ctx = ExprToIRContext::new(&mut aexpr_arena, &schema.0);
+            ctx.allow_unknown = true;
+            let node = to_expr_ir(self.inner.clone(), &mut ctx)?.node();
             let Some(node) = aexpr_to_skip_batch_predicate(node, &mut aexpr_arena, &schema.0)
             else {
                 return Ok(None);
@@ -956,4 +953,22 @@ impl PyExpr {
             }))
         })
     }
+
+    #[allow(clippy::wrong_self_convention)]
+    fn into_selector(&self) -> PyResult<PySelector> {
+        Ok(self
+            .inner
+            .clone()
+            .into_selector()
+            .ok_or_else(
+                || polars_err!(InvalidOperation: "expr `{}` is not a selector", &self.inner),
+            )
+            .map_err(PyPolarsErr::from)?
+            .into())
+    }
+
+    #[staticmethod]
+    fn new_selector(selector: PySelector) -> Self {
+        Expr::Selector(selector.inner).into()
+    }
 }
