@@ -1,8 +1,7 @@
-use std::borrow::Cow;
-
 use polars::prelude::*;
 use polars::series::ops::NullBehavior;
 use polars_utils::pl_str::PlSmallStr;
+use polars_utils::python_function::PythonObject;
 use pyo3::prelude::*;
 use pyo3::types::PySequence;
 
@@ -30,8 +29,12 @@ impl PyExpr {
     }
 
     #[cfg(feature = "is_in")]
-    fn list_contains(&self, other: PyExpr) -> Self {
-        self.inner.clone().list().contains(other.inner).into()
+    fn list_contains(&self, other: PyExpr, nulls_equal: bool) -> Self {
+        self.inner
+            .clone()
+            .list()
+            .contains(other.inner, nulls_equal)
+            .into()
     }
 
     #[cfg(feature = "list_count")]
@@ -43,8 +46,17 @@ impl PyExpr {
         Ok(self.inner.clone().list().diff(n, null_behavior.0).into())
     }
 
-    fn list_eval(&self, expr: PyExpr, parallel: bool) -> Self {
-        self.inner.clone().list().eval(expr.inner, parallel).into()
+    fn list_eval(&self, expr: PyExpr, _parallel: bool) -> Self {
+        self.inner.clone().list().eval(expr.inner).into()
+    }
+
+    #[cfg(feature = "list_filter")]
+    fn list_filter(&self, predicate: PyExpr) -> Self {
+        self.inner
+            .clone()
+            .list()
+            .eval(Expr::Column(PlSmallStr::EMPTY).filter(predicate.inner))
+            .into()
     }
 
     fn list_get(&self, index: PyExpr, null_on_oob: bool) -> Self {
@@ -72,39 +84,19 @@ impl PyExpr {
     }
 
     fn list_mean(&self) -> Self {
-        self.inner
-            .clone()
-            .list()
-            .mean()
-            .with_fmt("list.mean")
-            .into()
+        self.inner.clone().list().mean().into()
     }
 
     fn list_median(&self) -> Self {
-        self.inner
-            .clone()
-            .list()
-            .median()
-            .with_fmt("list.median")
-            .into()
+        self.inner.clone().list().median().into()
     }
 
     fn list_std(&self, ddof: u8) -> Self {
-        self.inner
-            .clone()
-            .list()
-            .std(ddof)
-            .with_fmt("list.std")
-            .into()
+        self.inner.clone().list().std(ddof).into()
     }
 
     fn list_var(&self, ddof: u8) -> Self {
-        self.inner
-            .clone()
-            .list()
-            .var(ddof)
-            .with_fmt("list.var")
-            .into()
+        self.inner.clone().list().var(ddof).into()
     }
 
     fn list_min(&self) -> Self {
@@ -145,7 +137,7 @@ impl PyExpr {
     }
 
     fn list_sum(&self) -> Self {
-        self.inner.clone().list().sum().with_fmt("list.sum").into()
+        self.inner.clone().list().sum().into()
     }
 
     #[cfg(feature = "list_drop_nulls")]
@@ -212,23 +204,15 @@ impl PyExpr {
         &self,
         width_strat: Wrap<ListToStructWidthStrategy>,
         name_gen: Option<PyObject>,
-        upper_bound: usize,
+        upper_bound: Option<usize>,
     ) -> PyResult<Self> {
-        let name_gen = name_gen.map(|lambda| {
-            NameGenerator::from_func(move |idx: usize| {
-                Python::with_gil(|py| {
-                    let out = lambda.call1(py, (idx,)).unwrap();
-                    let out: PlSmallStr = out.extract::<Cow<str>>(py).unwrap().as_ref().into();
-                    out
-                })
-            })
-        });
+        let name_gen = name_gen.map(|lambda| PlanCallback::new_python(PythonObject(lambda)));
 
         Ok(self
             .inner
             .clone()
             .list()
-            .to_struct(ListToStructArgs::InferWidth {
+            .to_struct(ListToStruct::InferWidth {
                 infer_field_strategy: width_strat.0,
                 get_index_name: name_gen,
                 max_fields: upper_bound,
@@ -242,7 +226,7 @@ impl PyExpr {
             .inner
             .clone()
             .list()
-            .to_struct(ListToStructArgs::FixedWidth(
+            .to_struct(ListToStruct::FixedWidth(
                 names
                     .try_iter()?
                     .map(|x| Ok(x?.extract::<Wrap<PlSmallStr>>()?.0))
