@@ -6,6 +6,7 @@ mod binary;
 mod bitwise;
 #[cfg(feature = "pymethods")]
 mod categorical;
+pub mod datatype;
 #[cfg(feature = "pymethods")]
 mod datetime;
 #[cfg(feature = "pymethods")]
@@ -18,6 +19,7 @@ mod meta;
 mod name;
 #[cfg(feature = "pymethods")]
 mod rolling;
+pub mod selector;
 #[cfg(feature = "pymethods")]
 mod serde;
 #[cfg(feature = "pymethods")]
