@@ -1,8 +1,4 @@
-use std::borrow::Cow;
-
-use polars::prelude::*;
-use polars_utils::format_pl_smallstr;
-use polars_utils::pl_str::PlSmallStr;
+use polars_utils::python_function::PythonObject;
 use pyo3::prelude::*;
 
 use crate::PyExpr;
@@ -17,15 +13,7 @@ impl PyExpr {
         self.inner
             .clone()
             .name()
-            .map(move |name| {
-                let out = Python::with_gil(|py| lambda.call1(py, (name.as_str(),)));
-                match out {
-                    Ok(out) => Ok(format_pl_smallstr!("{}", out)),
-                    Err(e) => Err(PolarsError::ComputeError(
-                        format!("Python function in 'name.map' produced an error: {e}.").into(),
-                    )),
-                }
-            })
+            .map_udf(PythonObject(lambda))
             .into()
     }
 
@@ -46,15 +34,11 @@ impl PyExpr {
     }
 
     fn name_map_fields(&self, name_mapper: PyObject) -> Self {
-        let name_mapper = Arc::new(move |name: &str| {
-            Python::with_gil(|py| {
-                let out = name_mapper.call1(py, (name,)).unwrap();
-                let out: PlSmallStr = out.extract::<Cow<str>>(py).unwrap().as_ref().into();
-                out
-            })
-        }) as FieldsNameMapper;
-
-        self.inner.clone().name().map_fields(name_mapper).into()
+        self.inner
+            .clone()
+            .name()
+            .map_fields_udf(PythonObject(name_mapper))
+            .into()
     }
 
     fn name_prefix_fields(&self, prefix: &str) -> Self {
