@@ -1,6 +1,7 @@
 use polars::prelude::*;
 use pyo3::prelude::*;
 
+use super::datatype::PyDataTypeExpr;
 use crate::PyExpr;
 use crate::conversion::Wrap;
 use crate::error::PyPolarsErr;
@@ -33,14 +34,14 @@ impl PyExpr {
         &self,
         format: Option<String>,
         time_unit: Option<Wrap<TimeUnit>>,
-        time_zone: Option<Wrap<TimeZone>>,
+        time_zone: Wrap<Option<TimeZone>>,
         strict: bool,
         exact: bool,
         cache: bool,
         ambiguous: Self,
     ) -> Self {
         let format = format.map(|x| x.into());
-        let time_zone = time_zone.map(|x| x.0);
+        let time_zone = time_zone.0;
 
         let options = StrptimeOptions {
             format,
@@ -164,15 +165,23 @@ impl PyExpr {
         self.inner.clone().str().reverse().into()
     }
 
-    fn str_pad_start(&self, length: usize, fill_char: char) -> Self {
-        self.inner.clone().str().pad_start(length, fill_char).into()
+    fn str_pad_start(&self, length: PyExpr, fill_char: char) -> Self {
+        self.inner
+            .clone()
+            .str()
+            .pad_start(length.inner, fill_char)
+            .into()
     }
 
-    fn str_pad_end(&self, length: usize, fill_char: char) -> Self {
-        self.inner.clone().str().pad_end(length, fill_char).into()
+    fn str_pad_end(&self, length: PyExpr, fill_char: char) -> Self {
+        self.inner
+            .clone()
+            .str()
+            .pad_end(length.inner, fill_char)
+            .into()
     }
 
-    fn str_zfill(&self, length: Self) -> Self {
+    fn str_zfill(&self, length: PyExpr) -> Self {
         self.inner.clone().str().zfill(length.inner).into()
     }
 
@@ -220,12 +229,12 @@ impl PyExpr {
         self.inner.clone().str().base64_decode(strict).into()
     }
 
-    fn str_to_integer(&self, base: Self, strict: bool) -> Self {
+    #[pyo3(signature = (base, dtype=Some(Wrap(DataType::Int64)), strict=true))]
+    fn str_to_integer(&self, base: Self, dtype: Option<Wrap<DataType>>, strict: bool) -> Self {
         self.inner
             .clone()
             .str()
-            .to_integer(base.inner, strict)
-            .with_fmt("str.to_integer")
+            .to_integer(base.inner, dtype.map(|wrap| wrap.0), strict)
             .into()
     }
 
@@ -233,10 +242,10 @@ impl PyExpr {
     #[pyo3(signature = (dtype=None, infer_schema_len=None))]
     fn str_json_decode(
         &self,
-        dtype: Option<Wrap<DataType>>,
+        dtype: Option<PyDataTypeExpr>,
         infer_schema_len: Option<usize>,
     ) -> Self {
-        let dtype = dtype.map(|wrap| wrap.0);
+        let dtype = dtype.map(|wrap| wrap.inner);
         self.inner
             .clone()
             .str()
