@@ -1,20 +1,17 @@
 use std::io::BufReader;
 
-use arrow::array::Utf8ViewArray;
 #[cfg(any(feature = "ipc", feature = "parquet"))]
 use polars::prelude::ArrowSchema;
-use polars_core::datatypes::create_enum_dtype;
 use pyo3::prelude::*;
 use pyo3::types::PyDict;
 
 use crate::conversion::Wrap;
 use crate::error::PyPolarsErr;
 use crate::file::{EitherRustPythonFile, get_either_file};
-use crate::prelude::ArrowDataType;
 
 #[cfg(feature = "ipc")]
 #[pyfunction]
-pub fn read_ipc_schema(py: Python, py_f: PyObject) -> PyResult<Bound<PyDict>> {
+pub fn read_ipc_schema(py: Python<'_>, py_f: PyObject) -> PyResult<Bound<'_, PyDict>> {
     use arrow::io::ipc::read::read_file_metadata;
     let metadata = match get_either_file(py_f, false)? {
         EitherRustPythonFile::Rust(r) => {
@@ -30,8 +27,9 @@ pub fn read_ipc_schema(py: Python, py_f: PyObject) -> PyResult<Bound<PyDict>> {
 
 #[cfg(feature = "parquet")]
 #[pyfunction]
-pub fn read_parquet_schema(py: Python, py_f: PyObject) -> PyResult<Bound<PyDict>> {
-    use polars_parquet::read::{infer_schema, read_metadata};
+pub fn read_parquet_metadata(py: Python, py_f: PyObject) -> PyResult<Bound<PyDict>> {
+    use polars_parquet::read::read_metadata;
+    use polars_parquet::read::schema::read_custom_key_value_metadata;
 
     let metadata = match get_either_file(py_f, false)? {
         EitherRustPythonFile::Rust(r) => {
@@ -39,23 +37,19 @@ pub fn read_parquet_schema(py: Python, py_f: PyObject) -> PyResult<Bound<PyDict>
         },
         EitherRustPythonFile::Py(mut r) => read_metadata(&mut r).map_err(PyPolarsErr::from)?,
     };
-    let arrow_schema = infer_schema(&metadata).map_err(PyPolarsErr::from)?;
 
+    let key_value_metadata = read_custom_key_value_metadata(metadata.key_value_metadata());
     let dict = PyDict::new(py);
-    fields_to_pydict(&arrow_schema, &dict)?;
+    for (key, value) in key_value_metadata.into_iter() {
+        dict.set_item(key.as_str(), value.as_str())?;
+    }
     Ok(dict)
 }
 
 #[cfg(any(feature = "ipc", feature = "parquet"))]
 fn fields_to_pydict(schema: &ArrowSchema, dict: &Bound<'_, PyDict>) -> PyResult<()> {
     for field in schema.iter_values() {
-        let dt = if field.is_enum() {
-            Wrap(create_enum_dtype(Utf8ViewArray::new_empty(
-                ArrowDataType::Utf8View,
-            )))
-        } else {
-            Wrap(polars::prelude::DataType::from_arrow_field(field))
-        };
+        let dt = Wrap(polars::prelude::DataType::from_arrow_field(field));
         dict.set_item(field.name.as_str(), &dt)?;
     }
     Ok(())
