@@ -2,6 +2,7 @@ use polars::lazy::dsl;
 use polars::prelude::*;
 use polars_plan::plans::DynLiteralValue;
 use polars_plan::prelude::UnionArgs;
+use polars_utils::python_function::PythonObject;
 use pyo3::exceptions::{PyTypeError, PyValueError};
 use pyo3::prelude::*;
 use pyo3::types::{PyBool, PyBytes, PyFloat, PyInt, PyString};
@@ -10,9 +11,8 @@ use crate::conversion::any_value::py_object_to_any_value;
 use crate::conversion::{Wrap, get_lf};
 use crate::error::PyPolarsErr;
 use crate::expr::ToExprs;
+use crate::expr::datatype::PyDataTypeExpr;
 use crate::lazyframe::PyOptFlags;
-use crate::map::lazy::binary_lambda;
-use crate::prelude::vec_extract_wrapped;
 use crate::utils::EnterPolarsExt;
 use crate::{PyDataFrame, PyExpr, PyLazyFrame, PySeries, map};
 
@@ -124,7 +124,7 @@ pub fn collect_all(
     lfs: Vec<PyLazyFrame>,
     engine: Wrap<Engine>,
     optflags: PyOptFlags,
-    py: Python,
+    py: Python<'_>,
 ) -> PyResult<Vec<PyDataFrame>> {
     let plans = lfs_to_plans(lfs);
     let dfs =
@@ -145,7 +145,7 @@ pub fn collect_all_with_callback(
     engine: Wrap<Engine>,
     optflags: PyOptFlags,
     lambda: PyObject,
-    py: Python,
+    py: Python<'_>,
 ) {
     let plans = lfs.into_iter().map(|lf| lf.ldf.logical_plan).collect();
     let result = py
@@ -169,11 +169,6 @@ pub fn collect_all_with_callback(
     })
 }
 
-#[pyfunction]
-pub fn cols(names: Vec<String>) -> PyExpr {
-    dsl::cols(names).into()
-}
-
 #[pyfunction]
 pub fn concat_lf(
     seq: &Bound<'_, PyAny>,
@@ -240,37 +235,42 @@ pub fn arctan2(y: PyExpr, x: PyExpr) -> PyExpr {
 }
 
 #[pyfunction]
-pub fn cum_fold(acc: PyExpr, lambda: PyObject, exprs: Vec<PyExpr>, include_init: bool) -> PyExpr {
+pub fn cum_fold(
+    acc: PyExpr,
+    lambda: PyObject,
+    exprs: Vec<PyExpr>,
+    returns_scalar: bool,
+    return_dtype: Option<PyDataTypeExpr>,
+    include_init: bool,
+) -> PyExpr {
     let exprs = exprs.to_exprs();
-
-    let func = move |a: Column, b: Column| {
-        binary_lambda(
-            &lambda,
-            a.take_materialized_series(),
-            b.take_materialized_series(),
-        )
-        .map(|v| v.map(Column::from))
-    };
-    dsl::cum_fold_exprs(acc.inner, func, exprs, include_init).into()
+    let func = PlanCallback::new_python(PythonObject(lambda));
+    dsl::cum_fold_exprs(
+        acc.inner,
+        func,
+        exprs,
+        returns_scalar,
+        return_dtype.map(|v| v.inner),
+        include_init,
+    )
+    .into()
 }
 
 #[pyfunction]
-pub fn cum_reduce(lambda: PyObject, exprs: Vec<PyExpr>) -> PyExpr {
+pub fn cum_reduce(
+    lambda: PyObject,
+    exprs: Vec<PyExpr>,
+    returns_scalar: bool,
+    return_dtype: Option<PyDataTypeExpr>,
+) -> PyExpr {
     let exprs = exprs.to_exprs();
 
-    let func = move |a: Column, b: Column| {
-        binary_lambda(
-            &lambda,
-            a.take_materialized_series(),
-            b.take_materialized_series(),
-        )
-        .map(|v| v.map(Column::from))
-    };
-    dsl::cum_reduce_exprs(func, exprs).into()
+    let func = PlanCallback::new_python(PythonObject(lambda));
+    dsl::cum_reduce_exprs(func, exprs, returns_scalar, return_dtype.map(|v| v.inner)).into()
 }
 
 #[pyfunction]
-#[pyo3(signature = (year, month, day, hour=None, minute=None, second=None, microsecond=None, time_unit=Wrap(TimeUnit::Microseconds), time_zone=None, ambiguous=PyExpr::from(dsl::lit(String::from("raise")))))]
+#[pyo3(signature = (year, month, day, hour=None, minute=None, second=None, microsecond=None, time_unit=Wrap(TimeUnit::Microseconds), time_zone=Wrap(None), ambiguous=PyExpr::from(dsl::lit(String::from("raise")))))]
 pub fn datetime(
     year: PyExpr,
     month: PyExpr,
@@ -280,7 +280,7 @@ pub fn datetime(
     second: Option<PyExpr>,
     microsecond: Option<PyExpr>,
     time_unit: Wrap<TimeUnit>,
-    time_zone: Option<Wrap<TimeZone>>,
+    time_zone: Wrap<Option<TimeZone>>,
     ambiguous: PyExpr,
 ) -> PyExpr {
     let year = year.inner;
@@ -289,7 +289,7 @@ pub fn datetime(
     set_unwrapped_or_0!(hour, minute, second, microsecond);
     let ambiguous = ambiguous.inner;
     let time_unit = time_unit.0;
-    let time_zone = time_zone.map(|x| x.0);
+    let time_zone = time_zone.0;
     let args = DatetimeArgs {
         year,
         month,
@@ -362,22 +362,6 @@ pub fn concat_expr(e: Vec<PyExpr>, rechunk: bool) -> PyResult<PyExpr> {
     Ok(e.into())
 }
 
-#[pyfunction]
-pub fn dtype_cols(dtypes: Vec<Wrap<DataType>>) -> PyResult<PyExpr> {
-    let dtypes = vec_extract_wrapped(dtypes);
-    Ok(dsl::dtype_cols(dtypes).into())
-}
-
-#[pyfunction]
-pub fn index_cols(indices: Vec<i64>) -> PyExpr {
-    if indices.len() == 1 {
-        dsl::nth(indices[0])
-    } else {
-        dsl::index_cols(indices)
-    }
-    .into()
-}
-
 #[pyfunction]
 #[pyo3(signature = (weeks, days, hours, minutes, seconds, milliseconds, microseconds, nanoseconds, time_unit))]
 pub fn duration(
@@ -415,49 +399,26 @@ pub fn duration(
     dsl::duration(args).into()
 }
 
-#[pyfunction]
-pub fn first() -> PyExpr {
-    dsl::first().into()
-}
-
 #[pyfunction]
 pub fn fold(
     acc: PyExpr,
     lambda: PyObject,
     exprs: Vec<PyExpr>,
     returns_scalar: bool,
-    return_dtype: Option<Wrap<DataType>>,
+    return_dtype: Option<PyDataTypeExpr>,
 ) -> PyExpr {
     let exprs = exprs.to_exprs();
-
-    let func = move |a: Column, b: Column| {
-        binary_lambda(
-            &lambda,
-            a.take_materialized_series(),
-            b.take_materialized_series(),
-        )
-        .map(|v| v.map(Column::from))
-    };
+    let func = PlanCallback::new_python(PythonObject(lambda));
     dsl::fold_exprs(
         acc.inner,
         func,
         exprs,
         returns_scalar,
-        return_dtype.map(|w| w.0),
+        return_dtype.map(|w| w.inner),
     )
     .into()
 }
 
-#[pyfunction]
-pub fn last() -> PyExpr {
-    dsl::last().into()
-}
-
-#[pyfunction]
-pub fn nth(n: i64) -> PyExpr {
-    dsl::nth(n).into()
-}
-
 #[pyfunction]
 pub fn lit(value: &Bound<'_, PyAny>, allow_object: bool, is_scalar: bool) -> PyResult<PyExpr> {
     let py = value.py();
@@ -514,7 +475,7 @@ pub fn lit(value: &Bound<'_, PyAny>, allow_object: bool, is_scalar: bool) -> PyR
 #[pyfunction]
 #[pyo3(signature = (pyexpr, lambda, output_type, map_groups, returns_scalar))]
 pub fn map_mul(
-    py: Python,
+    py: Python<'_>,
     pyexpr: Vec<PyExpr>,
     lambda: PyObject,
     output_type: Option<Wrap<DataType>>,
@@ -530,18 +491,15 @@ pub fn pearson_corr(a: PyExpr, b: PyExpr) -> PyExpr {
 }
 
 #[pyfunction]
-pub fn reduce(lambda: PyObject, exprs: Vec<PyExpr>) -> PyExpr {
+pub fn reduce(
+    lambda: PyObject,
+    exprs: Vec<PyExpr>,
+    returns_scalar: bool,
+    return_dtype: Option<PyDataTypeExpr>,
+) -> PyExpr {
     let exprs = exprs.to_exprs();
-
-    let func = move |a: Column, b: Column| {
-        binary_lambda(
-            &lambda,
-            a.take_materialized_series(),
-            b.take_materialized_series(),
-        )
-        .map(|v| v.map(Column::from))
-    };
-    dsl::reduce_exprs(func, exprs).into()
+    let func = PlanCallback::new_python(PythonObject(lambda));
+    dsl::reduce_exprs(func, exprs, returns_scalar, return_dtype.map(|v| v.inner)).into()
 }
 
 #[pyfunction]
