@@ -4,31 +4,39 @@ use polars_ops::series::ClosedInterval;
 use pyo3::prelude::*;
 
 use crate::error::PyPolarsErr;
+use crate::expr::datatype::PyDataTypeExpr;
 use crate::prelude::*;
 use crate::utils::EnterPolarsExt;
 use crate::{PyExpr, PySeries};
 
 #[pyfunction]
-pub fn int_range(start: PyExpr, end: PyExpr, step: i64, dtype: Wrap<DataType>) -> PyExpr {
+pub fn int_range(start: PyExpr, end: PyExpr, step: i64, dtype: PyDataTypeExpr) -> PyExpr {
     let start = start.inner;
     let end = end.inner;
-    let dtype = dtype.0;
+    let dtype = dtype.inner;
     dsl::int_range(start, end, step, dtype).into()
 }
 
 /// Eager version of `int_range` to avoid overhead from the expression engine.
 #[pyfunction]
 pub fn eager_int_range(
-    py: Python,
+    py: Python<'_>,
     lower: &Bound<'_, PyAny>,
     upper: &Bound<'_, PyAny>,
     step: &Bound<'_, PyAny>,
-    dtype: Wrap<DataType>,
+    dtype: PyDataTypeExpr,
 ) -> PyResult<PySeries> {
-    let dtype = dtype.0;
+    let dtype = dtype.inner;
+    let Some(dtype) = dtype.into_literal() else {
+        return Err(PyPolarsErr::from(
+            polars_err!(ComputeError: "eager `int_range` cannot be given lazy datatype expression"),
+        )
+        .into());
+    };
+
     if !dtype.is_integer() {
         return Err(PyPolarsErr::from(
-            polars_err!(ComputeError: "non-integer `dtype` passed to `int_range`: {:?}", dtype),
+            polars_err!(SchemaMismatch: "non-integer `dtype` passed to `int_range`: '{}'", dtype),
         )
         .into());
     }
@@ -46,23 +54,10 @@ pub fn int_ranges(
     start: PyExpr,
     end: PyExpr,
     step: PyExpr,
-    dtype: Wrap<DataType>,
+    dtype: PyDataTypeExpr,
 ) -> PyResult<PyExpr> {
-    let dtype = dtype.0;
-    if !dtype.is_integer() {
-        return Err(PyPolarsErr::from(
-            polars_err!(ComputeError: "non-integer `dtype` passed to `int_ranges`: {:?}", dtype),
-        )
-        .into());
-    }
-
-    let mut result = dsl::int_ranges(start.inner, end.inner, step.inner);
-
-    if dtype != DataType::Int64 {
-        result = result.cast(DataType::List(Box::new(dtype)))
-    }
-
-    Ok(result.into())
+    let dtype = dtype.inner;
+    Ok(dsl::int_ranges(start.inner, end.inner, step.inner, dtype).into())
 }
 
 #[pyfunction]
@@ -94,40 +89,40 @@ pub fn date_ranges(
 }
 
 #[pyfunction]
-#[pyo3(signature = (start, end, every, closed, time_unit=None, time_zone=None))]
+#[pyo3(signature = (start, end, every, closed, time_unit=None, time_zone=Wrap(None)))]
 pub fn datetime_range(
     start: PyExpr,
     end: PyExpr,
     every: &str,
     closed: Wrap<ClosedWindow>,
     time_unit: Option<Wrap<TimeUnit>>,
-    time_zone: Option<Wrap<TimeZone>>,
+    time_zone: Wrap<Option<TimeZone>>,
 ) -> PyResult<PyExpr> {
     let start = start.inner;
     let end = end.inner;
     let every = Duration::try_parse(every).map_err(PyPolarsErr::from)?;
     let closed = closed.0;
     let time_unit = time_unit.map(|x| x.0);
-    let time_zone = time_zone.map(|x| x.0);
+    let time_zone = time_zone.0;
     Ok(dsl::datetime_range(start, end, every, closed, time_unit, time_zone).into())
 }
 
 #[pyfunction]
-#[pyo3(signature = (start, end, every, closed, time_unit=None, time_zone=None))]
+#[pyo3(signature = (start, end, every, closed, time_unit=None, time_zone=Wrap(None)))]
 pub fn datetime_ranges(
     start: PyExpr,
     end: PyExpr,
     every: &str,
     closed: Wrap<ClosedWindow>,
     time_unit: Option<Wrap<TimeUnit>>,
-    time_zone: Option<Wrap<TimeZone>>,
+    time_zone: Wrap<Option<TimeZone>>,
 ) -> PyResult<PyExpr> {
     let start = start.inner;
     let end = end.inner;
     let every = Duration::try_parse(every).map_err(PyPolarsErr::from)?;
     let closed = closed.0;
     let time_unit = time_unit.map(|x| x.0);
-    let time_zone = time_zone.map(|x| x.0);
+    let time_zone = time_zone.0;
     Ok(dsl::datetime_ranges(start, end, every, closed, time_unit, time_zone).into())
 }
 
