@@ -12,15 +12,19 @@ use crate::utils::EnterPolarsExt;
 #[allow(clippy::should_implement_trait)]
 impl PyLazyFrame {
     /// Serialize into binary data.
-    fn serialize_binary(&self, py: Python, py_f: PyObject) -> PyResult<()> {
+    fn serialize_binary(&self, py: Python<'_>, py_f: PyObject) -> PyResult<()> {
         let file = get_file_like(py_f, true)?;
         let writer = BufWriter::new(file);
-        py.enter_polars(|| self.ldf.logical_plan.serialize_versioned(writer))
+        py.enter_polars(|| {
+            self.ldf
+                .logical_plan
+                .serialize_versioned(writer, Default::default())
+        })
     }
 
     /// Serialize into a JSON string.
     #[cfg(feature = "json")]
-    fn serialize_json(&self, py: Python, py_f: PyObject) -> PyResult<()> {
+    fn serialize_json(&self, py: Python<'_>, py_f: PyObject) -> PyResult<()> {
         let file = get_file_like(py_f, true)?;
         let writer = BufWriter::new(file);
         py.enter_polars(|| {
@@ -31,7 +35,7 @@ impl PyLazyFrame {
 
     /// Deserialize a file-like object containing binary data into a LazyFrame.
     #[staticmethod]
-    fn deserialize_binary(py: Python, py_f: PyObject) -> PyResult<Self> {
+    fn deserialize_binary(py: Python<'_>, py_f: PyObject) -> PyResult<Self> {
         let file = get_file_like(py_f, false)?;
         let reader = BufReader::new(file);
 
@@ -42,7 +46,7 @@ impl PyLazyFrame {
     /// Deserialize a file-like object containing JSON string data into a LazyFrame.
     #[staticmethod]
     #[cfg(feature = "json")]
-    fn deserialize_json(py: Python, py_f: PyObject) -> PyResult<Self> {
+    fn deserialize_json(py: Python<'_>, py_f: PyObject) -> PyResult<Self> {
         // it is faster to first read to memory and then parse: https://github.com/serde-rs/json/issues/160
         // so don't bother with files.
         let mut json = String::new();
