@@ -6,7 +6,6 @@ use polars::prelude::*;
 use polars_core::chunked_array::object::builder::ObjectChunkedBuilder;
 use polars_core::chunked_array::object::registry::AnonymousObjectBuilder;
 use polars_core::chunked_array::object::{registry, set_polars_allow_extension};
-use polars_core::error::PolarsError::ComputeError;
 use polars_error::PolarsWarning;
 use polars_error::signals::register_polars_keyboard_interrupt_hook;
 use polars_ffi::version_0::SeriesExport;
@@ -20,11 +19,16 @@ use crate::dataframe::PyDataFrame;
 use crate::map::lazy::{ToSeries, call_lambda_with_series};
 use crate::prelude::ObjectValue;
 use crate::py_modules::{pl_df, pl_utils, polars, polars_rs};
+use crate::series::PySeries;
 
-fn python_function_caller_series(s: Column, lambda: &PyObject) -> PolarsResult<Column> {
+fn python_function_caller_series(
+    s: Column,
+    output_dtype: Option<DataType>,
+    lambda: &PyObject,
+) -> PolarsResult<Column> {
     Python::with_gil(|py| {
-        let object = call_lambda_with_series(py, s.as_materialized_series(), lambda)
-            .map_err(|s| ComputeError(format!("{}", s).into()))?;
+        let object =
+            call_lambda_with_series(py, s.as_materialized_series(), Some(output_dtype), lambda)?;
         object.to_series(py, polars(py), s.name()).map(Column::from)
     })
 }
@@ -64,9 +68,7 @@ fn python_function_caller_df(df: DataFrame, lambda: &PyObject) -> PolarsResult<D
                 .unwrap();
         }
         // call the lambda and get a python side df wrapper
-        let result_df_wrapper = lambda.call1(py, (python_df_wrapper,)).map_err(|e| {
-            PolarsError::ComputeError(format!("User provided python function failed: {e}").into())
-        })?;
+        let result_df_wrapper = lambda.call1(py, (python_df_wrapper,))?;
 
         // unpack the wrapper in a PyDataFrame
         let py_pydf = result_df_wrapper.getattr(py, "_df").map_err(|_| {
@@ -132,12 +134,27 @@ pub unsafe fn register_startup_deps(catch_keyboard_interrupt: bool) {
 
         polars_utils::python_convert_registry::register_converters(PythonConvertRegistry {
             from_py: FromPythonConvertRegistry {
-                sink_target: Arc::new(|py_f| {
+                partition_target_cb_result: Arc::new(|py_f| {
                     Python::with_gil(|py| {
-                        Ok(
-                            Box::new(py_f.extract::<Wrap<polars_plan::dsl::SinkTarget>>(py)?.0)
-                                as _,
-                        )
+                        Ok(Box::new(
+                            py_f.extract::<Wrap<polars_plan::dsl::PartitionTargetCallbackResult>>(
+                                py,
+                            )?
+                            .0,
+                        ) as _)
+                    })
+                }),
+                series: Arc::new(|py_f| {
+                    Python::with_gil(|py| Ok(Box::new(py_f.extract::<PySeries>(py)?.series) as _))
+                }),
+            },
+            to_py: polars_utils::python_convert_registry::ToPythonConvertRegistry {
+                df: Arc::new(|df| {
+                    Python::with_gil(|py| PyDataFrame::new(*df.downcast().unwrap()).into_py_any(py))
+                }),
+                series: Arc::new(|series| {
+                    Python::with_gil(|py| {
+                        PySeries::new(*series.downcast().unwrap()).into_py_any(py)
                     })
                 }),
             },
