@@ -31,10 +31,11 @@ impl PySeries {
                 DataType::Int128 => PyList::new(py, series.i128().map_err(PyPolarsErr::from)?)?,
                 DataType::Float32 => PyList::new(py, series.f32().map_err(PyPolarsErr::from)?)?,
                 DataType::Float64 => PyList::new(py, series.f64().map_err(PyPolarsErr::from)?)?,
-                DataType::Categorical(_, _) | DataType::Enum(_, _) => PyList::new(
-                    py,
-                    series.categorical().map_err(PyPolarsErr::from)?.iter_str(),
-                )?,
+                DataType::Categorical(_, _) | DataType::Enum(_, _) => {
+                    with_match_categorical_physical_type!(series.dtype().cat_physical().unwrap(), |$C| {
+                        PyList::new(py, series.cat::<$C>().unwrap().iter_str())?
+                    })
+                },
                 #[cfg(feature = "object")]
                 DataType::Object(_) => {
                     let v = PyList::empty(py);
@@ -146,7 +147,7 @@ impl PySeries {
 
     /// Return the underlying Arrow array.
     #[allow(clippy::wrong_self_convention)]
-    fn to_arrow(&mut self, py: Python, compat_level: PyCompatLevel) -> PyResult<PyObject> {
+    fn to_arrow(&mut self, py: Python<'_>, compat_level: PyCompatLevel) -> PyResult<PyObject> {
         self.rechunk(py, true)?;
         let pyarrow = py.import("pyarrow")?;
 
@@ -160,14 +161,14 @@ impl PySeries {
     #[allow(unused_variables)]
     #[pyo3(signature = (requested_schema=None))]
     fn __arrow_c_stream__<'py>(
-        &'py self,
+        &self,
         py: Python<'py>,
         requested_schema: Option<PyObject>,
     ) -> PyResult<Bound<'py, PyCapsule>> {
         series_to_stream(&self.series, py)
     }
 
-    pub fn _export(&mut self, _py: Python, location: usize) {
+    pub fn _export(&mut self, _py: Python<'_>, location: usize) {
         let export = polars_ffi::version_0::export_series(&self.series);
         unsafe {
             (location as *mut SeriesExport).write(export);
