@@ -17,11 +17,13 @@ mod general;
 #[cfg(feature = "pymethods")]
 mod import;
 #[cfg(feature = "pymethods")]
+mod map;
+#[cfg(feature = "pymethods")]
 mod numpy_ufunc;
 #[cfg(feature = "pymethods")]
 mod scatter;
 
-use polars::prelude::Series;
+use polars::prelude::{Column, Series};
 use pyo3::pyclass;
 
 #[pyclass]
@@ -64,3 +66,14 @@ impl ToPySeries for Vec<Series> {
         unsafe { std::mem::transmute(self) }
     }
 }
+
+impl ToPySeries for Vec<Column> {
+    fn to_pyseries(self) -> Vec<PySeries> {
+        // @scalar-opt
+        let series: Vec<Series> = self
+            .into_iter()
+            .map(|c| c.take_materialized_series())
+            .collect();
+        series.to_pyseries()
+    }
+}
