@@ -7,7 +7,7 @@ use crate::utils::EnterPolarsExt;
 
 #[pymethods]
 impl PySeries {
-    fn scatter(&mut self, py: Python, idx: PySeries, values: PySeries) -> PyResult<()> {
+    fn scatter(&mut self, py: Python<'_>, idx: PySeries, values: PySeries) -> PyResult<()> {
         // we take the value because we want a ref count of 1 so that we can
         // have mutable access cheaply via _get_inner_mut().
         let s = std::mem::take(&mut self.series);
@@ -31,6 +31,25 @@ impl PySeries {
 fn scatter(mut s: Series, idx: &Series, values: &Series) -> Result<Series, (Series, PolarsError)> {
     let logical_dtype = s.dtype().clone();
 
+    let values = if logical_dtype.is_categorical() || logical_dtype.is_enum() {
+        if matches!(
+            values.dtype(),
+            DataType::Categorical(_, _) | DataType::Enum(_, _) | DataType::String
+        ) {
+            match values.strict_cast(&logical_dtype) {
+                Ok(values) => values,
+                Err(err) => return Err((s, err)),
+            }
+        } else {
+            return Err((
+                s,
+                polars_err!(InvalidOperation: "invalid values dtype '{}' for scattering into dtype '{}'", values.dtype(), logical_dtype),
+            ));
+        }
+    } else {
+        values.clone()
+    };
+
     let idx = match polars_ops::prelude::convert_to_unsigned_index(idx, s.len()) {
         Ok(idx) => idx,
         Err(err) => return Err((s, err)),
