@@ -688,7 +688,7 @@ impl SQLContext {
                         .iter_names()
                         .filter_map(|name| {
                             // col exists in both tables and is aliased in the joined result
-                            let aliased_name = format!("{}:{}", name, r_name);
+                            let aliased_name = format!("{name}:{r_name}");
                             if left_schema.contains(name)
                                 && joined_schema.contains(aliased_name.as_str())
                             {
@@ -913,7 +913,7 @@ impl SQLContext {
                     .map(|e| {
                         let expr = parse_sql_expr(e, self, schema.as_deref())?;
                         if let Expr::Column(name) = expr {
-                            Ok(name.clone())
+                            Ok(name)
                         } else {
                             Err(polars_err!(SQLSyntax:"DISTINCT ON only supports column names"))
                         }
@@ -922,7 +922,13 @@ impl SQLContext {
 
                 // DISTINCT ON has to apply the ORDER BY before the operation.
                 lf = self.process_order_by(lf, &query.order_by, None)?;
-                return Ok(lf.unique_stable(Some(cols.clone()), UniqueKeepStrategy::First));
+                return Ok(lf.unique_stable(
+                    Some(Selector::ByName {
+                        names: cols.into(),
+                        strict: true,
+                    }),
+                    UniqueKeepStrategy::First,
+                ));
             },
             None => lf,
         };
@@ -1177,9 +1183,9 @@ impl SQLContext {
                         .zip(column_names)
                         .map(|(s, name)| {
                             if let Some(name) = name {
-                                s.clone().with_name(name)
+                                s.with_name(name)
                             } else {
-                                s.clone()
+                                s
                             }
                         })
                         .map(Column::from)
@@ -1194,7 +1200,7 @@ impl SQLContext {
                     }
                     let table_name = alias.name.value.clone();
                     self.table_map.insert(table_name.clone(), lf.clone());
-                    Ok((table_name.clone(), lf))
+                    Ok((table_name, lf))
                 } else {
                     polars_bail!(SQLSyntax: "UNNEST table must have an alias");
                 }
@@ -1227,7 +1233,7 @@ impl SQLContext {
         let tbl_name = alias
             .as_ref()
             .map(|a| a.name.value.clone())
-            .unwrap_or_else(|| tbl_name);
+            .unwrap_or_else(|| tbl_name.to_str().to_string());
 
         self.table_map.insert(tbl_name.clone(), lf.clone());
         Ok((tbl_name, lf))
@@ -1311,9 +1317,11 @@ impl SQLContext {
                 matches!(e, Expr::Agg(_) | Expr::Len | Expr::Window { .. })
             });
 
+            let mut is_function_under_alias = false;
+
             // Note: if simple aliased expression we defer aliasing until after the group_by.
             if let Expr::Alias(expr, alias) = e {
-                if e.clone().meta().is_simple_projection() {
+                if e.clone().meta().is_simple_projection(Some(&schema_before)) {
                     group_key_aliases.insert(alias.as_ref());
                     e = expr
                 } else if let Expr::Function {
@@ -1323,6 +1331,8 @@ impl SQLContext {
                 {
                     projection_overrides
                         .insert(alias.as_ref(), col(name.clone()).alias(alias.clone()));
+                } else if let Expr::Function { .. } = expr.deref() {
+                    is_function_under_alias = true;
                 } else if !is_agg_or_window && !group_by_keys_schema.contains(alias) {
                     projection_aliases.insert(alias.as_ref());
                 }
@@ -1348,6 +1358,17 @@ impl SQLContext {
                 if !group_by_keys_schema.contains(&field.name) {
                     polars_bail!(SQLSyntax: "'{}' should participate in the GROUP BY clause or an aggregate function", &field.name);
                 }
+            } else if is_function_under_alias || matches!(e, Expr::Function { .. }) {
+                aggregation_projection.push(e.clone());
+            } else if let Expr::Literal { .. }
+            | Expr::Cast { .. }
+            | Expr::Ternary { .. }
+            | Expr::Field { .. }
+            | Expr::Alias { .. } = e
+            {
+                // do nothing
+            } else {
+                polars_bail!(SQLSyntax: "Unsupported operation in the GROUP BY clause: {}", e);
             }
         }
         let aggregated = lf.group_by(group_by_keys).agg(&aggregation_projection);
@@ -1476,8 +1497,7 @@ impl SQLContext {
                 .replace('_', ".");
 
             modifiers.ilike = Some(
-                polars_utils::regex_cache::compile_regex(format!("^(?is){}$", rx).as_str())
-                    .unwrap(),
+                polars_utils::regex_cache::compile_regex(format!("^(?is){rx}$").as_str()).unwrap(),
             );
         }
 
@@ -1570,10 +1590,6 @@ fn collect_compound_identifiers(
 
 fn expand_exprs(expr: Expr, schema: &SchemaRef) -> Vec<Expr> {
     match expr {
-        Expr::Wildcard => schema
-            .iter_names()
-            .map(|name| col(name.clone()))
-            .collect::<Vec<_>>(),
         Expr::Column(nm) if is_regex_colname(nm.as_str()) => {
             let re = polars_utils::regex_cache::compile_regex(&nm).unwrap();
             schema
@@ -1582,9 +1598,11 @@ fn expand_exprs(expr: Expr, schema: &SchemaRef) -> Vec<Expr> {
                 .map(|name| col(name.clone()))
                 .collect::<Vec<_>>()
         },
-        Expr::Columns(names) => names
-            .iter()
-            .map(|name| col(name.clone()))
+        Expr::Selector(s) => s
+            .into_columns(schema, &Default::default())
+            .unwrap()
+            .into_iter()
+            .map(col)
             .collect::<Vec<_>>(),
         _ => vec![expr],
     }
@@ -1683,16 +1701,17 @@ impl ExprSqlProjectionHeightBehavior {
         for e in expr.into_iter() {
             use Expr::*;
 
-            has_column |= matches!(e, Column(_) | Columns(_) | DtypeColumn(_) | IndexColumn(_));
+            has_column |= matches!(e, Column(_) | Selector(_));
 
             has_independent |= match e {
-                Function { options, .. } | AnonymousFunction { options, .. } => {
+                // @TODO: This is broken now with functions.
+                AnonymousFunction { options, .. } => {
                     options.returns_scalar() || !options.is_length_preserving()
                 },
 
                 Literal(v) => !v.is_scalar(),
 
-                Explode(_) | Filter { .. } | Gather { .. } | Slice { .. } => true,
+                Explode { .. } | Filter { .. } | Gather { .. } | Slice { .. } => true,
 
                 Agg { .. } | Len => true,
 
