@@ -1,14 +1,26 @@
-use polars_core::frame::DataFrame;
+use std::sync::Arc;
+
 use polars_core::POOL;
+use polars_core::frame::DataFrame;
 use polars_error::PolarsResult;
 use polars_expr::state::ExecutionState;
 use polars_utils::aliases::PlHashSet;
+use polars_utils::relaxed_cell::RelaxedCell;
 use slotmap::{SecondaryMap, SparseSecondaryMap};
 
 use crate::async_executor;
 use crate::graph::{Graph, GraphNode, GraphNodeKey, LogicalPipeKey, PortState};
 use crate::pipe::PhysicalPipe;
 
+#[derive(Clone)]
+pub struct StreamingExecutionState {
+    // The number of parallel pipelines we have within each stream.
+    pub num_pipelines: usize,
+
+    // The ExecutionState passed to any non-streaming operations.
+    pub in_memory_exec_state: ExecutionState,
+}
+
 /// Finds all runnable pipeline blockers in the graph, that is, nodes which:
 ///  - Only have blocked output ports.
 ///  - Have at least one ready input port connected to a ready output port.
@@ -77,12 +89,12 @@ fn find_runnable_subgraph(graph: &mut Graph) -> (PlHashSet<GraphNodeKey>, Vec<Lo
     // TODO: choose which expensive pipeline blocker to run more intelligently.
     expensive.sort_by_key(|node_key| {
         // Prefer to run nodes whose outputs are ready to be consumed.
-        let outputs_ready_to_receive = graph.nodes[*node_key]
+        // outputs_ready_to_receive
+        graph.nodes[*node_key]
             .outputs
             .iter()
             .filter(|o| graph.pipes[**o].recv_state == PortState::Ready)
-            .count();
-        outputs_ready_to_receive
+            .count()
     });
 
     let mut to_run = cheap;
@@ -104,12 +116,18 @@ fn run_subgraph(
     graph: &mut Graph,
     nodes: &PlHashSet<GraphNodeKey>,
     pipes: &[LogicalPipeKey],
-    num_pipelines: usize,
+    pipe_seq_offsets: &mut SecondaryMap<LogicalPipeKey, Arc<RelaxedCell<u64>>>,
+    state: &StreamingExecutionState,
 ) -> PolarsResult<()> {
     // Construct physical pipes for the logical pipes we'll use.
     let mut physical_pipes = SecondaryMap::new();
     for pipe_key in pipes.iter().copied() {
-        physical_pipes.insert(pipe_key, PhysicalPipe::new(num_pipelines));
+        let seq_offset = pipe_seq_offsets
+            .entry(pipe_key)
+            .unwrap()
+            .or_default()
+            .clone();
+        physical_pipes.insert(pipe_key, PhysicalPipe::new(state.num_pipelines, seq_offset));
     }
 
     // We do a topological sort of the graph: we want to spawn each node,
@@ -131,7 +149,6 @@ fn run_subgraph(
         }
     }
 
-    let execution_state = ExecutionState::default();
     async_executor::task_scope(|scope| {
         // Using SlotMap::iter_mut we can get simultaneous mutable references. By storing them and
         // removing the references from the secondary map as we do our topological sort we ensure
@@ -170,7 +187,7 @@ fn run_subgraph(
                 scope,
                 &mut recv_ports[..],
                 &mut send_ports[..],
-                &execution_state,
+                state,
                 &mut join_handles,
             );
 
@@ -247,15 +264,29 @@ pub fn execute_graph(
     let num_pipelines = POOL.current_num_threads();
     async_executor::set_num_threads(num_pipelines);
 
-    for node in graph.nodes.values_mut() {
-        node.compute.initialize(num_pipelines);
+    let state = StreamingExecutionState {
+        num_pipelines,
+        in_memory_exec_state: ExecutionState::default(),
+    };
+
+    // Ensure everything is properly connected.
+    for (node_key, node) in &graph.nodes {
+        for (i, input) in node.inputs.iter().enumerate() {
+            assert!(graph.pipes[*input].receiver == node_key);
+            assert!(graph.pipes[*input].recv_port == i);
+        }
+        for (i, output) in node.outputs.iter().enumerate() {
+            assert!(graph.pipes[*output].sender == node_key);
+            assert!(graph.pipes[*output].send_port == i);
+        }
     }
 
+    let mut pipe_seq_offsets = SecondaryMap::new();
     loop {
         if polars_core::config::verbose() {
             eprintln!("polars-stream: updating graph state");
         }
-        graph.update_all_states()?;
+        graph.update_all_states(&state)?;
         let (nodes, pipes) = find_runnable_subgraph(graph);
         if polars_core::config::verbose() {
             for node in &nodes {
@@ -268,7 +299,7 @@ pub fn execute_graph(
         if nodes.is_empty() {
             break;
         }
-        run_subgraph(graph, &nodes, &pipes, num_pipelines)?;
+        run_subgraph(graph, &nodes, &pipes, &mut pipe_seq_offsets, &state)?;
         if polars_core::config::verbose() {
             eprintln!("polars-stream: done running graph phase");
         }
