@@ -44,7 +44,7 @@ impl CsvSinkNode {
 
 impl SinkNode for CsvSinkNode {
     fn name(&self) -> &str {
-        "csv_sink"
+        "csv-sink"
     }
 
     fn is_sink_input_parallel(&self) -> bool {
@@ -96,6 +96,7 @@ impl SinkNode for CsvSinkNode {
                             .with_time_format(options.serialize_options.time_format.clone())
                             .with_float_scientific(options.serialize_options.float_scientific)
                             .with_float_precision(options.serialize_options.float_precision)
+                            .with_decimal_comma(options.serialize_options.decimal_comma)
                             .with_null_value(options.serialize_options.null.clone())
                             .with_quote_style(options.serialize_options.quote_style)
                             .n_threads(1) // Disable rayon parallelism
@@ -144,6 +145,7 @@ impl SinkNode for CsvSinkNode {
                     .with_time_format(options.serialize_options.time_format.clone())
                     .with_float_scientific(options.serialize_options.float_scientific)
                     .with_float_precision(options.serialize_options.float_precision)
+                    .with_decimal_comma(options.serialize_options.decimal_comma)
                     .with_null_value(options.serialize_options.null.clone())
                     .with_quote_style(options.serialize_options.quote_style)
                     .n_threads(1) // Disable rayon parallelism
