@@ -1,23 +1,25 @@
 use std::cmp::Reverse;
 use std::io::BufWriter;
-use std::sync::Mutex;
+use std::sync::{Arc, Mutex};
 
 use polars_core::prelude::{ArrowSchema, CompatLevel};
 use polars_core::schema::SchemaRef;
 use polars_error::PolarsResult;
 use polars_io::cloud::CloudOptions;
 use polars_io::parquet::write::BatchedWriter;
-use polars_io::prelude::{ParquetWriteOptions, get_encodings};
+use polars_io::prelude::{ParquetWriteOptions, get_column_write_options};
 use polars_io::schema_to_arrow_checked;
 use polars_parquet::parquet::error::ParquetResult;
 use polars_parquet::read::ParquetError;
 use polars_parquet::write::{
-    CompressedPage, Compressor, Encoding, FileWriter, SchemaDescriptor, Version, WriteOptions,
-    array_to_columns, to_parquet_schema,
+    ColumnWriteOptions, CompressedPage, Compressor, FileWriter, SchemaDescriptor, Version,
+    WriteOptions, array_to_columns, to_parquet_schema,
 };
 use polars_plan::dsl::{SinkOptions, SinkTarget};
 use polars_utils::priority::Priority;
+use polars_utils::relaxed_cell::RelaxedCell;
 
+use super::metrics::WriteMetrics;
 use super::{
     DEFAULT_SINK_DISTRIBUTOR_BUFFER_SIZE, DEFAULT_SINK_LINEARIZER_BUFFER_SIZE, SinkInputPort,
     SinkNode, buffer_and_distribute_columns_task,
@@ -39,8 +41,11 @@ pub struct ParquetSinkNode {
 
     parquet_schema: SchemaDescriptor,
     arrow_schema: ArrowSchema,
-    encodings: Vec<Vec<Encoding>>,
+    column_options: Vec<ColumnWriteOptions>,
     cloud_options: Option<CloudOptions>,
+
+    file_size: Arc<RelaxedCell<u64>>,
+    metrics: Arc<Mutex<Option<WriteMetrics>>>,
 }
 
 impl ParquetSinkNode {
@@ -50,22 +55,31 @@ impl ParquetSinkNode {
         sink_options: SinkOptions,
         write_options: &ParquetWriteOptions,
         cloud_options: Option<CloudOptions>,
+        collect_metrics: bool,
     ) -> PolarsResult<Self> {
         let schema = schema_to_arrow_checked(&input_schema, CompatLevel::newest(), "parquet")?;
-        let parquet_schema = to_parquet_schema(&schema)?;
-        let encodings: Vec<Vec<Encoding>> = get_encodings(&schema);
+        let column_options: Vec<ColumnWriteOptions> =
+            get_column_write_options(&schema, &write_options.field_overwrites);
+        let parquet_schema = to_parquet_schema(&schema, &column_options)?;
+        let metrics =
+            Arc::new(Mutex::new(collect_metrics.then(|| {
+                WriteMetrics::new(target.to_display_string(), &input_schema)
+            })));
 
         Ok(Self {
             target,
 
             input_schema,
             sink_options,
-            write_options: *write_options,
+            write_options: write_options.clone(),
 
             parquet_schema,
             arrow_schema: schema,
-            encodings,
+            column_options,
             cloud_options,
+
+            file_size: Arc::default(),
+            metrics,
         })
     }
 }
@@ -100,7 +114,7 @@ impl SinkNode for ParquetSinkNode {
         // Collect task -> IO task
         let (mut io_tx, mut io_rx) = connector::<Vec<Vec<CompressedPage>>>();
 
-        let write_options = self.write_options;
+        let write_options = &self.write_options;
 
         let options = WriteOptions {
             statistics: write_options.statistics,
@@ -117,6 +131,7 @@ impl SinkNode for ParquetSinkNode {
                 .row_group_size
                 .unwrap_or(DEFAULT_ROW_GROUP_SIZE),
             self.input_schema.clone(),
+            self.metrics.clone(),
         ));
 
         // Encode task.
@@ -128,12 +143,12 @@ impl SinkNode for ParquetSinkNode {
                 .zip(lin_txs)
                 .map(|(mut dist_rx, mut lin_tx)| {
                     let parquet_schema = self.parquet_schema.clone();
-                    let encodings = self.encodings.clone();
+                    let column_options = self.column_options.clone();
 
                     spawn(TaskPriority::High, async move {
                         while let Ok((rg_idx, col_idx, column)) = dist_rx.recv().await {
                             let type_ = &parquet_schema.fields()[col_idx];
-                            let encodings = &encodings[col_idx];
+                            let column_options = &column_options[col_idx];
 
                             let array = column.as_materialized_series().rechunk();
                             let array = array.to_arrow(0, CompatLevel::newest());
@@ -146,7 +161,7 @@ impl SinkNode for ParquetSinkNode {
 
                             // Array -> Parquet pages.
                             let encoded_columns =
-                                array_to_columns(array, type_.clone(), options, encodings)?;
+                                array_to_columns(array, type_.clone(), column_options, options)?;
 
                             // Compress the pages.
                             let compressed_pages = encoded_columns
@@ -236,16 +251,18 @@ impl SinkNode for ParquetSinkNode {
         let target = self.target.clone();
         let sink_options = self.sink_options.clone();
         let cloud_options = self.cloud_options.clone();
-        let write_options = self.write_options;
+        let write_options = self.write_options.clone();
         let arrow_schema = self.arrow_schema.clone();
         let parquet_schema = self.parquet_schema.clone();
-        let encodings = self.encodings.clone();
+        let column_options = self.column_options.clone();
+        let output_file_size = self.file_size.clone();
         let io_task = polars_io::pl_async::get_runtime().spawn(async move {
             let mut file = target
                 .open_into_writeable_async(&sink_options, cloud_options.as_ref())
                 .await?;
 
             let writer = BufWriter::new(&mut *file);
+            let key_value_metadata = write_options.key_value_metadata;
             let write_options = WriteOptions {
                 statistics: write_options.statistics,
                 compression: write_options.compression.into(),
@@ -258,7 +275,13 @@ impl SinkNode for ParquetSinkNode {
                 parquet_schema,
                 write_options,
             ));
-            let mut writer = BatchedWriter::new(file_writer, encodings, write_options, false);
+            let mut writer = BatchedWriter::new(
+                file_writer,
+                column_options,
+                write_options,
+                false,
+                key_value_metadata,
+            );
 
             let num_parquet_columns = writer.parquet_schema().leaves().len();
             while let Ok(current_row_group) = io_rx.recv().await {
@@ -268,12 +291,13 @@ impl SinkNode for ParquetSinkNode {
                 writer.write_row_group(&current_row_group)?;
             }
 
-            writer.finish()?;
+            let file_size = writer.finish()?;
             drop(writer);
 
             file.sync_on_close(sink_options.sync_on_close)?;
             file.close()?;
 
+            output_file_size.store(file_size);
             PolarsResult::Ok(())
         });
         join_handles.push(spawn(TaskPriority::Low, async move {
@@ -282,4 +306,14 @@ impl SinkNode for ParquetSinkNode {
                 .unwrap_or_else(|e| Err(std::io::Error::from(e).into()))
         }));
     }
+
+    fn get_metrics(&self) -> PolarsResult<Option<WriteMetrics>> {
+        let file_size = self.file_size.load();
+        let metrics = self.metrics.lock().unwrap().take();
+
+        Ok(metrics.map(|mut m| {
+            m.file_size = file_size;
+            m
+        }))
+    }
 }
