@@ -2,11 +2,9 @@ use std::ops::Range;
 use std::sync::Arc;
 
 use async_trait::async_trait;
-#[cfg(feature = "dtype-categorical")]
-use polars_core::StringCacheHolder;
 use polars_core::prelude::{Column, Field};
 use polars_core::schema::{SchemaExt, SchemaRef};
-use polars_error::{PolarsResult, polars_bail, polars_err};
+use polars_error::{PolarsResult, polars_bail, polars_err, polars_warn};
 use polars_io::RowIndex;
 use polars_io::cloud::CloudOptions;
 use polars_io::prelude::_csv_read_internal::{
@@ -31,6 +29,7 @@ use crate::async_executor::{AbortOnDropHandle, spawn};
 use crate::async_primitives::distributor_channel::{self, distributor_channel};
 use crate::morsel::SourceToken;
 use crate::nodes::compute_node_prelude::*;
+use crate::nodes::io_sources::multi_file_reader::reader_interface::Projection;
 use crate::nodes::io_sources::multi_file_reader::reader_interface::output::FileReaderOutputSend;
 use crate::nodes::{MorselSeq, TaskPriority};
 
@@ -134,7 +133,7 @@ impl FileReader for CsvFileReader {
         let memslice = self.get_bytes_maybe_decompress()?;
 
         let BeginReadArgs {
-            projected_schema,
+            projection: Projection::Plain(projected_schema),
             // Because we currently only support PRE_SLICE we don't need to handle row index here.
             row_index,
             pre_slice,
@@ -153,11 +152,17 @@ impl FileReader for CsvFileReader {
         };
 
         match &pre_slice {
+            Some(Slice::Negative { .. }) => unimplemented!(),
+
             // We don't account for comments when slicing lines. We should never hit this panic -
             // the FileReaderBuilder does not indicate PRE_SLICE support when we have a comment
             // prefix.
-            Some(..) if self.options.parse_options.comment_prefix.is_some() => panic!(),
-            Some(Slice::Negative { .. }) => unimplemented!(),
+            Some(pre_slice)
+                if self.options.parse_options.comment_prefix.is_some() && pre_slice.len() > 0 =>
+            {
+                panic!("{pre_slice:?}")
+            },
+
             _ => {},
         }
 
@@ -183,7 +188,10 @@ impl FileReader for CsvFileReader {
         )?;
 
         if let Some(schema) = &self.options.schema {
-            if schema.len() != inferred_schema.len()
+            // Note: User can provide schema with more columns, they will simply
+            // be projected as NULL.
+            // TODO: Should maybe expose a missing_columns parameter to the API for this.
+            if schema.len() < inferred_schema.len()
                 && !self.options.parse_options.truncate_ragged_lines
             {
                 polars_bail!(
@@ -337,8 +345,7 @@ impl FileReader for CsvFileReader {
                     if needs_full_row_count {
                         if verbose {
                             eprintln!(
-                                "[CSV LineBatchProcessor {}]: entering row count mode",
-                                worker_idx
+                                "[CSV LineBatchProcessor {worker_idx}]: entering row count mode"
                             );
                         }
 
@@ -582,8 +589,6 @@ struct ChunkReader {
     reader_schema: SchemaRef,
     parse_options: Arc<CsvParseOptions>,
     fields_to_cast: Vec<Field>,
-    #[cfg(feature = "dtype-categorical")]
-    _cat_lock: Option<StringCacheHolder>,
     ignore_errors: bool,
     projection: Vec<usize>,
     null_values: Option<NullValuesCompiled>,
@@ -602,10 +607,7 @@ impl ChunkReader {
         alt_count_lines: Option<Arc<CountLinesWithComments>>,
     ) -> PolarsResult<Self> {
         let mut fields_to_cast: Vec<Field> = options.fields_to_cast.clone();
-        let has_categorical = prepare_csv_schema(&mut reader_schema, &mut fields_to_cast)?;
-
-        #[cfg(feature = "dtype-categorical")]
-        let _cat_lock = has_categorical.then(polars_core::StringCacheHolder::hold);
+        prepare_csv_schema(&mut reader_schema, &mut fields_to_cast)?;
 
         let parse_options = options.parse_options.clone();
 
@@ -624,8 +626,6 @@ impl ChunkReader {
             reader_schema,
             parse_options,
             fields_to_cast,
-            #[cfg(feature = "dtype-categorical")]
-            _cat_lock,
             ignore_errors: options.ignore_errors,
             projection,
             null_values,
@@ -676,9 +676,28 @@ impl ChunkReader {
         let height = df.height();
         let n_lines_is_correct = df.height() == n_lines;
 
+        // Check malformed
+        if df.height() > n_lines
+            || (df.height() < n_lines && self.parse_options.comment_prefix.is_none())
+        {
+            // Note: in case data is malformed, df.height() is more likely to be correct than n_lines.
+            let msg = format!(
+                "CSV malformed: expected {} rows, actual {} rows, in chunk starting at row_offset {}, length {}",
+                n_lines,
+                df.height(),
+                chunk_row_offset,
+                chunk.len()
+            );
+            if self.ignore_errors {
+                polars_warn!(msg);
+            } else {
+                polars_bail!(ComputeError: msg);
+            }
+        }
+
         if slice != NO_SLICE {
             assert!(slice != SLICE_ENDED);
-            assert!(n_lines_is_correct);
+            assert!(n_lines_is_correct || slice.1 == 0);
 
             df = df.slice(i64::try_from(slice.0).unwrap(), slice.1);
         }
