@@ -4,8 +4,8 @@ pub mod initialization;
 pub mod post_apply_pipeline;
 pub mod reader_interface;
 pub mod reader_pipelines;
+pub mod row_counter;
 
-use std::sync::atomic::AtomicUsize;
 use std::sync::{Arc, Mutex};
 
 use bridge::BridgeState;
@@ -15,12 +15,15 @@ use polars_error::PolarsResult;
 use polars_io::cloud::CloudOptions;
 use polars_io::predicates::ScanIOPredicate;
 use polars_io::{RowIndex, pl_async};
-use polars_plan::dsl::{CastColumnsPolicy, ExtraColumnsPolicy, MissingColumnsPolicy, ScanSources};
+use polars_plan::dsl::deletion::DeletionFilesList;
+use polars_plan::dsl::{CastColumnsPolicy, MissingColumnsPolicy, ScanSources};
 use polars_plan::plans::hive::HivePartitionsDf;
 use polars_utils::format_pl_smallstr;
 use polars_utils::pl_str::PlSmallStr;
+use polars_utils::relaxed_cell::RelaxedCell;
 use polars_utils::slice_enum::Slice;
 use reader_interface::builder::FileReaderBuilder;
+use reader_interface::capabilities::ReaderCapabilities;
 
 use crate::async_executor::{self, AbortOnDropHandle, TaskPriority};
 use crate::async_primitives::connector;
@@ -29,8 +32,8 @@ use crate::execute::StreamingExecutionState;
 use crate::graph::PortState;
 use crate::morsel::Morsel;
 use crate::nodes::ComputeNode;
-
-pub const DEFAULT_N_READERS_PRE_INIT: usize = 3;
+use crate::nodes::io_sources::multi_file_reader::extra_ops::ForbidExtraColumns;
+use crate::nodes::io_sources::multi_file_reader::initialization::projection::ProjectionBuilder;
 
 // Some parts are called MultiFileReader for now to avoid conflict with existing MultiScan.
 
@@ -42,9 +45,7 @@ pub struct MultiFileReaderConfig {
     /// Final output schema of MultiScan node. Includes all e.g. row index / missing columns / file paths / hive etc.
     pub final_output_schema: SchemaRef,
     /// Columns to be projected from the file.
-    pub projected_file_schema: SchemaRef,
-    /// Full schema of the file. Used for complaining about extra columns.
-    pub full_file_schema: SchemaRef,
+    pub file_projection_builder: ProjectionBuilder,
 
     pub row_index: Option<RowIndex>,
     pub pre_slice: Option<Slice>,
@@ -53,21 +54,38 @@ pub struct MultiFileReaderConfig {
     pub hive_parts: Option<Arc<HivePartitionsDf>>,
     pub include_file_paths: Option<PlSmallStr>,
     pub missing_columns_policy: MissingColumnsPolicy,
-    pub extra_columns_policy: ExtraColumnsPolicy,
     pub cast_columns_policy: CastColumnsPolicy,
+    pub forbid_extra_columns: Option<ForbidExtraColumns>,
+    pub deletion_files: Option<DeletionFilesList>,
 
-    pub num_pipelines: AtomicUsize,
+    pub num_pipelines: RelaxedCell<usize>,
     /// Number of readers to initialize concurrently. e.g. Parquet will want to fetch metadata in this
     /// step.
-    pub n_readers_pre_init: usize,
+    pub n_readers_pre_init: RelaxedCell<usize>,
+    pub max_concurrent_scans: RelaxedCell<usize>,
 
     pub verbose: bool,
 }
 
 impl MultiFileReaderConfig {
     fn num_pipelines(&self) -> usize {
-        self.num_pipelines
-            .load(std::sync::atomic::Ordering::Relaxed)
+        self.num_pipelines.load()
+    }
+
+    fn n_readers_pre_init(&self) -> usize {
+        self.n_readers_pre_init.load()
+    }
+
+    fn max_concurrent_scans(&self) -> usize {
+        self.max_concurrent_scans.load()
+    }
+
+    fn reader_capabilities(&self) -> ReaderCapabilities {
+        if std::env::var("POLARS_FORCE_EMPTY_READER_CAPABILITIES").as_deref() == Ok("1") {
+            ReaderCapabilities::empty()
+        } else {
+            self.file_reader_builder.reader_capabilities()
+        }
     }
 }
 
@@ -148,19 +166,18 @@ impl ComputeNode for MultiFileReader {
         scope: &'s crate::async_executor::TaskScope<'s, 'env>,
         recv_ports: &mut [Option<crate::pipe::RecvPort<'_>>],
         send_ports: &mut [Option<crate::pipe::SendPort<'_>>],
-        state: &StreamingExecutionState,
+        state: &'s StreamingExecutionState,
         join_handles: &mut Vec<crate::async_executor::JoinHandle<polars_error::PolarsResult<()>>>,
     ) {
         assert!(recv_ports.is_empty() && send_ports.len() == 1);
 
         let phase_morsel_tx = send_ports[0].take().unwrap().serial();
-        let num_pipelines = state.num_pipelines;
         let verbose = self.verbose;
 
         join_handles.push(scope.spawn_task(TaskPriority::Low, async move {
             use MultiScanState::*;
 
-            self.state.initialize(num_pipelines);
+            self.state.initialize(state);
             self.state.refresh(verbose).await?;
 
             match &mut self.state {
@@ -256,7 +273,7 @@ impl MultiScanState {
     }
 
     /// Initialize state if not yet initialized.
-    fn initialize(&mut self, num_pipelines: usize) {
+    fn initialize(&mut self, execution_state: &StreamingExecutionState) {
         use MultiScanState::*;
 
         let slf = std::mem::replace(self, Finished);
@@ -267,8 +284,20 @@ impl MultiScanState {
         };
 
         config
-            .num_pipelines
-            .store(num_pipelines, std::sync::atomic::Ordering::Relaxed);
+            .file_reader_builder
+            .set_execution_state(execution_state);
+
+        let num_pipelines = execution_state.num_pipelines;
+
+        config.num_pipelines.store(num_pipelines);
+
+        config
+            .n_readers_pre_init
+            .store(calc_n_readers_pre_init(num_pipelines, &config));
+
+        config
+            .max_concurrent_scans
+            .store(calc_max_concurrent_scans(num_pipelines, &config));
 
         let (join_handle, send_phase_tx_to_bridge, bridge_state) =
             MultiScanTaskInitializer::new(config).spawn_background_tasks();
@@ -283,3 +312,40 @@ impl MultiScanState {
         };
     }
 }
+
+fn calc_n_readers_pre_init(num_pipelines: usize, config: &MultiFileReaderConfig) -> usize {
+    if let Ok(v) = std::env::var("POLARS_NUM_READERS_PRE_INIT").map(|x| {
+        x.parse::<usize>()
+            .ok()
+            .filter(|x| *x > 0)
+            .unwrap_or_else(|| panic!("invalid value for POLARS_NUM_READERS_PRE_INIT: {x}"))
+    }) {
+        return v;
+    }
+
+    let max_files_with_slice = match &config.pre_slice {
+        // Calculate the max number of files assuming 1 row per file.
+        Some(v @ Slice::Positive { .. }) => v.end_position().max(1),
+        Some(Slice::Negative { .. }) | None => usize::MAX,
+    };
+
+    // Set this generously high, there are users who scan 10,000's of small files from the cloud.
+    num_pipelines
+        .saturating_add(3)
+        .min(max_files_with_slice)
+        .min(config.sources.len().max(1))
+        .min(128)
+}
+
+fn calc_max_concurrent_scans(num_pipelines: usize, config: &MultiFileReaderConfig) -> usize {
+    if let Ok(v) = std::env::var("POLARS_MAX_CONCURRENT_SCANS").map(|x| {
+        x.parse::<usize>()
+            .ok()
+            .filter(|x| *x > 0)
+            .unwrap_or_else(|| panic!("invalid value for POLARS_MAX_CONCURRENT_SCANS: {x}"))
+    }) {
+        return v;
+    }
+
+    num_pipelines.min(config.sources.len().max(1)).min(128)
+}
