@@ -2,38 +2,37 @@ use std::ops::Deref;
 use std::sync::Arc;
 
 use polars_core::frame::DataFrame;
-use polars_core::prelude::{
-    ArrowField, ArrowSchema, BooleanChunked, ChunkFilter, Column, DataType, IdxCa, IntoColumn,
-};
-use polars_core::series::{IsSorted, Series};
+use polars_core::prelude::{ArrowField, BooleanChunked, ChunkFilter, Column, DataType, IntoColumn};
+use polars_core::series::Series;
 use polars_core::utils::arrow::bitmap::{Bitmap, MutableBitmap};
-use polars_error::{PolarsResult, polars_bail};
-use polars_io::hive;
+use polars_error::PolarsResult;
+use polars_io::RowIndex;
 use polars_io::predicates::{
-    ColumnPredicateExpr, ColumnPredicates, ScanIOPredicate, SpecializedColumnPredicateExpr,
+    ColumnPredicateExpr, ColumnPredicates, ScanIOPredicate, SpecializedColumnPredicate,
 };
 pub use polars_io::prelude::_internal::PrefilterMaskSetting;
 use polars_io::prelude::_internal::calc_prefilter_cost;
 use polars_io::prelude::try_set_sorted_flag;
 use polars_parquet::read::{Filter, ParquetType, PredicateFilter, PrimitiveLogicalType};
 use polars_utils::IdxSize;
-use polars_utils::index::AtomicIdxSize;
 use polars_utils::pl_str::PlSmallStr;
 
 use super::row_group_data_fetch::RowGroupData;
 use crate::async_primitives::opt_spawned_future::parallelize_first_to_local;
+use crate::nodes::io_sources::parquet::projection::ArrowFieldProjection;
 
 /// Turns row group data into DataFrames.
 pub(super) struct RowGroupDecoder {
     pub(super) num_pipelines: usize,
-    pub(super) projected_arrow_schema: Arc<ArrowSchema>,
-    pub(super) row_index: Option<Arc<(PlSmallStr, AtomicIdxSize)>>,
+    pub(super) projected_arrow_fields: Arc<[ArrowFieldProjection]>,
+    pub(super) allow_column_predicates: bool,
+    pub(super) row_index: Option<RowIndex>,
     pub(super) predicate: Option<ScanIOPredicate>,
     pub(super) use_prefiltered: Option<PrefilterMaskSetting>,
-    /// Indices into `projected_arrow_schema. This must be sorted.
-    pub(super) predicate_arrow_field_indices: Arc<Vec<usize>>,
-    /// Indices into `projected_arrow_schema. This must be sorted.
-    pub(super) non_predicate_arrow_field_indices: Arc<Vec<usize>>,
+    /// Indices into `projected_arrow_fields. This must be sorted.
+    pub(super) predicate_field_indices: Arc<[usize]>,
+    /// Indices into `projected_arrow_fields. This must be sorted.
+    pub(super) non_predicate_field_indices: Arc<[usize]>,
     pub(super) min_values_per_thread: usize,
 }
 
@@ -50,7 +49,7 @@ impl RowGroupDecoder {
 
         if self.use_prefiltered.is_some()
             && row_group_data.slice.is_none()
-            && !self.predicate_arrow_field_indices.is_empty()
+            && !self.predicate_field_indices.is_empty()
         {
             self.row_group_data_to_df_prefiltered(row_group_data).await
         } else {
@@ -64,7 +63,7 @@ impl RowGroupDecoder {
     ) -> PolarsResult<DataFrame> {
         let row_group_data = Arc::new(row_group_data);
 
-        let out_width = self.row_index.is_some() as usize + self.projected_arrow_schema.len();
+        let out_width = self.row_index.is_some() as usize + self.projected_arrow_fields.len();
 
         let mut out_columns = Vec::with_capacity(out_width);
 
@@ -121,35 +120,22 @@ impl RowGroupDecoder {
         row_group_data: &RowGroupData,
         slice_range: core::ops::Range<usize>,
     ) -> PolarsResult<Option<Column>> {
-        if let Some((name, offset)) = self.row_index.as_deref() {
-            let offset = offset.load(std::sync::atomic::Ordering::Relaxed);
+        if let Some(RowIndex { name, offset }) = self.row_index.clone() {
             let projection_height = slice_range.len();
 
-            let Some(offset) = (|| {
-                let offset = offset
-                    .checked_add((row_group_data.row_offset + slice_range.start) as IdxSize)?;
-                offset.checked_add(projection_height as IdxSize)?;
-
-                Some(offset)
-            })() else {
-                let msg = format!(
-                    "adding a row index column with offset {} overflows at {} rows",
-                    offset,
-                    row_group_data.row_offset + slice_range.end
-                );
-                polars_bail!(ComputeError: msg)
-            };
+            let offset = offset.saturating_add(
+                IdxSize::try_from(row_group_data.row_offset + slice_range.start)
+                    .unwrap_or(IdxSize::MAX),
+            );
 
             // The DataFrame can be empty at this point if no columns were projected from the file,
             // so we create the row index column manually instead of using `df.with_row_index` to
             // ensure it has the correct number of rows.
-            let mut ca = IdxCa::from_vec(
-                name.clone(),
-                (offset..offset + projection_height as IdxSize).collect(),
-            );
-            ca.set_sorted_flag(IsSorted::Ascending);
-
-            Ok(Some(ca.into_column()))
+            Ok(Some(Column::new_row_index(
+                name,
+                offset,
+                projection_height,
+            )?))
         } else {
             Ok(None)
         }
@@ -163,26 +149,48 @@ impl RowGroupDecoder {
         row_group_data: &Arc<RowGroupData>,
         filter: Option<polars_parquet::read::Filter>,
     ) -> PolarsResult<()> {
-        let projected_arrow_schema = &self.projected_arrow_schema;
+        let projected_arrow_fields = &self.projected_arrow_fields;
         let expected_num_rows = filter
             .as_ref()
             .map_or(row_group_data.row_group_metadata.num_rows(), |x| {
                 x.num_rows(row_group_data.row_group_metadata.num_rows())
             });
 
+        // Ensure we provide the same output column order as the pre-filtered decode.
+        let get_projected_field_at_output_index = {
+            let predicate_field_indices = self.predicate_field_indices.clone();
+            let non_predicate_field_indices = self.non_predicate_field_indices.clone();
+
+            move |i: usize| {
+                if predicate_field_indices.is_empty() {
+                    i
+                } else if i < predicate_field_indices.len() {
+                    predicate_field_indices[i]
+                } else {
+                    non_predicate_field_indices[i - predicate_field_indices.len()]
+                }
+            }
+        };
+
         let Some((cols_per_thread, _)) = calc_cols_per_thread(
             row_group_data.row_group_metadata.num_rows(),
-            projected_arrow_schema.len(),
+            projected_arrow_fields.len(),
             self.min_values_per_thread,
         ) else {
             // Single-threaded
-            for s in projected_arrow_schema.iter_values().map(|arrow_field| {
-                decode_column(
-                    arrow_field,
+            for s in (0..projected_arrow_fields.len()).map(|i| {
+                let projection = &projected_arrow_fields[get_projected_field_at_output_index(i)];
+
+                let (col, pred_true_mask) = decode_column(
+                    projection.arrow_field(),
                     row_group_data,
                     filter.clone(),
                     expected_num_rows,
-                )
+                )?;
+
+                let col = projection.apply_transform(col)?;
+
+                PolarsResult::Ok((col, pred_true_mask))
             }) {
                 out_vec.push(s?.0)
             }
@@ -190,37 +198,43 @@ impl RowGroupDecoder {
             return Ok(());
         };
 
-        let projected_arrow_schema = projected_arrow_schema.clone();
+        let projected_arrow_fields = projected_arrow_fields.clone();
         let row_group_data_2 = row_group_data.clone();
 
         let task_handles = {
-            let projected_arrow_schema = projected_arrow_schema.clone();
+            let projected_arrow_fields = projected_arrow_fields.clone();
             let filter = filter.clone();
 
             parallelize_first_to_local(
-                (0..projected_arrow_schema.len())
+                (0..projected_arrow_fields.len())
                     .step_by(cols_per_thread)
                     .map(move |offset| {
                         let row_group_data = row_group_data_2.clone();
-                        let projected_arrow_schema = projected_arrow_schema.clone();
+                        let projected_arrow_fields = projected_arrow_fields.clone();
                         let filter = filter.clone();
+                        let get_projected_field_at_output_index =
+                            get_projected_field_at_output_index.clone();
 
                         async move {
                             // This is exact as we have already taken out the remainder.
                             (offset
                                 ..offset
                                     .saturating_add(cols_per_thread)
-                                    .min(projected_arrow_schema.len()))
+                                    .min(projected_arrow_fields.len()))
                                 .map(|i| {
-                                    let (_, arrow_field) =
-                                        projected_arrow_schema.get_at_index(i).unwrap();
+                                    let projection = &projected_arrow_fields
+                                        [get_projected_field_at_output_index(i)];
 
-                                    decode_column(
-                                        arrow_field,
+                                    let (col, pred_true_mask) = decode_column(
+                                        projection.arrow_field(),
                                         &row_group_data,
                                         filter.clone(),
                                         expected_num_rows,
-                                    )
+                                    )?;
+
+                                    let col = projection.apply_transform(col)?;
+
+                                    Ok((col, pred_true_mask))
                                 })
                                 .collect::<PolarsResult<Vec<_>>>()
                         }
@@ -381,8 +395,7 @@ fn decode_column_in_filter(
             column_predicates.predicates.get(&arrow_field.name)
         {
             constant = specialized.as_ref().and_then(|s| match s {
-                SpecializedColumnPredicateExpr::Eq(sc) if !sc.is_null() => Some(sc),
-                SpecializedColumnPredicateExpr::EqMissing(sc) => Some(sc),
+                SpecializedColumnPredicate::Equal(sc) if !sc.is_null() => Some(sc),
                 _ => None,
             });
 
@@ -413,17 +426,19 @@ impl RowGroupDecoder {
         row_group_data: RowGroupData,
     ) -> PolarsResult<DataFrame> {
         debug_assert!(row_group_data.slice.is_none()); // Invariant of the optimizer.
-        assert!(self.predicate_arrow_field_indices.len() <= self.projected_arrow_schema.len());
+        assert!(self.predicate_field_indices.len() <= self.projected_arrow_fields.len());
 
         let prefilter_setting = self.use_prefiltered.as_ref().unwrap();
         let row_group_data = Arc::new(row_group_data);
         let projection_height = row_group_data.row_group_metadata.num_rows();
 
         let mut live_columns = Vec::with_capacity(
-            self.row_index.is_some() as usize + self.predicate_arrow_field_indices.len(),
+            self.row_index.is_some() as usize
+                + self.predicate_field_indices.len()
+                + self.non_predicate_field_indices.len(),
         );
         let mut masks = Vec::with_capacity(
-            self.row_index.is_some() as usize + self.predicate_arrow_field_indices.len(),
+            self.row_index.is_some() as usize + self.predicate_field_indices.len(),
         );
 
         if let Some(s) = self.materialize_row_index(
@@ -435,14 +450,7 @@ impl RowGroupDecoder {
 
         let scan_predicate = self.predicate.as_ref().unwrap();
 
-        // Materialize file and hive columns in sorted order - this is important for correct merging
-        // later.
-        //
-        // We do a trick to turn `Iterator<Item = Result<Column>>` into `Iterator<Item = Column>`
-        // for `hive::merge_sorted_to_schema_order`.
-
-        let use_column_predicates = scan_predicate.column_predicates.is_sumwise_complete
-            && self.row_index.is_none()
+        let use_column_predicates = self.allow_column_predicates
             && !row_group_data
                 .row_group_metadata
                 .parquet_columns()
@@ -452,49 +460,54 @@ impl RowGroupDecoder {
                         return false;
                     };
                     matches!(pt.logical_type, Some(PrimitiveLogicalType::Float16))
-                })
-            && self
-                .predicate_arrow_field_indices
-                .iter()
-                .map(|&i| self.projected_arrow_schema.get_at_index(i).unwrap())
-                .all(|(_, arrow_field)| !arrow_field.dtype().is_nested());
+                });
 
         let cols_per_thread = (self
-            .predicate_arrow_field_indices
+            .predicate_field_indices
             .len()
             .div_ceil(self.num_pipelines))
         .max(1);
         let task_handles = {
-            let predicate_arrow_field_indices = self.predicate_arrow_field_indices.clone();
-            let projected_arrow_schema = self.projected_arrow_schema.clone();
+            let predicate_field_indices = self.predicate_field_indices.clone();
+            let projected_arrow_fields = self.projected_arrow_fields.clone();
             let row_group_data = row_group_data.clone();
 
             parallelize_first_to_local(
-                (0..self.predicate_arrow_field_indices.len())
+                (0..self.predicate_field_indices.len())
                     .step_by(cols_per_thread)
                     .map(move |offset| {
                         let row_group_data = row_group_data.clone();
-                        let predicate_arrow_field_indices = predicate_arrow_field_indices.clone();
-                        let projected_arrow_schema = projected_arrow_schema.clone();
+                        let predicate_field_indices = predicate_field_indices.clone();
+                        let projected_arrow_fields = projected_arrow_fields.clone();
                         let column_predicates = scan_predicate.column_predicates.clone();
 
                         async move {
                             (offset
                                 ..offset
                                     .saturating_add(cols_per_thread)
-                                    .min(predicate_arrow_field_indices.len()))
+                                    .min(predicate_field_indices.len()))
                                 .map(|i| {
-                                    let (_, arrow_field) = projected_arrow_schema
-                                        .get_at_index(predicate_arrow_field_indices[i])
-                                        .unwrap();
+                                    let projection =
+                                        &projected_arrow_fields[predicate_field_indices[i]];
 
-                                    decode_column_in_filter(
-                                        arrow_field,
+                                    if use_column_predicates {
+                                        debug_assert!(matches!(
+                                            projection,
+                                            ArrowFieldProjection::Plain(_)
+                                        ));
+                                    }
+
+                                    let (col, pred_true_mask) = decode_column_in_filter(
+                                        projection.arrow_field(),
                                         use_column_predicates,
                                         column_predicates.as_ref(),
                                         row_group_data.as_ref(),
                                         projection_height,
-                                    )
+                                    )?;
+
+                                    let col = projection.apply_transform(col)?;
+
+                                    Ok((col, pred_true_mask))
                                 })
                                 .collect::<PolarsResult<Vec<_>>>()
                         }
@@ -548,7 +561,7 @@ impl RowGroupDecoder {
 
             unsafe {
                 live_df.get_columns_mut().truncate(
-                    self.row_index.is_some() as usize + self.predicate_arrow_field_indices.len(),
+                    self.row_index.is_some() as usize + self.predicate_field_indices.len(),
                 )
             }
 
@@ -568,16 +581,9 @@ impl RowGroupDecoder {
             )
         };
 
-        if self.non_predicate_arrow_field_indices.is_empty() {
+        if self.non_predicate_field_indices.is_empty() {
             // User or test may have explicitly requested prefiltering
-            let iter = self.projected_arrow_schema.iter_names().cloned();
-            return Ok(if let Some(ri) = self.row_index.as_ref() {
-                live_df_filtered
-                    .select(std::iter::once(ri.0.clone()).chain(iter))
-                    .unwrap()
-            } else {
-                live_df_filtered.select(iter).unwrap()
-            });
+            return Ok(live_df_filtered);
         }
 
         mask.rechunk_mut();
@@ -593,23 +599,23 @@ impl RowGroupDecoder {
         let expected_num_rows = mask_bitmap.set_bits();
 
         let cols_per_thread = (self
-            .predicate_arrow_field_indices
+            .predicate_field_indices
             .len()
             .div_ceil(self.num_pipelines))
         .max(1);
+
         let task_handles = {
-            let non_predicate_arrow_field_indices = self.non_predicate_arrow_field_indices.clone();
-            let non_predicate_len = non_predicate_arrow_field_indices.len();
-            let projected_arrow_schema = self.projected_arrow_schema.clone();
+            let non_predicate_field_indices = self.non_predicate_field_indices.clone();
+            let non_predicate_len = non_predicate_field_indices.len();
+            let projected_arrow_fields = self.projected_arrow_fields.clone();
             let row_group_data = row_group_data.clone();
             let prefilter_setting = *prefilter_setting;
 
             parallelize_first_to_local((0..non_predicate_len).step_by(cols_per_thread).map(
                 move |offset| {
                     let row_group_data = row_group_data.clone();
-                    let non_predicate_arrow_field_indices =
-                        non_predicate_arrow_field_indices.clone();
-                    let projected_arrow_schema = projected_arrow_schema.clone();
+                    let non_predicate_field_indices = non_predicate_field_indices.clone();
+                    let projected_arrow_fields = projected_arrow_fields.clone();
                     let mask = mask.clone();
                     let mask_bitmap = mask_bitmap.clone();
                     let max_col = offset
@@ -619,19 +625,20 @@ impl RowGroupDecoder {
                     async move {
                         (offset..max_col)
                             .map(|i| {
-                                let (_, arrow_field) = projected_arrow_schema
-                                    .get_at_index(non_predicate_arrow_field_indices[i])
-                                    .unwrap();
+                                let projection =
+                                    &projected_arrow_fields[non_predicate_field_indices[i]];
 
-                                decode_column_prefiltered(
-                                    arrow_field,
+                                let col = decode_column_prefiltered(
+                                    projection.arrow_field(),
                                     row_group_data.as_ref(),
                                     prefilter_cost,
                                     &prefilter_setting,
                                     &mask,
                                     &mask_bitmap,
                                     expected_num_rows,
-                                )
+                                )?;
+
+                                projection.apply_transform(col)
                             })
                             .collect::<PolarsResult<Vec<_>>>()
                     }
@@ -641,33 +648,13 @@ impl RowGroupDecoder {
 
         let live_columns = live_df_filtered.take_columns();
 
-        let mut dead_cols = Vec::with_capacity(self.non_predicate_arrow_field_indices.len());
+        let mut dead_cols = Vec::with_capacity(self.non_predicate_field_indices.len());
         for fut in task_handles {
             dead_cols.extend(fut.await?);
         }
 
-        // dead_columns
-        // [ ..arrow_fields ]
-        // live_df_filtered
-        // [ row_index?, ..arrow_fields, ..hive_cols, file_path? ]
-        // We re-use `hive::merge_sorted_to_schema_order()` as it performs most of the merge operation we want.
-        // But we take out the `row_index` column as it isn't on the correct side.
-
-        let mut merged = Vec::with_capacity(live_columns.len() + dead_cols.len());
-
-        if self.row_index.is_some() {
-            merged.push(live_columns[0].clone());
-        };
-
-        hive::merge_sorted_to_schema_order(
-            &mut dead_cols.into_iter(), // df_columns
-            &mut live_columns
-                .into_iter()
-                .skip(self.row_index.is_some() as usize), // hive_columns
-            &self.projected_arrow_schema,
-            &mut merged,
-        );
-
+        let mut merged = live_columns;
+        merged.extend(dead_cols);
         let df = unsafe { DataFrame::new_no_checks(expected_num_rows, merged) };
         Ok(df)
     }
