@@ -1,12 +1,10 @@
 use std::fmt::Write;
 
-use polars_ops::frame::JoinType;
 use polars_plan::dsl::PartitionVariantIR;
 use polars_plan::plans::expr_ir::ExprIR;
 use polars_plan::plans::{AExpr, EscapeLabel};
 use polars_plan::prelude::FileType;
 use polars_utils::arena::Arena;
-use polars_utils::itertools::Itertools;
 use polars_utils::slice_enum::Slice;
 use slotmap::{Key, SecondaryMap, SlotMap};
 
@@ -37,8 +35,9 @@ impl NodeStyle {
             | K::EquiJoin { .. }
             | K::SemiAntiJoin { .. }
             | K::InMemoryJoin { .. }
-            | K::MergeSorted { .. }
             | K::Multiplexer { .. } => Self::MemoryIntensive,
+            #[cfg(feature = "merge_sorted")]
+            K::MergeSorted { .. } => Self::MemoryIntensive,
             _ => Self::Generic,
         }
     }
@@ -61,7 +60,7 @@ impl NodeStyle {
     /// Returns a legend explaining the node style meaning.
     pub fn legend() -> String {
         format!(
-            "fontsize=\"10\"\nlabelloc=\"b\"\nlabel=<<BR/><BR/><B>Legend</B><BR/><BR/>◯ streaming engine node<FONT COLOR=\"{}\">⬤</FONT>potentially memory-intensive node<FONT COLOR=\"{}\">⬤</FONT>in-memory engine fallback>",
+            "fontname=\"Helvetica\"\nfontsize=\"10\"\nlabelloc=\"b\"\nlabel=<<BR/><BR/><B>Legend</B><BR/><BR/>◯ streaming engine node <FONT COLOR=\"{}\">⬤</FONT> potentially memory-intensive node <FONT COLOR=\"{}\">⬤</FONT> in-memory engine fallback>",
             Self::COLOR_MEM_INTENSIVE,
             Self::COLOR_IN_MEM_FALLBACK,
         )
@@ -74,12 +73,76 @@ fn escape_graphviz(s: &str) -> String {
         .replace('"', "\\\"")
 }
 
-fn fmt_exprs(exprs: &[ExprIR], expr_arena: &Arena<AExpr>) -> String {
-    exprs
-        .iter()
-        .map(|e| escape_graphviz(&e.display(expr_arena).to_string()))
-        .collect_vec()
-        .join("\\n")
+fn fmt_expr(f: &mut dyn Write, expr: &ExprIR, expr_arena: &Arena<AExpr>) -> std::fmt::Result {
+    // Remove the alias to make the display better
+    let without_alias = ExprIR::from_node(expr.node(), expr_arena);
+    write!(
+        f,
+        "{} = {}",
+        expr.output_name(),
+        without_alias.display(expr_arena)
+    )
+}
+
+pub enum FormatExprStyle {
+    Select,
+    NoAliases,
+}
+
+pub fn fmt_exprs_to_label(
+    exprs: &[ExprIR],
+    expr_arena: &Arena<AExpr>,
+    style: FormatExprStyle,
+) -> String {
+    let mut buffer = String::new();
+    let mut f = EscapeLabel(&mut buffer);
+    fmt_exprs(&mut f, exprs, expr_arena, style);
+    buffer
+}
+
+pub fn fmt_exprs(
+    f: &mut dyn Write,
+    exprs: &[ExprIR],
+    expr_arena: &Arena<AExpr>,
+    style: FormatExprStyle,
+) {
+    if matches!(style, FormatExprStyle::Select) {
+        let mut formatted = Vec::new();
+
+        let mut max_name_width = 0;
+        let mut max_expr_width = 0;
+
+        for e in exprs {
+            let mut name = String::new();
+            let mut expr = String::new();
+
+            // Remove the alias to make the display better
+            let without_alias = ExprIR::from_node(e.node(), expr_arena);
+
+            write!(name, "{}", e.output_name()).unwrap();
+            write!(expr, "{}", without_alias.display(expr_arena)).unwrap();
+
+            max_name_width = max_name_width.max(name.chars().count());
+            max_expr_width = max_expr_width.max(expr.chars().count());
+
+            formatted.push((name, expr));
+        }
+
+        for (name, expr) in formatted {
+            write!(f, "{name:>max_name_width$} = {expr:<max_expr_width$}\\n").unwrap();
+        }
+    } else {
+        let Some(e) = exprs.first() else {
+            return;
+        };
+
+        fmt_expr(f, e, expr_arena).unwrap();
+
+        for e in &exprs[1..] {
+            f.write_str("\\n").unwrap();
+            fmt_expr(f, e, expr_arena).unwrap();
+        }
+    }
 }
 
 #[recursive::recursive]
@@ -125,7 +188,10 @@ fn visualize_plan_rec(
                 "select"
             };
             (
-                format!("{label}\\n{}", fmt_exprs(selectors, expr_arena)),
+                format!(
+                    "{label}\\n{}",
+                    fmt_exprs_to_label(selectors, expr_arena, FormatExprStyle::Select)
+                ),
                 from_ref(input),
             )
         },
@@ -140,12 +206,15 @@ fn visualize_plan_rec(
         PhysNodeKind::InputIndependentSelect { selectors } => (
             format!(
                 "input-independent-select\\n{}",
-                fmt_exprs(selectors, expr_arena)
+                fmt_exprs_to_label(selectors, expr_arena, FormatExprStyle::Select)
             ),
             &[][..],
         ),
         PhysNodeKind::Reduce { input, exprs } => (
-            format!("reduce\\n{}", fmt_exprs(exprs, expr_arena)),
+            format!(
+                "reduce\\n{}",
+                fmt_exprs_to_label(exprs, expr_arena, FormatExprStyle::Select)
+            ),
             from_ref(input),
         ),
         PhysNodeKind::StreamingSlice {
@@ -164,8 +233,16 @@ fn visualize_plan_rec(
             format!("slice\\noffset: {offset}, length: {length}"),
             from_ref(input),
         ),
+        PhysNodeKind::DynamicSlice {
+            input,
+            offset,
+            length,
+        } => ("slice".to_owned(), &[*input, *offset, *length][..]),
         PhysNodeKind::Filter { input, predicate } => (
-            format!("filter\\n{}", fmt_exprs(from_ref(predicate), expr_arena)),
+            format!(
+                "filter\\n{}",
+                fmt_exprs_to_label(from_ref(predicate), expr_arena, FormatExprStyle::Select)
+            ),
             from_ref(input),
         ),
         PhysNodeKind::SimpleProjection { input, columns } => (
@@ -201,19 +278,31 @@ fn visualize_plan_rec(
 
             match file_type {
                 #[cfg(feature = "parquet")]
-                FileType::Parquet(_) => (format!("{}[parquet]", variant), from_ref(input)),
+                FileType::Parquet(_) => (format!("{variant}[parquet]"), from_ref(input)),
                 #[cfg(feature = "ipc")]
-                FileType::Ipc(_) => (format!("{}[ipc]", variant), from_ref(input)),
+                FileType::Ipc(_) => (format!("{variant}[ipc]"), from_ref(input)),
                 #[cfg(feature = "csv")]
-                FileType::Csv(_) => (format!("{}[csv]", variant), from_ref(input)),
+                FileType::Csv(_) => (format!("{variant}[csv]"), from_ref(input)),
                 #[cfg(feature = "json")]
-                FileType::Json(_) => (format!("{}[ndjson]", variant), from_ref(input)),
+                FileType::Json(_) => (format!("{variant}[ndjson]"), from_ref(input)),
                 #[allow(unreachable_patterns)]
                 _ => todo!(),
             }
         },
-        PhysNodeKind::InMemoryMap { input, map: _ } => {
-            ("in-memory-map".to_string(), from_ref(input))
+        PhysNodeKind::InMemoryMap {
+            input,
+            map: _,
+            format_str,
+        } => {
+            let mut label = String::new();
+            label.push_str("in-memory-map");
+            if let Some(format_str) = format_str {
+                label.push('\n');
+
+                let mut f = EscapeLabel(&mut label);
+                write!(f, "{format_str}").unwrap();
+            }
+            (label, from_ref(input))
         },
         PhysNodeKind::Map { input, map: _ } => ("map".to_string(), from_ref(input)),
         PhysNodeKind::Sort {
@@ -222,9 +311,13 @@ fn visualize_plan_rec(
             slice: _,
             sort_options: _,
         } => (
-            format!("sort\\n{}", fmt_exprs(by_column, expr_arena)),
+            format!(
+                "sort\\n{}",
+                fmt_exprs_to_label(by_column, expr_arena, FormatExprStyle::NoAliases)
+            ),
             from_ref(input),
         ),
+        PhysNodeKind::Repeat { value, repeats } => ("repeat".to_owned(), &[*value, *repeats][..]),
         PhysNodeKind::OrderedUnion { inputs } => ("ordered-union".to_string(), inputs.as_slice()),
         PhysNodeKind::Zip {
             inputs,
@@ -242,7 +335,7 @@ fn visualize_plan_rec(
             scan_sources,
             file_reader_builder,
             cloud_options: _,
-            projected_file_schema,
+            file_projection_builder,
             output_schema,
             row_index,
             pre_slice,
@@ -251,7 +344,8 @@ fn visualize_plan_rec(
             include_file_paths,
             cast_columns_policy: _,
             missing_columns_policy: _,
-            extra_columns_policy: _,
+            forbid_extra_columns: _,
+            deletion_files,
             file_schema: _,
         } => {
             let mut out = format!("multi-scan[{}]", file_reader_builder.reader_name());
@@ -267,7 +361,7 @@ fn visualize_plan_rec(
                 f,
                 "\nproject: {} total, {} from file",
                 output_schema.len(),
-                projected_file_schema.len()
+                file_projection_builder.num_projections(),
             )
             .unwrap();
 
@@ -276,7 +370,7 @@ fn visualize_plan_rec(
             }
 
             if let Some(col_name) = include_file_paths {
-                write!(f, "\nfile path column: {}", col_name).unwrap();
+                write!(f, "\nfile path column: {col_name}").unwrap();
             }
 
             if let Some(pre_slice) = pre_slice {
@@ -299,20 +393,24 @@ fn visualize_plan_rec(
             }
 
             if let Some(v) = hive_parts.as_ref().map(|h| h.df().width()) {
-                write!(f, "\nhive: {} column", v).unwrap();
+                write!(f, "\nhive: {v} column").unwrap();
 
                 if v != 1 {
                     write!(f, "s").unwrap();
                 }
             }
 
+            if let Some(deletion_files) = deletion_files {
+                write!(f, "\n{deletion_files}").unwrap();
+            }
+
             (out, &[][..])
         },
         PhysNodeKind::GroupBy { input, key, aggs } => (
             format!(
                 "group-by\\nkey:\\n{}\\naggs:\\n{}",
-                fmt_exprs(key, expr_arena),
-                fmt_exprs(aggs, expr_arena)
+                fmt_exprs_to_label(key, expr_arena, FormatExprStyle::Select),
+                fmt_exprs_to_label(aggs, expr_arena, FormatExprStyle::Select)
             ),
             from_ref(input),
         ),
@@ -342,23 +440,34 @@ fn visualize_plan_rec(
             let label = match phys_sm[node_key].kind {
                 PhysNodeKind::EquiJoin { .. } => "equi-join",
                 PhysNodeKind::InMemoryJoin { .. } => "in-memory-join",
+                PhysNodeKind::CrossJoin { .. } => "cross-join",
                 PhysNodeKind::SemiAntiJoin {
                     output_bool: false, ..
-                } if args.how == JoinType::Semi => "semi-join",
+                } if args.how.is_semi() => "semi-join",
                 PhysNodeKind::SemiAntiJoin {
                     output_bool: false, ..
-                } if args.how == JoinType::Anti => "anti-join",
+                } if args.how.is_anti() => "anti-join",
                 PhysNodeKind::SemiAntiJoin {
                     output_bool: true, ..
-                } if args.how == JoinType::Semi => "is-in",
+                } if args.how.is_semi() => "is-in",
                 PhysNodeKind::SemiAntiJoin {
                     output_bool: true, ..
-                } if args.how == JoinType::Anti => "is-not-in",
+                } if args.how.is_anti() => "is-not-in",
                 _ => unreachable!(),
             };
             let mut label = label.to_string();
-            write!(label, r"\nleft_on:\n{}", fmt_exprs(left_on, expr_arena)).unwrap();
-            write!(label, r"\nright_on:\n{}", fmt_exprs(right_on, expr_arena)).unwrap();
+            write!(
+                label,
+                r"\nleft_on:\n{}",
+                fmt_exprs_to_label(left_on, expr_arena, FormatExprStyle::NoAliases)
+            )
+            .unwrap();
+            write!(
+                label,
+                r"\nright_on:\n{}",
+                fmt_exprs_to_label(right_on, expr_arena, FormatExprStyle::NoAliases)
+            )
+            .unwrap();
             if args.how.is_equi() {
                 write!(
                     label,
@@ -372,6 +481,11 @@ fn visualize_plan_rec(
             }
             (label, &[*input_left, *input_right][..])
         },
+        PhysNodeKind::CrossJoin {
+            input_left,
+            input_right,
+            args: _,
+        } => ("cross-join".to_string(), &[*input_left, *input_right][..]),
         #[cfg(feature = "merge_sorted")]
         PhysNodeKind::MergeSorted {
             input_left,
@@ -381,7 +495,7 @@ fn visualize_plan_rec(
             let mut out = "merge-sorted".to_string();
             let mut f = EscapeLabel(&mut out);
 
-            write!(f, "\nkey: {}", key).unwrap();
+            write!(f, "\nkey: {key}").unwrap();
 
             (out, &[*input_left, *input_right][..])
         },
@@ -412,7 +526,7 @@ pub fn visualize_plan(
 ) -> String {
     let mut visited: SecondaryMap<PhysNodeKey, ()> = SecondaryMap::new();
     let mut out = Vec::with_capacity(phys_sm.len() + 3);
-    out.push("digraph polars {\nrankdir=\"BT\"".to_string());
+    out.push("digraph polars {\nrankdir=\"BT\"\nnode [fontname=\"Monospace\"]".to_string());
     out.push(NodeStyle::legend());
     visualize_plan_rec(root, phys_sm, expr_arena, &mut visited, &mut out);
     out.push("}".to_string());
