@@ -1,5 +1,6 @@
 use std::sync::Arc;
 
+use polars_core::chunked_array::cast::CastOptions;
 use polars_core::frame::DataFrame;
 use polars_core::prelude::{DataType, Field, InitHashMaps, PlHashMap, PlHashSet};
 use polars_core::schema::{Schema, SchemaExt};
@@ -16,7 +17,8 @@ use polars_utils::pl_str::PlSmallStr;
 use polars_utils::{unique_column_name, unitvec};
 use slotmap::SlotMap;
 
-use super::{PhysNode, PhysNodeKey, PhysNodeKind, PhysStream};
+use super::fmt::fmt_exprs;
+use super::{PhysNode, PhysNodeKey, PhysNodeKind, PhysStream, StreamingLowerIRContext};
 use crate::physical_plan::lower_group_by::build_group_by_stream;
 
 type ExprNodeKey = Node;
@@ -38,11 +40,27 @@ impl ExprCache {
 }
 
 struct LowerExprContext<'a> {
+    prepare_visualization: bool,
     expr_arena: &'a mut Arena<AExpr>,
     phys_sm: &'a mut SlotMap<PhysNodeKey, PhysNode>,
     cache: &'a mut ExprCache,
 }
 
+impl<'a> From<LowerExprContext<'a>> for StreamingLowerIRContext {
+    fn from(value: LowerExprContext<'a>) -> Self {
+        Self {
+            prepare_visualization: value.prepare_visualization,
+        }
+    }
+}
+impl<'a> From<&LowerExprContext<'a>> for StreamingLowerIRContext {
+    fn from(value: &LowerExprContext<'a>) -> Self {
+        Self {
+            prepare_visualization: value.prepare_visualization,
+        }
+    }
+}
+
 pub(crate) fn is_fake_elementwise_function(expr: &AExpr) -> bool {
     // The in-memory engine treats ApplyList as elementwise but this is not actually
     // the case. It doesn't cause any problems for the in-memory engine because of
@@ -51,21 +69,15 @@ pub(crate) fn is_fake_elementwise_function(expr: &AExpr) -> bool {
     // Some other functions are also marked as elementwise for filter pushdown
     // but aren't actually elementwise (e.g. arguments aren't same length).
     match expr {
-        AExpr::AnonymousFunction { options, .. } => {
-            options.collect_groups == ApplyOptions::ApplyList
-        },
-        AExpr::Function {
-            function, options, ..
-        } => {
-            if options.collect_groups == ApplyOptions::ApplyList {
-                return true;
+        AExpr::Function { function, .. } => {
+            use IRFunctionExpr as F;
+            match function {
+                #[cfg(feature = "is_in")]
+                F::Boolean(IRBooleanFunction::IsIn { .. }) => true,
+                #[cfg(feature = "replace")]
+                F::Replace | F::ReplaceStrict { .. } => true,
+                _ => false,
             }
-
-            use FunctionExpr as F;
-            matches!(
-                function,
-                F::Boolean(BooleanFunction::IsIn { .. }) | F::Replace | F::ReplaceStrict { .. }
-            )
         },
         _ => false,
     }
@@ -120,8 +132,7 @@ pub fn is_input_independent_rec(
     }
 
     let ret = match arena.get(expr_key) {
-        AExpr::Explode(inner)
-        | AExpr::Alias(inner, _)
+        AExpr::Explode { expr: inner, .. }
         | AExpr::Cast {
             expr: inner,
             dtype: _,
@@ -182,6 +193,7 @@ pub fn is_input_independent_rec(
             function: _,
             output_type: _,
             options: _,
+            fmt_str: _,
         }
         | AExpr::Function {
             input,
@@ -190,6 +202,11 @@ pub fn is_input_independent_rec(
         } => input
             .iter()
             .all(|expr| is_input_independent_rec(expr.node(), arena, cache)),
+        AExpr::Eval {
+            expr,
+            evaluation: _,
+            variant: _,
+        } => is_input_independent_rec(*expr, arena, cache),
         AExpr::Window {
             function,
             partition_by,
@@ -261,7 +278,7 @@ pub fn is_length_preserving_rec(
 
     let ret = match arena.get(expr_key) {
         AExpr::Gather { .. }
-        | AExpr::Explode(_)
+        | AExpr::Explode { .. }
         | AExpr::Filter { .. }
         | AExpr::Agg(_)
         | AExpr::Slice { .. }
@@ -270,8 +287,7 @@ pub fn is_length_preserving_rec(
 
         AExpr::Column(_) => true,
 
-        AExpr::Alias(inner, _)
-        | AExpr::Cast {
+        AExpr::Cast {
             expr: inner,
             dtype: _,
             options: _,
@@ -306,6 +322,7 @@ pub fn is_length_preserving_rec(
             function: _,
             output_type: _,
             options,
+            fmt_str: _,
         }
         | AExpr::Function {
             input,
@@ -318,6 +335,7 @@ pub fn is_length_preserving_rec(
                     .iter()
                     .all(|expr| is_length_preserving_rec(expr.node(), arena, cache))
         },
+        AExpr::Eval { .. } => true,
         AExpr::Window {
             function: _, // Actually shouldn't matter for window functions.
             partition_by: _,
@@ -405,9 +423,23 @@ fn build_fallback_node_with_ctx(
             .try_collect()?;
         DataFrame::new_with_broadcast(columns)
     };
+
+    let format_str = ctx.prepare_visualization.then(|| {
+        let mut buffer = String::new();
+        buffer.push_str("SELECT [\n");
+        fmt_exprs(
+            &mut buffer,
+            exprs,
+            ctx.expr_arena,
+            super::fmt::FormatExprStyle::Select,
+        );
+        buffer.push(']');
+        buffer
+    });
     let kind = PhysNodeKind::InMemoryMap {
         input: input_stream,
         map: Arc::new(map),
+        format_str,
     };
     Ok(ctx.phys_sm.insert(PhysNode::new(output_schema, kind)))
 }
@@ -467,6 +499,36 @@ fn simplify_input_streams(
     Ok(input_streams)
 }
 
+// Assuming that agg_node is a single-input reduction, lowers its input recursively
+// and returns a Reduce node as well a node corresponding to the column to select
+// from the Reduce node for the aggregate.
+fn lower_unary_reduce_node(
+    input: PhysStream,
+    agg_node: Node,
+    ctx: &mut LowerExprContext,
+) -> PolarsResult<(PhysStream, Node)> {
+    let agg_aexpr = ctx.expr_arena.get(agg_node).clone();
+    let mut agg_input = Vec::with_capacity(1);
+    agg_aexpr.inputs_rev(&mut agg_input);
+    assert!(agg_input.len() == 1);
+
+    let (trans_input, trans_exprs) = lower_exprs_with_ctx(input, &agg_input, ctx)?;
+    let trans_agg_node = ctx.expr_arena.add(agg_aexpr.replace_inputs(&trans_exprs));
+
+    let out_name = unique_column_name();
+    let expr_ir = ExprIR::new(trans_agg_node, OutputName::Alias(out_name.clone()));
+    let output_schema = schema_for_select(trans_input, std::slice::from_ref(&expr_ir), ctx)?;
+    let kind = PhysNodeKind::Reduce {
+        input: trans_input,
+        exprs: vec![expr_ir],
+    };
+
+    let reduce_node_key = ctx.phys_sm.insert(PhysNode::new(output_schema, kind));
+    let reduce_stream = PhysStream::first(reduce_node_key);
+    let out_node = ctx.expr_arena.add(AExpr::Column(out_name));
+    Ok((reduce_stream, out_node))
+}
+
 // In the recursive lowering we don't bother with named expressions at all, so
 // we work directly with Nodes.
 #[recursive::recursive]
@@ -510,15 +572,22 @@ fn lower_exprs_with_ctx(
         }
 
         match ctx.expr_arena.get(expr).clone() {
-            AExpr::Explode(inner) => {
+            AExpr::Explode {
+                expr: inner,
+                skip_empty,
+            } => {
                 // While explode is streamable, it is not elementwise, so we
                 // have to transform it to a select node.
                 let (trans_input, trans_exprs) = lower_exprs_with_ctx(input, &[inner], ctx)?;
                 let exploded_name = unique_column_name();
-                let trans_inner = ctx.expr_arena.add(AExpr::Explode(trans_exprs[0]));
+                let trans_inner = ctx.expr_arena.add(AExpr::Explode {
+                    expr: trans_exprs[0],
+                    skip_empty,
+                });
                 let explode_expr =
                     ExprIR::new(trans_inner, OutputName::Alias(exploded_name.clone()));
-                let output_schema = schema_for_select(trans_input, &[explode_expr.clone()], ctx)?;
+                let output_schema =
+                    schema_for_select(trans_input, std::slice::from_ref(&explode_expr), ctx)?;
                 let node_kind = PhysNodeKind::Select {
                     input: trans_input,
                     selectors: vec![explode_expr.clone()],
@@ -528,7 +597,6 @@ fn lower_exprs_with_ctx(
                 input_streams.insert(PhysStream::first(node_key));
                 transformed_exprs.push(ctx.expr_arena.add(AExpr::Column(exploded_name)));
             },
-            AExpr::Alias(_, _) => unreachable!("alias found in physical plan"),
             AExpr::Column(_) => unreachable!("column should always be streamable"),
             AExpr::Literal(_) => {
                 let out_name = unique_column_name();
@@ -540,7 +608,79 @@ fn lower_exprs_with_ctx(
 
             AExpr::Function {
                 input: ref inner_exprs,
-                function: FunctionExpr::ConcatExpr(_rechunk),
+                function: IRFunctionExpr::Repeat,
+                options: _,
+            } => {
+                assert!(inner_exprs.len() == 2);
+                let out_name = unique_column_name();
+                let value_expr_ir = inner_exprs[0].with_alias(out_name.clone());
+                let repeats_expr_ir = inner_exprs[1].clone();
+                let value_stream = build_select_stream_with_ctx(input, &[value_expr_ir], ctx)?;
+                let repeats_stream = build_select_stream_with_ctx(input, &[repeats_expr_ir], ctx)?;
+
+                let output_schema = ctx.phys_sm[value_stream.node].output_schema.clone();
+                let kind = PhysNodeKind::Repeat {
+                    value: value_stream,
+                    repeats: repeats_stream,
+                };
+                let repeat_node_key = ctx.phys_sm.insert(PhysNode::new(output_schema, kind));
+                input_streams.insert(PhysStream::first(repeat_node_key));
+                transformed_exprs.push(ctx.expr_arena.add(AExpr::Column(out_name)));
+            },
+
+            AExpr::Function {
+                input: ref inner_exprs,
+                function: IRFunctionExpr::ExtendConstant,
+                options: _,
+            } => {
+                assert!(inner_exprs.len() == 3);
+                let input_schema = &ctx.phys_sm[input.node].output_schema;
+                let out_name = unique_column_name();
+                let first_ir = inner_exprs[0].with_alias(out_name.clone());
+                let out_dtype = first_ir.dtype(input_schema, Context::Default, ctx.expr_arena)?;
+                let mut value_expr_ir = inner_exprs[1].with_alias(out_name.clone());
+                let repeats_expr_ir = inner_exprs[2].clone();
+
+                // Cast the value if necessary.
+                if value_expr_ir.dtype(input_schema, Context::Default, ctx.expr_arena)? != out_dtype
+                {
+                    let cast_expr = AExpr::Cast {
+                        expr: value_expr_ir.node(),
+                        dtype: out_dtype.clone(),
+                        options: CastOptions::NonStrict,
+                    };
+                    value_expr_ir = ExprIR::new(
+                        ctx.expr_arena.add(cast_expr),
+                        OutputName::Alias(out_name.clone()),
+                    );
+                }
+
+                let first_stream = build_select_stream_with_ctx(input, &[first_ir], ctx)?;
+                let value_stream = build_select_stream_with_ctx(input, &[value_expr_ir], ctx)?;
+                let repeats_stream = build_select_stream_with_ctx(input, &[repeats_expr_ir], ctx)?;
+
+                let output_schema = ctx.phys_sm[first_stream.node].output_schema.clone();
+                let repeat_kind = PhysNodeKind::Repeat {
+                    value: value_stream,
+                    repeats: repeats_stream,
+                };
+                let repeat_node_key = ctx
+                    .phys_sm
+                    .insert(PhysNode::new(output_schema.clone(), repeat_kind));
+
+                let concat_kind = PhysNodeKind::OrderedUnion {
+                    inputs: vec![first_stream, PhysStream::first(repeat_node_key)],
+                };
+                let concat_node_key = ctx
+                    .phys_sm
+                    .insert(PhysNode::new(output_schema, concat_kind));
+                input_streams.insert(PhysStream::first(concat_node_key));
+                transformed_exprs.push(ctx.expr_arena.add(AExpr::Column(out_name)));
+            },
+
+            AExpr::Function {
+                input: ref inner_exprs,
+                function: IRFunctionExpr::ConcatExpr(_rechunk),
                 options: _,
             } => {
                 // We have to lower each expression separately as they might have different lengths.
@@ -569,7 +709,7 @@ fn lower_exprs_with_ctx(
 
             AExpr::Function {
                 input: ref inner_exprs,
-                function: FunctionExpr::Unique(maintain_order),
+                function: IRFunctionExpr::Unique(maintain_order),
                 options: _,
             } => {
                 assert!(inner_exprs.len() == 1);
@@ -580,7 +720,7 @@ fn lower_exprs_with_ctx(
                 let group_by_key_expr =
                     ExprIR::new(trans_inner_exprs[0], OutputName::Alias(tmp_name.clone()));
                 let group_by_output_schema =
-                    schema_for_select(trans_input, &[group_by_key_expr.clone()], ctx)?;
+                    schema_for_select(trans_input, std::slice::from_ref(&group_by_key_expr), ctx)?;
                 let group_by_stream = build_group_by_stream(
                     trans_input,
                     &[group_by_key_expr],
@@ -592,14 +732,16 @@ fn lower_exprs_with_ctx(
                     ctx.expr_arena,
                     ctx.phys_sm,
                     ctx.cache,
+                    StreamingLowerIRContext::from(&*ctx),
                 )?;
                 input_streams.insert(group_by_stream);
                 transformed_exprs.push(ctx.expr_arena.add(AExpr::Column(tmp_name)));
             },
 
+            #[cfg(feature = "is_in")]
             AExpr::Function {
                 input: ref inner_exprs,
-                function: FunctionExpr::Boolean(BooleanFunction::IsIn { nulls_equal }),
+                function: IRFunctionExpr::Boolean(IRBooleanFunction::IsIn { nulls_equal }),
                 options: _,
             } if is_scalar_ae(inner_exprs[1].node(), ctx.expr_arena) => {
                 // Translate left and right side separately (they could have different lengths).
@@ -610,7 +752,10 @@ fn lower_exprs_with_ctx(
                 let right_expr_exploded_node = match ctx.expr_arena.get(inner_exprs[1].node()) {
                     // expr.implode().explode() ~= expr (and avoids rechunking)
                     AExpr::Agg(IRAggExpr::Implode(n)) => *n,
-                    _ => ctx.expr_arena.add(AExpr::Explode(inner_exprs[1].node())),
+                    _ => ctx.expr_arena.add(AExpr::Explode {
+                        expr: inner_exprs[1].node(),
+                        skip_empty: true,
+                    }),
                 };
                 let (trans_input_right, trans_expr_right) =
                     lower_exprs_with_ctx(input, &[right_expr_exploded_node], ctx)?;
@@ -660,6 +805,7 @@ fn lower_exprs_with_ctx(
                 transformed_exprs.push(left_col_expr);
             },
 
+            // Lower arbitrary elementwise functions.
             ref node @ AExpr::Function {
                 input: ref inner_exprs,
                 options,
@@ -692,6 +838,38 @@ fn lower_exprs_with_ctx(
                 transformed_exprs.push(ctx.expr_arena.add(new_node));
             },
 
+            // Lower arbitrary row-separable functions.
+            ref node @ AExpr::Function {
+                input: ref inner_exprs,
+                ref function,
+                options,
+            } if options.is_row_separable() && !is_fake_elementwise_function(node) => {
+                // While these functions are streamable, they are not elementwise, so we
+                // have to transform them to a select node.
+                let inner_nodes = inner_exprs.iter().map(|x| x.node()).collect_vec();
+                let (trans_input, trans_exprs) = lower_exprs_with_ctx(input, &inner_nodes, ctx)?;
+                let out_name = unique_column_name();
+                let trans_inner = ctx.expr_arena.add(AExpr::Function {
+                    input: trans_exprs
+                        .iter()
+                        .map(|node| ExprIR::from_node(*node, ctx.expr_arena))
+                        .collect(),
+                    function: function.clone(),
+                    options,
+                });
+                let func_expr = ExprIR::new(trans_inner, OutputName::Alias(out_name.clone()));
+                let output_schema =
+                    schema_for_select(trans_input, std::slice::from_ref(&func_expr), ctx)?;
+                let node_kind = PhysNodeKind::Select {
+                    input: trans_input,
+                    selectors: vec![func_expr.clone()],
+                    extend_original: false,
+                };
+                let node_key = ctx.phys_sm.insert(PhysNode::new(output_schema, node_kind));
+                input_streams.insert(PhysStream::first(node_key));
+                transformed_exprs.push(ctx.expr_arena.add(AExpr::Column(out_name)));
+            },
+
             AExpr::BinaryExpr { left, op, right } => {
                 let (trans_input, trans_exprs) = lower_exprs_with_ctx(input, &[left, right], ctx)?;
                 let bin_expr = AExpr::BinaryExpr {
@@ -702,6 +880,28 @@ fn lower_exprs_with_ctx(
                 input_streams.insert(trans_input);
                 transformed_exprs.push(ctx.expr_arena.add(bin_expr));
             },
+            AExpr::Eval {
+                expr: inner,
+                evaluation,
+                variant,
+            } => match variant {
+                EvalVariant::List => {
+                    let (trans_input, trans_expr) = lower_exprs_with_ctx(input, &[inner], ctx)?;
+                    let eval_expr = AExpr::Eval {
+                        expr: trans_expr[0],
+                        evaluation,
+                        variant,
+                    };
+                    input_streams.insert(trans_input);
+                    transformed_exprs.push(ctx.expr_arena.add(eval_expr));
+                },
+                EvalVariant::Cumulative { .. } => {
+                    // Cumulative is not elementwise, this would need a special node.
+                    let out_name = unique_column_name();
+                    fallback_subset.push(ExprIR::new(expr, OutputName::Alias(out_name.clone())));
+                    transformed_exprs.push(ctx.expr_arena.add(AExpr::Column(out_name)));
+                },
+            },
             AExpr::Ternary {
                 predicate,
                 truthy,
@@ -739,7 +939,7 @@ fn lower_exprs_with_ctx(
                 let sorted_name = unique_column_name();
                 let inner_expr_ir = ExprIR::new(inner, OutputName::Alias(sorted_name.clone()));
                 let select_stream =
-                    build_select_stream_with_ctx(input, &[inner_expr_ir.clone()], ctx)?;
+                    build_select_stream_with_ctx(input, std::slice::from_ref(&inner_expr_ir), ctx)?;
                 let col_expr = ctx.expr_arena.add(AExpr::Column(sorted_name.clone()));
                 let kind = PhysNodeKind::Sort {
                     input: select_stream,
@@ -818,37 +1018,22 @@ fn lower_exprs_with_ctx(
                 input_streams.insert(PhysStream::first(filter_node_key));
                 transformed_exprs.push(ctx.expr_arena.add(AExpr::Column(out_name)));
             },
-            AExpr::Agg(mut agg) => match agg {
-                // Change agg mutably so we can share the codepath for all of these.
-                IRAggExpr::Min {
-                    input: ref mut inner,
-                    ..
-                }
-                | IRAggExpr::Max {
-                    input: ref mut inner,
-                    ..
-                }
-                | IRAggExpr::First(ref mut inner)
-                | IRAggExpr::Last(ref mut inner)
-                | IRAggExpr::Sum(ref mut inner)
-                | IRAggExpr::Mean(ref mut inner)
-                | IRAggExpr::Var(ref mut inner, _ /* ddof */)
-                | IRAggExpr::Std(ref mut inner, _ /* ddof */)
-                | IRAggExpr::Count(ref mut inner, _ /* count_nulls */) => {
-                    let (trans_input, trans_exprs) = lower_exprs_with_ctx(input, &[*inner], ctx)?;
-                    *inner = trans_exprs[0];
 
-                    let out_name = unique_column_name();
-                    let trans_agg_expr = ctx.expr_arena.add(AExpr::Agg(agg));
-                    let expr_ir = ExprIR::new(trans_agg_expr, OutputName::Alias(out_name.clone()));
-                    let output_schema = schema_for_select(trans_input, &[expr_ir.clone()], ctx)?;
-                    let kind = PhysNodeKind::Reduce {
-                        input: trans_input,
-                        exprs: vec![expr_ir],
-                    };
-                    let reduce_node_key = ctx.phys_sm.insert(PhysNode::new(output_schema, kind));
-                    input_streams.insert(PhysStream::first(reduce_node_key));
-                    transformed_exprs.push(ctx.expr_arena.add(AExpr::Column(out_name)));
+            // Aggregates.
+            AExpr::Agg(agg) => match agg {
+                // Change agg mutably so we can share the codepath for all of these.
+                IRAggExpr::Min { .. }
+                | IRAggExpr::Max { .. }
+                | IRAggExpr::First(_)
+                | IRAggExpr::Last(_)
+                | IRAggExpr::Sum(_)
+                | IRAggExpr::Mean(_)
+                | IRAggExpr::Var { .. }
+                | IRAggExpr::Std { .. }
+                | IRAggExpr::Count { .. } => {
+                    let (trans_stream, trans_expr) = lower_unary_reduce_node(input, expr, ctx)?;
+                    input_streams.insert(trans_stream);
+                    transformed_exprs.push(trans_expr);
                 },
                 IRAggExpr::NUnique(inner) => {
                     // Lower to no-aggregate group-by with unique name feeding into len aggregate.
@@ -857,8 +1042,11 @@ fn lower_exprs_with_ctx(
                         lower_exprs_with_ctx(input, &[inner], ctx)?;
                     let group_by_key_expr =
                         ExprIR::new(trans_inner_exprs[0], OutputName::Alias(tmp_name.clone()));
-                    let group_by_output_schema =
-                        schema_for_select(trans_input, &[group_by_key_expr.clone()], ctx)?;
+                    let group_by_output_schema = schema_for_select(
+                        trans_input,
+                        std::slice::from_ref(&group_by_key_expr),
+                        ctx,
+                    )?;
                     let group_by_stream = build_group_by_stream(
                         trans_input,
                         &[group_by_key_expr],
@@ -870,12 +1058,16 @@ fn lower_exprs_with_ctx(
                         ctx.expr_arena,
                         ctx.phys_sm,
                         ctx.cache,
+                        StreamingLowerIRContext::from(&*ctx),
                     )?;
 
                     let len_node = ctx.expr_arena.add(AExpr::Len);
                     let len_expr_ir = ExprIR::new(len_node, OutputName::Alias(tmp_name.clone()));
-                    let output_schema =
-                        schema_for_select(group_by_stream, &[len_expr_ir.clone()], ctx)?;
+                    let output_schema = schema_for_select(
+                        group_by_stream,
+                        std::slice::from_ref(&len_expr_ir),
+                        ctx,
+                    )?;
                     let kind = PhysNodeKind::Reduce {
                         input: group_by_stream,
                         exprs: vec![len_expr_ir],
@@ -894,10 +1086,37 @@ fn lower_exprs_with_ctx(
                     transformed_exprs.push(ctx.expr_arena.add(AExpr::Column(out_name)));
                 },
             },
+
+            #[cfg(feature = "bitwise")]
+            AExpr::Function {
+                function:
+                    IRFunctionExpr::Bitwise(
+                        IRBitwiseFunction::And | IRBitwiseFunction::Or | IRBitwiseFunction::Xor,
+                    ),
+                ..
+            } => {
+                let (trans_stream, trans_expr) = lower_unary_reduce_node(input, expr, ctx)?;
+                input_streams.insert(trans_stream);
+                transformed_exprs.push(trans_expr);
+            },
+
+            AExpr::Function {
+                function:
+                    IRFunctionExpr::Boolean(
+                        IRBooleanFunction::Any { .. } | IRBooleanFunction::All { .. },
+                    ),
+                ..
+            } => {
+                let (trans_stream, trans_expr) = lower_unary_reduce_node(input, expr, ctx)?;
+                input_streams.insert(trans_stream);
+                transformed_exprs.push(trans_expr);
+            },
+
+            // Length-based expressions.
             AExpr::Len => {
                 let out_name = unique_column_name();
                 let expr_ir = ExprIR::new(expr, OutputName::Alias(out_name.clone()));
-                let output_schema = schema_for_select(input, &[expr_ir.clone()], ctx)?;
+                let output_schema = schema_for_select(input, std::slice::from_ref(&expr_ir), ctx)?;
                 let kind = PhysNodeKind::Reduce {
                     input,
                     exprs: vec![expr_ir],
@@ -906,9 +1125,75 @@ fn lower_exprs_with_ctx(
                 input_streams.insert(PhysStream::first(reduce_node_key));
                 transformed_exprs.push(ctx.expr_arena.add(AExpr::Column(out_name)));
             },
+
+            // pl.row_index() maps to this.
+            #[cfg(feature = "range")]
+            AExpr::Function {
+                input: ref inner_exprs,
+                function: IRFunctionExpr::Range(IRRangeFunction::IntRange { step: 1, dtype }),
+                options: _,
+            } if {
+                let start_is_zero = match ctx.expr_arena.get(inner_exprs[0].node()) {
+                    AExpr::Literal(lit) => lit.extract_usize().ok() == Some(0),
+                    _ => false,
+                };
+                let stop_is_len = matches!(ctx.expr_arena.get(inner_exprs[1].node()), AExpr::Len);
+
+                dtype == DataType::IDX_DTYPE && start_is_zero && stop_is_len
+            } =>
+            {
+                let out_name = unique_column_name();
+                let input_schema = &ctx.phys_sm[input.node].output_schema;
+                let mut output_schema = (**input_schema).clone();
+                output_schema
+                    .insert_at_index(0, out_name.clone(), DataType::IDX_DTYPE)
+                    .unwrap();
+                let kind = PhysNodeKind::WithRowIndex {
+                    input,
+                    name: out_name.clone(),
+                    offset: None,
+                };
+                let with_row_idx_node_key = ctx
+                    .phys_sm
+                    .insert(PhysNode::new(Arc::new(output_schema), kind));
+                let row_idx_col_aexpr = ctx.expr_arena.add(AExpr::Column(out_name.clone()));
+                let row_idx_col_expr_ir =
+                    ExprIR::new(row_idx_col_aexpr, OutputName::ColumnLhs(out_name));
+                let row_idx_stream = build_select_stream_with_ctx(
+                    PhysStream::first(with_row_idx_node_key),
+                    &[row_idx_col_expr_ir],
+                    ctx,
+                )?;
+                input_streams.insert(row_idx_stream);
+                transformed_exprs.push(row_idx_col_aexpr);
+            },
+
+            AExpr::Slice {
+                input: inner,
+                offset,
+                length,
+            } => {
+                let out_name = unique_column_name();
+                let inner_expr_ir = ExprIR::new(inner, OutputName::Alias(out_name.clone()));
+                let offset_expr_ir = ExprIR::from_node(offset, ctx.expr_arena);
+                let length_expr_ir = ExprIR::from_node(length, ctx.expr_arena);
+                let input_stream = build_select_stream_with_ctx(input, &[inner_expr_ir], ctx)?;
+                let offset_stream = build_select_stream_with_ctx(input, &[offset_expr_ir], ctx)?;
+                let length_stream = build_select_stream_with_ctx(input, &[length_expr_ir], ctx)?;
+
+                let output_schema = ctx.phys_sm[input_stream.node].output_schema.clone();
+                let kind = PhysNodeKind::DynamicSlice {
+                    input: input_stream,
+                    offset: offset_stream,
+                    length: length_stream,
+                };
+                let slice_node_key = ctx.phys_sm.insert(PhysNode::new(output_schema, kind));
+                input_streams.insert(PhysStream::first(slice_node_key));
+                transformed_exprs.push(ctx.expr_arena.add(AExpr::Column(out_name)));
+            },
+
             AExpr::AnonymousFunction { .. }
             | AExpr::Function { .. }
-            | AExpr::Slice { .. }
             | AExpr::Window { .. }
             | AExpr::Gather { .. } => {
                 let out_name = unique_column_name();
@@ -1048,11 +1333,13 @@ pub fn lower_exprs(
     expr_arena: &mut Arena<AExpr>,
     phys_sm: &mut SlotMap<PhysNodeKey, PhysNode>,
     expr_cache: &mut ExprCache,
+    ctx: StreamingLowerIRContext,
 ) -> PolarsResult<(PhysStream, Vec<ExprIR>)> {
     let mut ctx = LowerExprContext {
         expr_arena,
         phys_sm,
         cache: expr_cache,
+        prepare_visualization: ctx.prepare_visualization,
     };
     let node_exprs = exprs.iter().map(|e| e.node()).collect_vec();
     let (transformed_input, transformed_exprs) =
@@ -1073,11 +1360,13 @@ pub fn build_select_stream(
     expr_arena: &mut Arena<AExpr>,
     phys_sm: &mut SlotMap<PhysNodeKey, PhysNode>,
     expr_cache: &mut ExprCache,
+    ctx: StreamingLowerIRContext,
 ) -> PolarsResult<PhysStream> {
     let mut ctx = LowerExprContext {
         expr_arena,
         phys_sm,
         cache: expr_cache,
+        prepare_visualization: ctx.prepare_visualization,
     };
     build_select_stream_with_ctx(input, exprs, &mut ctx)
 }
@@ -1090,11 +1379,13 @@ pub fn build_length_preserving_select_stream(
     expr_arena: &mut Arena<AExpr>,
     phys_sm: &mut SlotMap<PhysNodeKey, PhysNode>,
     expr_cache: &mut ExprCache,
+    ctx: StreamingLowerIRContext,
 ) -> PolarsResult<PhysStream> {
     let mut ctx = LowerExprContext {
         expr_arena,
         phys_sm,
         cache: expr_cache,
+        prepare_visualization: ctx.prepare_visualization,
     };
     let already_length_preserving = exprs
         .iter()
