@@ -7,15 +7,17 @@ use polars_error::{PolarsResult, polars_err};
 use polars_expr::state::ExecutionState;
 use polars_mem_engine::create_physical_plan;
 use polars_plan::plans::expr_ir::{ExprIR, OutputName};
-use polars_plan::plans::{AExpr, DataFrameUdf, IR, IRAggExpr, NaiveExprMerger};
-use polars_plan::prelude::GroupbyOptions;
+use polars_plan::plans::{
+    AExpr, DataFrameUdf, IR, IRAggExpr, IRFunctionExpr, NaiveExprMerger, write_group_by,
+};
+use polars_plan::prelude::{GroupbyOptions, *};
 use polars_utils::arena::{Arena, Node};
 use polars_utils::pl_str::PlSmallStr;
 use polars_utils::unique_column_name;
 use recursive::recursive;
 use slotmap::SlotMap;
 
-use super::{ExprCache, PhysNode, PhysNodeKey, PhysNodeKind, PhysStream};
+use super::{ExprCache, PhysNode, PhysNodeKey, PhysNodeKind, PhysStream, StreamingLowerIRContext};
 use crate::physical_plan::lower_expr::{
     build_select_stream, compute_output_schema, is_elementwise_rec_cached,
     is_fake_elementwise_function, is_input_independent,
@@ -34,11 +36,12 @@ fn build_group_by_fallback(
     apply: Option<Arc<dyn DataFrameUdf>>,
     expr_arena: &mut Arena<AExpr>,
     phys_sm: &mut SlotMap<PhysNodeKey, PhysNode>,
+    format_str: Option<String>,
 ) -> PolarsResult<PhysStream> {
     let input_schema = phys_sm[input.node].output_schema.clone();
     let lmdf = Arc::new(LateMaterializedDataFrame::default());
     let mut lp_arena = Arena::default();
-    let input_lp_node = lp_arena.add(lmdf.clone().as_ir_node(input_schema.clone()));
+    let input_lp_node = lp_arena.add(lmdf.clone().as_ir_node(input_schema));
     let group_by_lp_node = lp_arena.add(IR::GroupBy {
         input: input_lp_node,
         keys: keys.to_vec(),
@@ -64,6 +67,7 @@ fn build_group_by_fallback(
                 let mut state = ExecutionState::new();
                 executor.lock().execute(&mut state)
             }),
+            format_str,
         },
     };
 
@@ -75,6 +79,7 @@ fn build_group_by_fallback(
 /// Such an expression is defined as the elementwise combination of scalar
 /// aggregations of elementwise combinations of the input columns / scalar literals.
 #[recursive]
+#[allow(clippy::too_many_arguments)]
 fn try_lower_elementwise_scalar_agg_expr(
     expr: Node,
     outer_name: Option<PlSmallStr>,
@@ -83,6 +88,7 @@ fn try_lower_elementwise_scalar_agg_expr(
     expr_arena: &mut Arena<AExpr>,
     agg_exprs: &mut Vec<ExprIR>,
     uniq_input_exprs: &mut PlIndexMap<u32, PlSmallStr>,
+    uniq_agg_exprs: &mut PlIndexMap<u32, PlSmallStr>,
 ) -> Option<Node> {
     // Helper macro to simplify recursive calls.
     macro_rules! lower_rec {
@@ -95,13 +101,12 @@ fn try_lower_elementwise_scalar_agg_expr(
                 expr_arena,
                 agg_exprs,
                 uniq_input_exprs,
+                uniq_agg_exprs,
             )
         };
     }
 
     match expr_arena.get(expr) {
-        AExpr::Alias(..) => unreachable!("alias found in physical plan"),
-
         AExpr::Column(_) => {
             // Implicit implode not yet supported.
             None
@@ -125,7 +130,7 @@ fn try_lower_elementwise_scalar_agg_expr(
         // in a streaming fashion but they change the length of the input which
         // means the same filter/explode should also be applied to the key
         // column, which is not (yet) supported.
-        AExpr::Explode(_) | AExpr::Filter { .. } => None,
+        AExpr::Explode { .. } | AExpr::Filter { .. } => None,
 
         AExpr::BinaryExpr { left, op, right } => {
             let (left, op, right) = (*left, *op, *right);
@@ -134,6 +139,20 @@ fn try_lower_elementwise_scalar_agg_expr(
             Some(expr_arena.add(AExpr::BinaryExpr { left, op, right }))
         },
 
+        AExpr::Eval {
+            expr,
+            evaluation,
+            variant,
+        } => {
+            let (expr, evaluation, variant) = (*expr, *evaluation, *variant);
+            let expr = lower_rec!(expr)?;
+            Some(expr_arena.add(AExpr::Eval {
+                expr,
+                evaluation,
+                variant,
+            }))
+        },
+
         AExpr::Ternary {
             predicate,
             truthy,
@@ -150,6 +169,117 @@ fn try_lower_elementwise_scalar_agg_expr(
             }))
         },
 
+        #[cfg(feature = "bitwise")]
+        AExpr::Function {
+            input: inner_exprs,
+            function:
+                IRFunctionExpr::Bitwise(
+                    inner_fn @ (IRBitwiseFunction::And
+                    | IRBitwiseFunction::Or
+                    | IRBitwiseFunction::Xor),
+                ),
+            options,
+        } => {
+            assert!(inner_exprs.len() == 1);
+
+            let input = inner_exprs[0].clone().node();
+            let inner_fn = *inner_fn;
+            let options = *options;
+
+            if is_input_independent(input, expr_arena, expr_cache) {
+                // TODO: we could simply return expr here, but we first need an is_scalar function, because if
+                // it is not a scalar we need to return expr.implode().
+                return None;
+            }
+
+            if !is_elementwise_rec_cached(input, expr_arena, expr_cache) {
+                return None;
+            }
+
+            let agg_id = expr_merger.get_uniq_id(expr).unwrap();
+            let name = uniq_agg_exprs
+                .entry(agg_id)
+                .or_insert_with(|| {
+                    let input_id = expr_merger.get_uniq_id(input).unwrap();
+                    let input_col = uniq_input_exprs
+                        .entry(input_id)
+                        .or_insert_with(unique_column_name)
+                        .clone();
+                    let input_col_node = expr_arena.add(AExpr::Column(input_col));
+                    let trans_agg_node = expr_arena.add(AExpr::Function {
+                        input: vec![ExprIR::from_node(input_col_node, expr_arena)],
+                        function: IRFunctionExpr::Bitwise(inner_fn),
+                        options,
+                    });
+
+                    // Add to aggregation expressions and replace with a reference to its output.
+                    let agg_expr = if let Some(name) = outer_name {
+                        ExprIR::new(trans_agg_node, OutputName::Alias(name))
+                    } else {
+                        ExprIR::new(trans_agg_node, OutputName::Alias(unique_column_name()))
+                    };
+                    agg_exprs.push(agg_expr.clone());
+                    agg_expr.output_name().clone()
+                })
+                .clone();
+            let result_node = expr_arena.add(AExpr::Column(name));
+            Some(result_node)
+        },
+
+        AExpr::Function {
+            input: inner_exprs,
+            function:
+                IRFunctionExpr::Boolean(
+                    inner_fn @ (IRBooleanFunction::Any { .. } | IRBooleanFunction::All { .. }),
+                ),
+            options,
+        } => {
+            assert!(inner_exprs.len() == 1);
+
+            let input = inner_exprs[0].clone().node();
+            let inner_fn = inner_fn.clone();
+            let options = *options;
+
+            if is_input_independent(input, expr_arena, expr_cache) {
+                // TODO: we could simply return expr here, but we first need an is_scalar function, because if
+                // it is not a scalar we need to return expr.implode().
+                return None;
+            }
+
+            if !is_elementwise_rec_cached(input, expr_arena, expr_cache) {
+                return None;
+            }
+
+            let agg_id = expr_merger.get_uniq_id(expr).unwrap();
+            let name = uniq_agg_exprs
+                .entry(agg_id)
+                .or_insert_with(|| {
+                    let input_id = expr_merger.get_uniq_id(input).unwrap();
+                    let input_col = uniq_input_exprs
+                        .entry(input_id)
+                        .or_insert_with(unique_column_name)
+                        .clone();
+                    let input_col_node = expr_arena.add(AExpr::Column(input_col));
+                    let trans_agg_node = expr_arena.add(AExpr::Function {
+                        input: vec![ExprIR::from_node(input_col_node, expr_arena)],
+                        function: IRFunctionExpr::Boolean(inner_fn),
+                        options,
+                    });
+
+                    // Add to aggregation expressions and replace with a reference to its output.
+                    let agg_expr = if let Some(name) = outer_name {
+                        ExprIR::new(trans_agg_node, OutputName::Alias(name))
+                    } else {
+                        ExprIR::new(trans_agg_node, OutputName::Alias(unique_column_name()))
+                    };
+                    agg_exprs.push(agg_expr.clone());
+                    agg_expr.output_name().clone()
+                })
+                .clone();
+            let result_node = expr_arena.add(AExpr::Column(name));
+            Some(result_node)
+        },
+
         node @ AExpr::Function { input, options, .. }
         | node @ AExpr::AnonymousFunction { input, options, .. }
             if options.is_elementwise() && !is_fake_elementwise_function(node) =>
@@ -168,7 +298,7 @@ fn try_lower_elementwise_scalar_agg_expr(
                 })
                 .collect::<Option<Vec<_>>>()?;
 
-            let mut new_node = node.clone();
+            let mut new_node = node;
             match &mut new_node {
                 AExpr::Function { input, .. } | AExpr::AnonymousFunction { input, .. } => {
                     *input = new_input;
@@ -205,34 +335,44 @@ fn try_lower_elementwise_scalar_agg_expr(
                 | IRAggExpr::Var(input, ..)
                 | IRAggExpr::Std(input, ..)
                 | IRAggExpr::Count(input, ..) => {
-                    if is_input_independent(*input, expr_arena, expr_cache) {
+                    let agg = agg.clone();
+                    let input = *input;
+                    if is_input_independent(input, expr_arena, expr_cache) {
                         // TODO: we could simply return expr here, but we first need an is_scalar function, because if
                         // it is not a scalar we need to return expr.implode().
                         return None;
                     }
 
-                    if !is_elementwise_rec_cached(*input, expr_arena, expr_cache) {
+                    if !is_elementwise_rec_cached(input, expr_arena, expr_cache) {
                         return None;
                     }
 
-                    let mut trans_agg = agg.clone();
-                    let input_id = expr_merger.get_uniq_id(*input).unwrap();
-                    let input_col = uniq_input_exprs
-                        .entry(input_id)
-                        .or_insert_with(unique_column_name)
-                        .clone();
-                    let input_col_node = expr_arena.add(AExpr::Column(input_col.clone()));
-                    trans_agg.set_input(input_col_node);
-                    let trans_agg_node = expr_arena.add(AExpr::Agg(trans_agg));
+                    let agg_id = expr_merger.get_uniq_id(expr).unwrap();
+                    let name = uniq_agg_exprs
+                        .entry(agg_id)
+                        .or_insert_with(|| {
+                            let mut trans_agg = agg;
+                            let input_id = expr_merger.get_uniq_id(input).unwrap();
+                            let input_col = uniq_input_exprs
+                                .entry(input_id)
+                                .or_insert_with(unique_column_name)
+                                .clone();
+                            let input_col_node = expr_arena.add(AExpr::Column(input_col));
+                            trans_agg.set_input(input_col_node);
+                            let trans_agg_node = expr_arena.add(AExpr::Agg(trans_agg));
 
-                    // Add to aggregation expressions and replace with a reference to its output.
-                    let agg_expr = if let Some(name) = outer_name {
-                        ExprIR::new(trans_agg_node, OutputName::Alias(name))
-                    } else {
-                        ExprIR::new(trans_agg_node, OutputName::Alias(unique_column_name()))
-                    };
-                    let result_node = expr_arena.add(AExpr::Column(agg_expr.output_name().clone()));
-                    agg_exprs.push(agg_expr);
+                            // Add to aggregation expressions and replace with a reference to its output.
+                            let agg_expr = if let Some(name) = outer_name {
+                                ExprIR::new(trans_agg_node, OutputName::Alias(name))
+                            } else {
+                                ExprIR::new(trans_agg_node, OutputName::Alias(unique_column_name()))
+                            };
+                            agg_exprs.push(agg_expr.clone());
+                            agg_expr.output_name().clone()
+                        })
+                        .clone();
+
+                    let result_node = expr_arena.add(AExpr::Column(name));
                     Some(result_node)
                 },
                 IRAggExpr::Median(..)
@@ -266,6 +406,7 @@ fn try_build_streaming_group_by(
     expr_arena: &mut Arena<AExpr>,
     phys_sm: &mut SlotMap<PhysNodeKey, PhysNode>,
     expr_cache: &mut ExprCache,
+    ctx: StreamingLowerIRContext,
 ) -> Option<PolarsResult<PhysStream>> {
     if apply.is_some() || maintain_order {
         return None; // TODO
@@ -323,6 +464,8 @@ fn try_build_streaming_group_by(
         let trans_output_node = expr_arena.add(AExpr::Column(uniq_name));
         trans_output_exprs.push(ExprIR::new(trans_output_node, output_name));
     }
+
+    let mut uniq_agg_exprs = PlIndexMap::new();
     for agg in aggs {
         let trans_node = try_lower_elementwise_scalar_agg_expr(
             agg.node(),
@@ -332,6 +475,7 @@ fn try_build_streaming_group_by(
             expr_arena,
             &mut trans_agg_exprs,
             &mut uniq_input_exprs,
+            &mut uniq_agg_exprs,
         )?;
         let output_name = OutputName::Alias(agg.output_name().clone());
         trans_output_exprs.push(ExprIR::new(trans_node, output_name));
@@ -345,7 +489,7 @@ fn try_build_streaming_group_by(
     }
 
     let pre_select =
-        build_select_stream(input, &input_exprs, expr_arena, phys_sm, expr_cache).ok()?;
+        build_select_stream(input, &input_exprs, expr_arena, phys_sm, expr_cache, ctx).ok()?;
 
     let input_schema = &phys_sm[pre_select.node].output_schema;
     let group_by_output_schema = compute_output_schema(
@@ -369,6 +513,7 @@ fn try_build_streaming_group_by(
         expr_arena,
         phys_sm,
         expr_cache,
+        ctx,
     );
     let out = if let Some((offset, len)) = options.slice {
         post_select.map(|s| build_slice_stream(s, offset, len, phys_sm))
@@ -390,6 +535,7 @@ pub fn build_group_by_stream(
     expr_arena: &mut Arena<AExpr>,
     phys_sm: &mut SlotMap<PhysNodeKey, PhysNode>,
     expr_cache: &mut ExprCache,
+    ctx: StreamingLowerIRContext,
 ) -> PolarsResult<PhysStream> {
     let streaming = try_build_streaming_group_by(
         input,
@@ -401,10 +547,25 @@ pub fn build_group_by_stream(
         expr_arena,
         phys_sm,
         expr_cache,
+        ctx,
     );
     if let Some(stream) = streaming {
         stream
     } else {
+        let format_str = ctx.prepare_visualization.then(|| {
+            let mut buffer = String::new();
+            write_group_by(
+                &mut buffer,
+                0,
+                expr_arena,
+                keys,
+                aggs,
+                apply.as_deref(),
+                maintain_order,
+            )
+            .unwrap();
+            buffer
+        });
         build_group_by_fallback(
             input,
             keys,
@@ -415,6 +576,7 @@ pub fn build_group_by_stream(
             apply,
             expr_arena,
             phys_sm,
+            format_str,
         )
     }
 }
