@@ -1,4 +1,3 @@
-use std::path::PathBuf;
 use std::sync::Arc;
 
 use polars_core::frame::DataFrame;
@@ -8,9 +7,10 @@ use polars_error::PolarsResult;
 use polars_io::RowIndex;
 use polars_io::cloud::CloudOptions;
 use polars_ops::frame::JoinArgs;
+use polars_plan::dsl::deletion::DeletionFilesList;
 use polars_plan::dsl::{
     CastColumnsPolicy, JoinTypeOptionsIR, MissingColumnsPolicy, PartitionTargetCallback,
-    PartitionVariantIR, ScanSources, SinkOptions, SinkTarget,
+    PartitionVariantIR, ScanSources, SinkFinishCallback, SinkOptions, SinkTarget, SortColumnIR,
 };
 use polars_plan::plans::hive::HivePartitionsDf;
 use polars_plan::plans::{AExpr, DataFrameUdf, IR};
@@ -24,14 +24,17 @@ mod lower_ir;
 mod to_graph;
 
 pub use fmt::visualize_plan;
-use polars_plan::dsl::ExtraColumnsPolicy;
 use polars_plan::prelude::FileType;
 use polars_utils::arena::{Arena, Node};
 use polars_utils::pl_str::PlSmallStr;
+use polars_utils::plpath::PlPath;
 use polars_utils::slice_enum::Slice;
 use slotmap::{SecondaryMap, SlotMap};
 pub use to_graph::physical_plan_to_graph;
 
+pub use self::lower_ir::StreamingLowerIRContext;
+use crate::nodes::io_sources::multi_file_reader::extra_ops::ForbidExtraColumns;
+use crate::nodes::io_sources::multi_file_reader::initialization::projection::ProjectionBuilder;
 use crate::nodes::io_sources::multi_file_reader::reader_interface::builder::FileReaderBuilder;
 use crate::physical_plan::lower_expr::ExprCache;
 
@@ -123,6 +126,12 @@ pub enum PhysNodeKind {
         length: usize,
     },
 
+    DynamicSlice {
+        input: PhysStream,
+        offset: PhysStream,
+        length: PhysStream,
+    },
+
     Filter {
         input: PhysStream,
         predicate: ExprIR,
@@ -146,13 +155,15 @@ pub enum PhysNodeKind {
     },
 
     PartitionSink {
-        base_path: Arc<PathBuf>,
+        input: PhysStream,
+        base_path: Arc<PlPath>,
         file_path_cb: Option<PartitionTargetCallback>,
         sink_options: SinkOptions,
         variant: PartitionVariantIR,
         file_type: FileType,
-        input: PhysStream,
         cloud_options: Option<CloudOptions>,
+        per_partition_sort_by: Option<Vec<SortColumnIR>>,
+        finish_callback: Option<SinkFinishCallback>,
     },
 
     SinkMultiple {
@@ -165,6 +176,9 @@ pub enum PhysNodeKind {
     InMemoryMap {
         input: PhysStream,
         map: Arc<dyn DataFrameUdf>,
+
+        /// A formatted explain of what the in-memory map. This usually calls format on the IR.
+        format_str: Option<String>,
     },
 
     Map {
@@ -179,6 +193,11 @@ pub enum PhysNodeKind {
         sort_options: SortMultipleOptions,
     },
 
+    Repeat {
+        value: PhysStream,
+        repeats: PhysStream,
+    },
+
     OrderedUnion {
         inputs: Vec<PhysStream>,
     },
@@ -203,7 +222,7 @@ pub enum PhysNodeKind {
         cloud_options: Option<Arc<CloudOptions>>,
 
         /// Columns to project from the file.
-        projected_file_schema: SchemaRef,
+        file_projection_builder: ProjectionBuilder,
         /// Final output schema of morsels being sent out of MultiScan.
         output_schema: SchemaRef,
 
@@ -215,7 +234,9 @@ pub enum PhysNodeKind {
         include_file_paths: Option<PlSmallStr>,
         cast_columns_policy: CastColumnsPolicy,
         missing_columns_policy: MissingColumnsPolicy,
-        extra_columns_policy: ExtraColumnsPolicy,
+        forbid_extra_columns: Option<ForbidExtraColumns>,
+
+        deletion_files: Option<DeletionFilesList>,
 
         /// Schema of columns contained in the file. Does not contain external columns (e.g. hive / row_index).
         file_schema: SchemaRef,
@@ -250,6 +271,12 @@ pub enum PhysNodeKind {
         output_bool: bool,
     },
 
+    CrossJoin {
+        input_left: PhysStream,
+        input_right: PhysStream,
+        args: JoinArgs,
+    },
+
     /// Generic fallback for (as-of-yet) unsupported streaming joins.
     /// Fully sinks all data to in-memory data frames and uses the in-memory
     /// engine to perform the join.
@@ -327,6 +354,11 @@ fn visit_node_inputs_mut(
                 input_left,
                 input_right,
                 ..
+            }
+            | PhysNodeKind::CrossJoin {
+                input_left,
+                input_right,
+                ..
             } => {
                 rec!(input_left.node);
                 rec!(input_right.node);
@@ -346,6 +378,26 @@ fn visit_node_inputs_mut(
                 visit(input_right);
             },
 
+            PhysNodeKind::DynamicSlice {
+                input,
+                offset,
+                length,
+            } => {
+                rec!(input.node);
+                rec!(offset.node);
+                rec!(length.node);
+                visit(input);
+                visit(offset);
+                visit(length);
+            },
+
+            PhysNodeKind::Repeat { value, repeats } => {
+                rec!(value.node);
+                rec!(repeats.node);
+                visit(value);
+                visit(repeats);
+            },
+
             PhysNodeKind::OrderedUnion { inputs } | PhysNodeKind::Zip { inputs, .. } => {
                 for input in inputs {
                     rec!(input.node);
@@ -395,6 +447,7 @@ pub fn build_physical_plan(
     ir_arena: &mut Arena<IR>,
     expr_arena: &mut Arena<AExpr>,
     phys_sm: &mut SlotMap<PhysNodeKey, PhysNode>,
+    ctx: StreamingLowerIRContext,
 ) -> PolarsResult<PhysNodeKey> {
     let mut schema_cache = PlHashMap::with_capacity(ir_arena.len());
     let mut expr_cache = ExprCache::with_capacity(expr_arena.len());
@@ -407,6 +460,7 @@ pub fn build_physical_plan(
         &mut schema_cache,
         &mut expr_cache,
         &mut cache_nodes,
+        ctx,
     )?;
     insert_multiplexers(vec![phys_root.node], phys_sm);
     Ok(phys_root.node)
