@@ -5,7 +5,13 @@
 //! - Compact activated with `FC: false`
 use polars_error::{PolarsResult, to_compute_err};
 
-fn serialize_impl<W, T, const FC: bool>(writer: W, value: &T) -> PolarsResult<()>
+fn config() -> bincode::config::Configuration {
+    bincode::config::standard()
+        .with_no_limit()
+        .with_variable_int_encoding()
+}
+
+fn serialize_impl<W, T, const FC: bool>(mut writer: W, value: &T) -> PolarsResult<()>
 where
     W: std::io::Write,
     T: serde::ser::Serialize,
@@ -14,11 +20,13 @@ where
         let mut s = rmp_serde::Serializer::new(writer).with_struct_map();
         value.serialize(&mut s).map_err(to_compute_err)
     } else {
-        bincode::serialize_into(writer, value).map_err(to_compute_err)
+        bincode::serde::encode_into_std_write(value, &mut writer, config())
+            .map_err(to_compute_err)
+            .map(|_| ())
     }
 }
 
-pub fn deserialize_impl<T, R, const FC: bool>(reader: R) -> PolarsResult<T>
+pub fn deserialize_impl<T, R, const FC: bool>(mut reader: R) -> PolarsResult<T>
 where
     T: serde::de::DeserializeOwned,
     R: std::io::Read,
@@ -26,7 +34,7 @@ where
     if FC {
         rmp_serde::from_read(reader).map_err(to_compute_err)
     } else {
-        bincode::deserialize_from(reader).map_err(to_compute_err)
+        bincode::serde::decode_from_std_read(&mut reader, config()).map_err(to_compute_err)
     }
 }
 
@@ -118,6 +126,29 @@ where
     Ok(v)
 }
 
+/// Serialize function customized for `DslPlan`, with stack overflow protection.
+pub fn serialize_dsl<W, T>(writer: W, value: &T) -> PolarsResult<()>
+where
+    W: std::io::Write,
+    T: serde::ser::Serialize,
+{
+    let mut s = rmp_serde::Serializer::new(writer).with_struct_map();
+    let s = serde_stacker::Serializer::new(&mut s);
+    value.serialize(s).map_err(to_compute_err)
+}
+
+/// Deserialize function customized for `DslPlan`, with stack overflow protection.
+pub fn deserialize_dsl<T, R>(reader: R) -> PolarsResult<T>
+where
+    T: serde::de::DeserializeOwned,
+    R: std::io::Read,
+{
+    let mut de = rmp_serde::Deserializer::new(reader);
+    de.set_max_depth(usize::MAX);
+    let de = serde_stacker::Deserializer::new(&mut de);
+    T::deserialize(de).map_err(to_compute_err)
+}
+
 /// Potentially avoids copying memory compared to a naive `Vec::<u8>::deserialize`.
 ///
 /// This is essentially boilerplate for visiting bytes without copying where possible.
@@ -173,6 +204,81 @@ where
     }
 }
 
+thread_local! {
+    pub static USE_CLOUDPICKLE: std::cell::Cell<bool> = const { std::cell::Cell::new(false) };
+}
+
+#[cfg(feature = "python")]
+pub fn python_object_serialize(
+    pyobj: &pyo3::Py<pyo3::PyAny>,
+    buf: &mut Vec<u8>,
+) -> PolarsResult<()> {
+    use pyo3::Python;
+    use pyo3::pybacked::PyBackedBytes;
+    use pyo3::types::{PyAnyMethods, PyModule};
+
+    use crate::python_function::PYTHON3_VERSION;
+
+    let mut use_cloudpickle = USE_CLOUDPICKLE.get();
+    let dumped = Python::with_gil(|py| {
+        // Pickle with whatever pickling method was selected.
+        if use_cloudpickle {
+            let cloudpickle = PyModule::import(py, "cloudpickle")?.getattr("dumps")?;
+            cloudpickle.call1((pyobj.clone_ref(py),))?
+        } else {
+            let pickle = PyModule::import(py, "pickle")?.getattr("dumps")?;
+            match pickle.call1((pyobj.clone_ref(py),)) {
+                Ok(dumped) => dumped,
+                Err(_) => {
+                    use_cloudpickle = true;
+                    let cloudpickle = PyModule::import(py, "cloudpickle")?.getattr("dumps")?;
+                    cloudpickle.call1((pyobj.clone_ref(py),))?
+                },
+            }
+        }
+        .extract::<PyBackedBytes>()
+    })?;
+
+    // Write pickle metadata
+    buf.push(use_cloudpickle as u8);
+    buf.extend_from_slice(&*PYTHON3_VERSION);
+
+    // Write UDF
+    buf.extend_from_slice(&dumped);
+    Ok(())
+}
+
+#[cfg(feature = "python")]
+pub fn python_object_deserialize(buf: &[u8]) -> PolarsResult<pyo3::Py<pyo3::PyAny>> {
+    use polars_error::polars_ensure;
+    use pyo3::Python;
+    use pyo3::types::{PyAnyMethods, PyBytes, PyModule};
+
+    use crate::python_function::PYTHON3_VERSION;
+
+    // Handle pickle metadata
+    let use_cloudpickle = buf[0] != 0;
+    if use_cloudpickle {
+        let ser_py_version = &buf[1..3];
+        let cur_py_version = *PYTHON3_VERSION;
+        polars_ensure!(
+            ser_py_version == cur_py_version,
+            InvalidOperation:
+            "current Python version {:?} does not match the Python version used to serialize the UDF {:?}",
+            (3, cur_py_version[0], cur_py_version[1]),
+            (3, ser_py_version[0], ser_py_version[1] )
+        );
+    }
+    let buf = &buf[3..];
+
+    Python::with_gil(|py| {
+        let loads = PyModule::import(py, "pickle")?.getattr("loads")?;
+        let arg = (PyBytes::new(py, buf),);
+        let python_function = loads.call1(arg)?;
+        Ok(python_function.into())
+    })
+}
+
 #[cfg(test)]
 mod tests {
     #[test]
