@@ -10,9 +10,14 @@ use allocator::create_allocator_capsule;
 use polars_python::batched_csv::PyBatchedCsv;
 #[cfg(feature = "catalog")]
 use polars_python::catalog::unity::PyCatalogClient;
-#[cfg(feature = "polars_cloud")]
-use polars_python::cloud;
+#[cfg(feature = "polars_cloud_client")]
+use polars_python::cloud_client;
+#[cfg(feature = "polars_cloud_server")]
+use polars_python::cloud_server;
+use polars_python::conversion::PyCategories;
 use polars_python::dataframe::PyDataFrame;
+use polars_python::expr::datatype::PyDataTypeExpr;
+use polars_python::expr::selector::PySelector;
 use polars_python::expr::PyExpr;
 use polars_python::functions::PyStringCacheHolder;
 #[cfg(not(target_arch = "wasm32"))]
@@ -22,7 +27,7 @@ use polars_python::lazygroupby::PyLazyGroupBy;
 use polars_python::series::PySeries;
 #[cfg(feature = "sql")]
 use polars_python::sql::PySQLContext;
-use polars_python::{datatypes, exceptions, functions};
+use polars_python::{datatypes, exceptions, functions, testing};
 use pyo3::prelude::*;
 use pyo3::{wrap_pyfunction, wrap_pymodule};
 
@@ -77,6 +82,7 @@ fn _expr_nodes(_py: Python, m: &Bound<PyModule>) -> PyResult<()> {
     m.add_class::<PyStringFunction>().unwrap();
     m.add_class::<PyBooleanFunction>().unwrap();
     m.add_class::<PyTemporalFunction>().unwrap();
+    m.add_class::<PyStructFunction>().unwrap();
     // Options
     m.add_class::<PyWindowMapping>().unwrap();
     m.add_class::<PyRollingGroupOptions>().unwrap();
@@ -95,12 +101,15 @@ fn polars(py: Python, m: &Bound<PyModule>) -> PyResult<()> {
     m.add_class::<PyInProcessQuery>().unwrap();
     m.add_class::<PyLazyGroupBy>().unwrap();
     m.add_class::<PyExpr>().unwrap();
+    m.add_class::<PyDataTypeExpr>().unwrap();
+    m.add_class::<PySelector>().unwrap();
     m.add_class::<PyPartitioning>().unwrap();
     m.add_class::<PyStringCacheHolder>().unwrap();
     #[cfg(feature = "csv")]
     m.add_class::<PyBatchedCsv>().unwrap();
     #[cfg(feature = "sql")]
     m.add_class::<PySQLContext>().unwrap();
+    m.add_class::<PyCategories>().unwrap();
 
     // Submodules
     // LogicalPlan objects
@@ -177,7 +186,6 @@ fn polars(py: Python, m: &Bound<PyModule>) -> PyResult<()> {
         .unwrap();
     m.add_wrapped(wrap_pyfunction!(functions::collect_all_with_callback))
         .unwrap();
-    m.add_wrapped(wrap_pyfunction!(functions::cols)).unwrap();
     m.add_wrapped(wrap_pyfunction!(functions::concat_lf))
         .unwrap();
     m.add_wrapped(wrap_pyfunction!(functions::concat_arr))
@@ -202,18 +210,11 @@ fn polars(py: Python, m: &Bound<PyModule>) -> PyResult<()> {
         .unwrap();
     m.add_wrapped(wrap_pyfunction!(functions::concat_lf_horizontal))
         .unwrap();
-    m.add_wrapped(wrap_pyfunction!(functions::dtype_cols))
-        .unwrap();
-    m.add_wrapped(wrap_pyfunction!(functions::index_cols))
-        .unwrap();
     m.add_wrapped(wrap_pyfunction!(functions::duration))
         .unwrap();
-    m.add_wrapped(wrap_pyfunction!(functions::first)).unwrap();
     m.add_wrapped(wrap_pyfunction!(functions::fold)).unwrap();
-    m.add_wrapped(wrap_pyfunction!(functions::last)).unwrap();
     m.add_wrapped(wrap_pyfunction!(functions::lit)).unwrap();
     m.add_wrapped(wrap_pyfunction!(functions::map_mul)).unwrap();
-    m.add_wrapped(wrap_pyfunction!(functions::nth)).unwrap();
     m.add_wrapped(wrap_pyfunction!(functions::pearson_corr))
         .unwrap();
     m.add_wrapped(wrap_pyfunction!(functions::rolling_corr))
@@ -241,7 +242,7 @@ fn polars(py: Python, m: &Bound<PyModule>) -> PyResult<()> {
     m.add_wrapped(wrap_pyfunction!(functions::read_ipc_schema))
         .unwrap();
     #[cfg(feature = "parquet")]
-    m.add_wrapped(wrap_pyfunction!(functions::read_parquet_schema))
+    m.add_wrapped(wrap_pyfunction!(functions::read_parquet_metadata))
         .unwrap();
     #[cfg(feature = "clipboard")]
     m.add_wrapped(wrap_pyfunction!(functions::read_clipboard_string))
@@ -309,6 +310,12 @@ fn polars(py: Python, m: &Bound<PyModule>) -> PyResult<()> {
     m.add_wrapped(wrap_pyfunction!(datatypes::_known_timezones))
         .unwrap();
 
+    // Testing
+    m.add_wrapped(wrap_pyfunction!(testing::assert_series_equal_py))
+        .unwrap();
+    m.add_wrapped(wrap_pyfunction!(testing::assert_dataframe_equal_py))
+        .unwrap();
+
     // Exceptions - Errors
     m.add("PolarsError", py.get_type::<exceptions::PolarsError>())
         .unwrap();
@@ -393,11 +400,11 @@ fn polars(py: Python, m: &Bound<PyModule>) -> PyResult<()> {
     .unwrap();
 
     // Cloud
-    #[cfg(feature = "polars_cloud")]
-    m.add_wrapped(wrap_pyfunction!(cloud::prepare_cloud_plan))
+    #[cfg(feature = "polars_cloud_client")]
+    m.add_wrapped(wrap_pyfunction!(cloud_client::prepare_cloud_plan))
         .unwrap();
-    #[cfg(feature = "polars_cloud")]
-    m.add_wrapped(wrap_pyfunction!(cloud::_execute_ir_plan_with_gpu))
+    #[cfg(feature = "polars_cloud_server")]
+    m.add_wrapped(wrap_pyfunction!(cloud_server::_execute_ir_plan_with_gpu))
         .unwrap();
 
     // Build info
