@@ -34,6 +34,8 @@ if TYPE_CHECKING:
     else:
         from typing_extensions import Self
 
+    from typing_extensions import assert_type
+
 
 # -----------------------------------------------------------------------------------
 # nested dataclasses, models, namedtuple classes (can't be defined inside test func)
@@ -256,7 +258,7 @@ def test_init_structured_objects() -> None:
         )
         assert df.schema == {
             "ts": pl.Datetime("ms"),
-            "tk": pl.Categorical(ordering="physical"),
+            "tk": pl.Categorical(ordering="lexical"),
             "pc": pl.Decimal(scale=1),
             "sz": pl.UInt16,
         }
@@ -960,7 +962,8 @@ def test_init_pandas(monkeypatch: Any) -> None:
     assert df.rows() == [(1.0, 2.0), (3.0, 4.0)]
 
     # subclassed pandas object, with/without data & overrides
-    class XSeries(pd.Series):  # type: ignore[type-arg]
+    # type error fixed in pandas-stubs 2.3.0.250703, which doesn't support Python3.9
+    class XSeries(pd.Series):  # type: ignore[type-arg, unused-ignore]
         @property
         def _constructor(self) -> type:
             return XSeries
@@ -1112,6 +1115,9 @@ def test_init_only_columns() -> None:
         assert df.dtypes == [pl.Date, pl.UInt64, pl.Int8, pl.List]
         assert pl.List(pl.UInt8).is_(df.schema["d"])
 
+        if TYPE_CHECKING:
+            assert_type(pl.List(pl.UInt8).is_(df.schema["d"]), bool)
+
         dfe = df.clear()
         assert len(dfe) == 0
         assert df.schema == dfe.schema
