@@ -1,3 +1,5 @@
+import json
+
 import polars as pl
 from polars.testing import assert_series_equal
 
@@ -28,3 +30,17 @@ def test_utf8_alias_lit() -> None:
     result = pl.select(a=pl.lit(5, dtype=pl.Utf8)).to_series()
     expected = pl.Series("a", ["5"], dtype=pl.String)
     assert_series_equal(result, expected)
+
+
+def test_json_decode_multiple_chunks() -> None:
+    a = json.dumps({"x": None})
+    b = json.dumps({"x": True})
+
+    df_1 = pl.Series([a]).to_frame("s")
+    df_2 = pl.Series([b]).to_frame("s")
+
+    df = pl.concat([df_1, df_2])
+
+    assert df.with_columns(pl.col("s").str.json_decode()).to_dict(as_series=False) == {
+        "s": [{"x": None}, {"x": True}]
+    }
