@@ -11,6 +11,7 @@ from hypothesis import given, settings
 from numpy.testing import assert_array_equal
 
 import polars as pl
+from polars.testing import assert_series_equal
 from polars.testing.parametric import series
 
 if TYPE_CHECKING:
@@ -51,7 +52,7 @@ def test_series_to_numpy_numeric_zero_copy(
     dtype: PolarsDataType, expected_dtype: npt.DTypeLike
 ) -> None:
     s = pl.Series([1, 2, 3]).cast(dtype)
-    result = s.to_numpy(allow_copy=False)
+    result: npt.NDArray[np.generic] = s.to_numpy(allow_copy=False)
 
     assert_zero_copy(s, result)
     assert result.tolist() == s.to_list()
@@ -77,7 +78,7 @@ def test_series_to_numpy_numeric_with_nulls(
     dtype: PolarsDataType, expected_dtype: npt.DTypeLike
 ) -> None:
     s = pl.Series([1, 2, None], dtype=dtype, strict=False)
-    result = s.to_numpy()
+    result: npt.NDArray[np.generic] = s.to_numpy()
 
     assert result.tolist()[:-1] == s.to_list()[:-1]
     assert np.isnan(result[-1])
@@ -103,7 +104,7 @@ def test_series_to_numpy_temporal_zero_copy(
 ) -> None:
     values = [0, 2_000, 1_000_000]
     s = pl.Series(values, dtype=dtype, strict=False)
-    result = s.to_numpy(allow_copy=False)
+    result: npt.NDArray[np.generic] = s.to_numpy(allow_copy=False)
 
     assert_zero_copy(s, result)
     # NumPy tolist returns integers for ns precision
@@ -117,7 +118,7 @@ def test_series_to_numpy_temporal_zero_copy(
 def test_series_to_numpy_datetime_with_tz_zero_copy() -> None:
     values = [datetime(1970, 1, 1), datetime(2024, 2, 28)]
     s = pl.Series(values).dt.convert_time_zone("Europe/Amsterdam").rechunk()
-    result = s.to_numpy(allow_copy=False)
+    result: npt.NDArray[np.generic] = s.to_numpy(allow_copy=False)
 
     assert_zero_copy(s, result)
     assert result.tolist() == values
@@ -128,7 +129,7 @@ def test_series_to_numpy_date() -> None:
     values = [date(1970, 1, 1), date(2024, 2, 28)]
     s = pl.Series(values)
 
-    result = s.to_numpy()
+    result: npt.NDArray[np.generic] = s.to_numpy()
 
     assert s.to_list() == result.tolist()
     assert result.dtype == np.dtype("datetime64[D]")
@@ -136,6 +137,19 @@ def test_series_to_numpy_date() -> None:
     assert_allow_copy_false_raises(s)
 
 
+def test_series_to_numpy_multi_dimensional_init() -> None:
+    s = pl.Series(np.atleast_3d(np.array([-10.5, 0.0, 10.5])))
+    assert_series_equal(
+        s,
+        pl.Series(
+            [[[-10.5], [0.0], [10.5]]],
+            dtype=pl.Array(pl.Float64, shape=(3, 1)),
+        ),
+    )
+    s = pl.Series(np.array(0), dtype=pl.Int32)
+    assert_series_equal(s, pl.Series([0], dtype=pl.Int32))
+
+
 @pytest.mark.parametrize(
     ("dtype", "expected_dtype"),
     [
@@ -154,7 +168,7 @@ def test_series_to_numpy_temporal_with_nulls(
 ) -> None:
     values = [0, 2_000, 1_000_000, None]
     s = pl.Series(values, dtype=dtype, strict=False)
-    result = s.to_numpy()
+    result: npt.NDArray[np.generic] = s.to_numpy()
 
     # NumPy tolist returns integers for ns precision
     if getattr(s.dtype, "time_unit", None) == "ns":
@@ -168,7 +182,7 @@ def test_series_to_numpy_temporal_with_nulls(
 def test_series_to_numpy_datetime_with_tz_with_nulls() -> None:
     values = [datetime(1970, 1, 1), datetime(2024, 2, 28), None]
     s = pl.Series(values).dt.convert_time_zone("Europe/Amsterdam")
-    result = s.to_numpy()
+    result: npt.NDArray[np.generic] = s.to_numpy()
 
     assert result.tolist() == values
     assert result.dtype == np.dtype("datetime64[us]")
@@ -195,7 +209,7 @@ def test_to_numpy_object_dtypes(
         values.append(None)
 
     s = pl.Series(values, dtype=dtype)
-    result = s.to_numpy()
+    result: npt.NDArray[np.generic] = s.to_numpy()
 
     assert result.tolist() == values
     assert result.dtype == np.object_
@@ -204,7 +218,7 @@ def test_to_numpy_object_dtypes(
 
 def test_series_to_numpy_bool() -> None:
     s = pl.Series([True, False])
-    result = s.to_numpy()
+    result: npt.NDArray[np.generic] = s.to_numpy()
 
     assert s.to_list() == result.tolist()
     assert result.dtype == np.bool_
@@ -214,7 +228,7 @@ def test_series_to_numpy_bool() -> None:
 
 def test_series_to_numpy_bool_with_nulls() -> None:
     s = pl.Series([True, False, None])
-    result = s.to_numpy()
+    result: npt.NDArray[np.generic] = s.to_numpy()
 
     assert s.to_list() == result.tolist()
     assert result.dtype == np.object_
@@ -235,7 +249,7 @@ def test_series_to_numpy_array_of_int() -> None:
 def test_series_to_numpy_array_of_str() -> None:
     values = [["1", "2", "3"], ["4", "5", "10000"]]
     s = pl.Series(values, dtype=pl.Array(pl.String, 3))
-    result = s.to_numpy()
+    result: npt.NDArray[np.generic] = s.to_numpy()
     assert result.tolist() == values
     assert result.dtype == np.object_
 
@@ -329,7 +343,7 @@ def test_to_numpy_chunked() -> None:
     s2 = pl.Series([3, 4])
     s = pl.concat([s1, s2], rechunk=False)
 
-    result = s.to_numpy()
+    result: npt.NDArray[np.generic] = s.to_numpy()
 
     assert result.tolist() == s.to_list()
     assert result.dtype == np.int64
@@ -348,7 +362,7 @@ def test_to_numpy_chunked_temporal_nested() -> None:
     s2 = pl.Series([[datetime(2022, 1, 1)], [datetime(2023, 1, 1)]], dtype=dtype)
     s = pl.concat([s1, s2], rechunk=False)
 
-    result = s.to_numpy()
+    result: npt.NDArray[np.generic] = s.to_numpy()
 
     assert result.tolist() == s.to_list()
     assert result.dtype == np.dtype("datetime64[us]")
@@ -361,7 +375,7 @@ def test_zero_copy_only_deprecated() -> None:
     values = [1, 2]
     s = pl.Series([1, 2])
     with pytest.deprecated_call():
-        result = s.to_numpy(zero_copy_only=True)
+        result: npt.NDArray[np.generic] = s.to_numpy(zero_copy_only=True)
     assert result.tolist() == values
 
 
