@@ -20,7 +20,7 @@ import pytest
 
 import polars as pl
 from polars.exceptions import ComputeError
-from polars.testing import assert_frame_equal
+from polars.testing import assert_frame_equal, assert_series_equal
 
 
 def test_write_json() -> None:
@@ -329,6 +329,19 @@ def test_ndjson_null_inference_13183() -> None:
     }
 
 
+def test_ndjson_expected_null_got_object_inference_22807() -> None:
+    buf = io.StringIO()
+    for _ in range(100):
+        buf.write('{"a":[]}\n')
+    buf.write('{"a":[{"b":[]}]}\n')
+
+    buf.seek(0)
+
+    assert pl.read_ndjson(buf, infer_schema_length=None).schema == (
+        {"a": pl.List(pl.Struct([pl.Field("b", pl.List(pl.Null))]))}
+    )
+
+
 @pytest.mark.write_disk
 def test_json_wrong_input_handle_textio(tmp_path: Path) -> None:
     # This shouldn't be passed, but still we test if we can handle it gracefully
@@ -353,6 +366,12 @@ def test_json_normalize() -> None:
         {"id": 2, "name": "Faye Raker"},
     ]
 
+    assert pl.json_normalize([], schema=pl.Schema({"test": pl.Int32})).to_dict(
+        as_series=False
+    ) == {
+        "test": [],
+    }
+
     assert pl.json_normalize(data, max_level=0).to_dict(as_series=False) == {
         "id": [1, None, 2],
         "name": [
@@ -613,3 +632,11 @@ def test_ndjson_22229() -> None:
     ]
 
     assert pl.read_ndjson(io.StringIO("\n".join(li))).to_dict(as_series=False)
+
+
+def test_json_encode_enum_23826() -> None:
+    s = pl.Series("a", ["b"], dtype=pl.Enum(["b"]))
+    assert_series_equal(
+        s.to_frame().select(c=pl.struct("a").struct.json_encode()).to_series(),
+        pl.Series("c", ['{"a":"0"}'], pl.String),
+    )
