@@ -104,6 +104,20 @@ a,b
     assert_fast_count(q, 3, capfd=capfd, monkeypatch=monkeypatch)
 
 
+def test_count_csv_no_newline_on_last_22564() -> None:
+    data = b"""\
+a,b
+1,2
+3,4
+5,6"""
+
+    assert pl.scan_csv(data).collect().height == 3
+    assert pl.scan_csv(data, comment_prefix="#").collect().height == 3
+
+    assert pl.scan_csv(data).select(pl.len()).collect().item() == 3
+    assert pl.scan_csv(data, comment_prefix="#").select(pl.len()).collect().item() == 3
+
+
 @pytest.mark.write_disk
 def test_commented_csv(
     capfd: pytest.CaptureFixture[str], monkeypatch: pytest.MonkeyPatch
@@ -187,7 +201,7 @@ def test_count_compressed_ndjson(
     df = pl.DataFrame({"x": range(5)})
 
     with gzip.open(path, "wb") as f:
-        df.write_ndjson(f)
+        df.write_ndjson(f)  # type: ignore[call-overload]
 
     lf = pl.scan_ndjson(path).select(pl.len())
     assert_fast_count(lf, 5, capfd=capfd, monkeypatch=monkeypatch)
@@ -216,3 +230,21 @@ def test_count_projection_pd(
 
     assert project_logs == {"project: 0"}
     assert result.item() == 3
+
+
+def test_csv_scan_skip_lines_len_22889(
+    capfd: pytest.CaptureFixture[str], monkeypatch: pytest.MonkeyPatch
+) -> None:
+    bb = b"col\n1\n2\n3"
+    lf = pl.scan_csv(bb, skip_lines=2).select(pl.len())
+    assert_fast_count(lf, 1, capfd=capfd, monkeypatch=monkeypatch)
+
+    ## trigger multi-threading code path
+    bb_10k = b"1\n2\n3\n4\n5\n6\n7\n8\n9\n0\n" * 1000
+    lf = pl.scan_csv(bb_10k, skip_lines=1000, has_header=False).select(pl.len())
+    assert_fast_count(lf, 9000, capfd=capfd, monkeypatch=monkeypatch)
+
+    # for comparison
+    out = pl.scan_csv(bb, skip_lines=2).collect().select(pl.len())
+    expected = pl.DataFrame({"len": [1]}, schema={"len": pl.UInt32})
+    assert_frame_equal(expected, out)
