@@ -25,7 +25,9 @@ def test_scan_csv(io_files_path: Path) -> None:
 
 def test_scan_csv_no_cse_deadlock(io_files_path: Path) -> None:
     dfs = [pl.scan_csv(io_files_path / "small.csv")] * (pl.thread_pool_size() + 1)
-    pl.concat(dfs, parallel=True).collect(comm_subplan_elim=False)
+    pl.concat(dfs, parallel=True).collect(
+        optimizations=pl.QueryOptFlags(comm_subplan_elim=False)
+    )
 
 
 def test_scan_empty_csv(io_files_path: Path) -> None:
@@ -207,7 +209,7 @@ def test_lazy_row_index_no_push_down(foods_file_path: Path) -> None:
         .with_row_index()
         .filter(pl.col("index") == 1)
         .filter(pl.col("category") == pl.lit("vegetables"))
-        .explain(predicate_pushdown=True)
+        .explain(optimizations=pl.QueryOptFlags(predicate_pushdown=True))
     )
     # related to row count is not pushed.
     assert 'FILTER [(col("index")) == (1)]\nFROM' in plan
@@ -406,9 +408,7 @@ c
             f.write(data)
 
     expect = pl.Series("a", ["1", "2", "b", "c"]).to_frame()
-    out = pl.scan_csv(paths).collect(  # type: ignore[call-overload]
-        engine="old-streaming" if streaming else "in-memory"
-    )
+    out = pl.scan_csv(paths).collect(engine="streaming" if streaming else "in-memory")
 
     assert_frame_equal(out, expect)
 
@@ -466,3 +466,59 @@ def test_select_nonexistent_column() -> None:
 
     with pytest.raises(pl.exceptions.ColumnNotFoundError):
         pl.scan_csv(f).select("b").collect()
+
+
+def test_scan_csv_provided_schema_with_extra_fields_22531() -> None:
+    data = b"""\
+a,b,c
+a,b,c
+"""
+
+    schema = {x: pl.String for x in ["a", "b", "c", "d", "e"]}
+
+    assert_frame_equal(
+        pl.scan_csv(data, schema=schema).collect(),
+        pl.DataFrame(
+            {
+                "a": "a",
+                "b": "b",
+                "c": "c",
+                "d": None,
+                "e": None,
+            },
+            schema=schema,
+        ),
+    )
+
+
+def test_csv_negative_slice_comment_char_22996() -> None:
+    f = b"""\
+a,b
+1,1
+"""
+
+    q = pl.scan_csv(2 * [f], comment_prefix="#").tail(100)
+    assert_frame_equal(q.collect(), pl.DataFrame({"a": [1, 1], "b": [1, 1]}))
+
+
+def test_csv_io_object_utf8_23629() -> None:
+    n_repeats = 10_000
+    for df in [
+        pl.DataFrame({"a": ["é,è"], "b": ["c,d"]}),
+        pl.DataFrame({"a": ["Ú;и"], "b": ["c;d"]}),
+        pl.DataFrame({"a": ["a,b"], "b": ["c,d"]}),
+        pl.DataFrame({"a": ["é," * n_repeats + "è"], "b": ["c," * n_repeats + "d"]}),
+    ]:
+        # bytes
+        f_bytes = io.BytesIO()
+        df.write_csv(f_bytes)
+        f_bytes.seek(0)
+        df_bytes = pl.read_csv(f_bytes)
+        assert_frame_equal(df, df_bytes)
+
+        # str
+        f_str = io.StringIO()
+        df.write_csv(f_str)
+        f_str.seek(0)
+        df_str = pl.read_csv(f_str)
+        assert_frame_equal(df, df_str)
