@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+import io
 import subprocess
 import sys
 from collections import OrderedDict
@@ -8,6 +9,7 @@ from threading import Thread
 from typing import TYPE_CHECKING, Any
 
 import pandas as pd
+import pyarrow.parquet as pq
 import pytest
 
 import polars as pl
@@ -68,7 +70,6 @@ def test_row_index_len_16543(foods_parquet_path: Path) -> None:
 
 
 @pytest.mark.write_disk
-@pytest.mark.usefixtures("test_global_and_local")
 def test_categorical_parquet_statistics(tmp_path: Path) -> None:
     tmp_path.mkdir(exist_ok=True)
 
@@ -272,7 +273,6 @@ def test_parquet_statistics(monkeypatch: Any, capfd: Any, tmp_path: Path) -> Non
 
 
 @pytest.mark.write_disk
-@pytest.mark.usefixtures("test_global_and_local")
 def test_categorical(tmp_path: Path) -> None:
     tmp_path.mkdir(exist_ok=True)
 
@@ -286,19 +286,18 @@ def test_categorical(tmp_path: Path) -> None:
     file_path = tmp_path / "categorical.parquet"
     df.write_parquet(file_path)
 
-    with pl.StringCache():
-        result = (
-            pl.scan_parquet(file_path)
-            .group_by("name")
-            .agg(pl.col("amount").sum())
-            .collect()
-            .sort("name")
-        )
-        expected = pl.DataFrame(
-            {"name": ["Bob", "Alice"], "amount": [400, 200]},
-            schema_overrides={"name": pl.Categorical},
-        )
-        assert_frame_equal(result, expected)
+    result = (
+        pl.scan_parquet(file_path)
+        .group_by("name")
+        .agg(pl.col("amount").sum())
+        .collect()
+        .sort("name")
+    )
+    expected = pl.DataFrame(
+        {"name": ["Alice", "Bob"], "amount": [200, 400]},
+        schema_overrides={"name": pl.Categorical},
+    )
+    assert_frame_equal(result, expected)
 
 
 def test_glob_n_rows(io_files_path: Path) -> None:
@@ -589,7 +588,9 @@ def test_predicate_slice_pushdown_row_index_20485(tmp_path: Path) -> None:
     ldf = pl.scan_parquet(file_path)
     sliced_df = ldf.with_row_index().slice(slice_start, slice_len).collect()
     sliced_df_no_pushdown = (
-        ldf.with_row_index().slice(slice_start, slice_len).collect(slice_pushdown=False)
+        ldf.with_row_index()
+        .slice(slice_start, slice_len)
+        .collect(optimizations=pl.QueryOptFlags(slice_pushdown=False))
     )
 
     expected_index = list(range(slice_start, slice_start + slice_len))
@@ -647,7 +648,7 @@ def test_parquet_unaligned_schema_read(tmp_path: Path) -> None:
     for df, path in zip(dfs, paths):
         df.write_parquet(path)
 
-    lf = pl.scan_parquet(paths)
+    lf = pl.scan_parquet(paths, extra_columns="ignore")
 
     assert_frame_equal(
         lf.select("a").collect(engine="in-memory"),
@@ -669,6 +670,8 @@ def test_parquet_unaligned_schema_read(tmp_path: Path) -> None:
         pl.DataFrame({"a": [1, 2], "b": [10, 11]}),
     )
 
+    lf = pl.scan_parquet(paths, extra_columns="raise")
+
     with pytest.raises(pl.exceptions.SchemaError):
         lf.collect(engine="in-memory")
 
@@ -749,7 +752,7 @@ def test_parquet_schema_arg(
         lf.collect(engine="streaming" if streaming else "in-memory")
 
     lf = pl.scan_parquet(
-        paths, parallel=parallel, schema=schema, allow_missing_columns=True
+        paths, parallel=parallel, schema=schema, missing_columns="insert"
     )
 
     assert_frame_equal(
@@ -760,17 +763,17 @@ def test_parquet_schema_arg(
     # Just one test that `read_parquet` is propagating this argument.
     assert_frame_equal(
         pl.read_parquet(
-            paths, parallel=parallel, schema=schema, allow_missing_columns=True
+            paths, parallel=parallel, schema=schema, missing_columns="insert"
         ),
         pl.DataFrame({"1": None, "a": [1, 2], "b": [1, 2]}, schema=schema),
     )
 
     # Issue #19081: If a schema arg is passed, ensure its fields are propagated
-    # to the IR, otherwise even if `allow_missing_columns=True`, downstream
+    # to the IR, otherwise even if `missing_columns='insert'`, downstream
     # `select()`s etc. will fail with ColumnNotFound if the column is not in
     # the first file.
     lf = pl.scan_parquet(
-        paths, parallel=parallel, schema=schema, allow_missing_columns=True
+        paths, parallel=parallel, schema=schema, missing_columns="insert"
     ).select("1")
 
     s = lf.collect(engine="streaming" if streaming else "in-memory").to_series()
@@ -781,18 +784,23 @@ def test_parquet_schema_arg(
 
     schema: dict[str, type[pl.DataType]] = {"a": pl.Int64}  # type: ignore[no-redef]
 
-    for allow_missing_columns in [True, False]:
+    for missing_columns in ["insert", "raise"]:
         lf = pl.scan_parquet(
             paths,
             parallel=parallel,
             schema=schema,
-            allow_missing_columns=allow_missing_columns,
+            missing_columns=missing_columns,  # type: ignore[arg-type]
         )
 
         with pytest.raises(pl.exceptions.SchemaError):
             lf.collect(engine="streaming" if streaming else "in-memory")
 
-    lf = pl.scan_parquet(paths, parallel=parallel, schema=schema).select("a")
+    lf = pl.scan_parquet(
+        paths,
+        parallel=parallel,
+        schema=schema,
+        extra_columns="ignore",
+    ).select("a")
 
     assert_frame_equal(
         lf.collect(engine="in-memory"),
@@ -805,7 +813,7 @@ def test_parquet_schema_arg(
 
     with pytest.raises(
         pl.exceptions.SchemaError,
-        match="data type mismatch for column b: expected: i8, found: i64",
+        match="data type mismatch for column b: incoming: Int64 != target: Int8",
     ):
         lf.collect(engine="streaming" if streaming else "in-memory")
 
@@ -842,11 +850,11 @@ def test_scan_parquet_schema_specified_with_empty_files_list(tmp_path: Path) ->
     )
 
 
-@pytest.mark.parametrize("allow_missing_columns", [True, False])
+@pytest.mark.parametrize("missing_columns", ["insert", "raise"])
 @pytest.mark.write_disk
 def test_scan_parquet_ignores_dtype_mismatch_for_non_projected_columns_19249(
     tmp_path: Path,
-    allow_missing_columns: bool,
+    missing_columns: str,
 ) -> None:
     tmp_path.mkdir(exist_ok=True)
     paths = [tmp_path / "1", tmp_path / "2"]
@@ -859,7 +867,7 @@ def test_scan_parquet_ignores_dtype_mismatch_for_non_projected_columns_19249(
     ).write_parquet(paths[1])
 
     assert_frame_equal(
-        pl.scan_parquet(paths, allow_missing_columns=allow_missing_columns)
+        pl.scan_parquet(paths, missing_columns=missing_columns)  # type: ignore[arg-type]
         .select("a")
         .collect(engine="in-memory"),
         pl.DataFrame({"a": [1, 1]}, schema={"a": pl.Int32}),
@@ -905,6 +913,8 @@ import io
 import polars as pl
 from polars.testing import assert_frame_equal
 
+assert pl.thread_pool_size() == 2
+
 f = io.BytesIO()
 
 df = pl.DataFrame({x: 1 for x in ["a", "b", "c", "d", "e"]})
@@ -924,3 +934,241 @@ print("OK", end="")
     )
 
     assert out == b"OK"
+
+
+def test_scan_parquet_in_mem_to_streaming_dispatch_deadlock_22641() -> None:
+    out = subprocess.check_output(
+        [
+            sys.executable,
+            "-c",
+            """\
+import os
+
+os.environ["POLARS_MAX_THREADS"] = "1"
+os.environ["POLARS_VERBOSE"] = "1"
+
+import io
+import sys
+from threading import Thread
+
+import polars as pl
+
+assert pl.thread_pool_size() == 1
+
+f = io.BytesIO()
+pl.DataFrame({"x": 1}).write_parquet(f)
+
+q = (
+    pl.scan_parquet(f)
+    .filter(pl.sum_horizontal(pl.col("x"), pl.col("x"), pl.col("x")) >= 0)
+    .join(pl.scan_parquet(f), on="x", how="left")
+)
+
+results = [
+    pl.DataFrame(),
+    pl.DataFrame(),
+    pl.DataFrame(),
+    pl.DataFrame(),
+    pl.DataFrame(),
+]
+
+
+def run():
+    # Also test just a single scan
+    pl.scan_parquet(f).collect()
+
+    print("QUERY-FENCE", file=sys.stderr)
+
+    results[0] = q.collect()
+
+    print("QUERY-FENCE", file=sys.stderr)
+
+    results[1] = pl.concat([q, q, q]).collect().head(1)
+
+    print("QUERY-FENCE", file=sys.stderr)
+
+    results[2] = pl.collect_all([q, q, q])[0]
+
+    print("QUERY-FENCE", file=sys.stderr)
+
+    results[3] = pl.collect_all(3 * [pl.concat(3 * [q])])[0].head(1)
+
+    print("QUERY-FENCE", file=sys.stderr)
+
+    results[4] = q.collect(background=True).fetch_blocking()
+
+
+t = Thread(target=run, daemon=True)
+t.start()
+t.join(5)
+
+assert [x.equals(pl.DataFrame({"x": 1})) for x in results] == [
+    True,
+    True,
+    True,
+    True,
+    True,
+]
+
+print("OK", end="", file=sys.stderr)
+""",
+        ],
+        stderr=subprocess.STDOUT,
+    )
+
+    assert out.endswith(b"OK")
+
+    def ensure_caches_dropped(verbose_log: str) -> None:
+        cache_hit_prefix = "CACHE HIT: cache id: "
+
+        ids_hit = {
+            x[len(cache_hit_prefix) :]
+            for x in verbose_log.splitlines()
+            if x.startswith(cache_hit_prefix)
+        }
+
+        cache_drop_prefix = "CACHE DROP: cache id: "
+
+        ids_dropped = {
+            x[len(cache_drop_prefix) :]
+            for x in verbose_log.splitlines()
+            if x.startswith(cache_drop_prefix)
+        }
+
+        assert ids_hit == ids_dropped
+
+    out_str = out.decode()
+
+    for logs in out_str.split("QUERY-FENCE"):
+        ensure_caches_dropped(logs)
+
+
+def test_parquet_prefiltering_inserted_column_23268() -> None:
+    df = pl.DataFrame({"a": [1, 2, 3, 4]}, schema={"a": pl.Int8})
+
+    f = io.BytesIO()
+
+    df.write_parquet(f)
+
+    assert_frame_equal(
+        (
+            pl.scan_parquet(
+                f,
+                schema={"a": pl.Int8, "b": pl.Int16},
+                missing_columns="insert",
+            )
+            .filter(pl.col("a") == 3)
+            .filter(pl.col("b") == 3)
+            .collect()
+        ),
+        pl.DataFrame(schema={"a": pl.Int8, "b": pl.Int16}),
+    )
+
+
+def test_scan_parquet_prefilter_with_cast(
+    monkeypatch: pytest.MonkeyPatch,
+    capfd: pytest.CaptureFixture[str],
+) -> None:
+    f = io.BytesIO()
+
+    df = pl.DataFrame(
+        {
+            "a": ["A", "B", "C", "D", "E", "F"],
+            "b": pl.Series([1, 1, 1, 1, 0, 1], dtype=pl.UInt8),
+        }
+    )
+
+    df.write_parquet(f, row_group_size=3)
+
+    md = pq.read_metadata(f)
+
+    assert [md.row_group(i).num_rows for i in range(md.num_row_groups)] == [3, 3]
+
+    q = pl.scan_parquet(
+        f,
+        schema={"a": pl.String, "b": pl.Int16},
+        cast_options=pl.ScanCastOptions(integer_cast="upcast"),
+        include_file_paths="file_path",
+    ).filter(pl.col("b") - 1 == pl.lit(-1, dtype=pl.Int16))
+
+    with monkeypatch.context() as cx:
+        cx.setenv("POLARS_VERBOSE", "1")
+        capfd.readouterr()
+        out = q.collect()
+        capture = capfd.readouterr().err
+
+    assert (
+        "[ParquetFileReader]: Pre-filtered decode enabled (1 live, 1 non-live)"
+        in capture
+    )
+    assert (
+        "[ParquetFileReader]: Predicate pushdown: reading 1 / 2 row groups" in capture
+    )
+
+    assert_frame_equal(
+        out,
+        pl.DataFrame(
+            {
+                "a": "E",
+                "b": pl.Series([0], dtype=pl.Int16),
+                "file_path": "in-mem",
+            }
+        ),
+    )
+
+
+def test_prefilter_with_n_rows_23790() -> None:
+    df = pl.DataFrame(
+        {
+            "a": ["A", "B", "C", "D", "E", "F"],
+            "b": [1, 2, 3, 4, 5, 6],
+        }
+    )
+
+    f = io.BytesIO()
+
+    df.write_parquet(f, row_group_size=2)
+
+    md = pq.read_metadata(f)
+
+    assert [md.row_group(i).num_rows for i in range(md.num_row_groups)] == [2, 2, 2]
+
+    q = pl.scan_parquet(f, n_rows=3).filter(pl.col("b").is_in([1, 3]))
+
+    assert_frame_equal(q.collect(), pl.DataFrame({"a": ["A", "C"], "b": [1, 3]}))
+
+    # With row index / file_path
+
+    df = pl.DataFrame(
+        {
+            "a": ["A", "B", "C", "D", "E", "F"],
+            "b": [1, 2, 3, 4, 5, 6],
+        }
+    )
+
+    f = io.BytesIO()
+
+    df.write_parquet(f, row_group_size=2)
+
+    md = pq.read_metadata(f)
+
+    assert [md.row_group(i).num_rows for i in range(md.num_row_groups)] == [2, 2, 2]
+
+    q = pl.scan_parquet(
+        f,
+        n_rows=3,
+        row_index_name="index",
+        include_file_paths="file_path",
+    ).filter(pl.col("b").is_in([1, 3]))
+
+    assert_frame_equal(
+        q.collect(),
+        pl.DataFrame(
+            {
+                "index": pl.Series([0, 2], dtype=pl.get_index_type()),
+                "a": ["A", "C"],
+                "b": [1, 3],
+                "file_path": "in-mem",
+            }
+        ),
+    )
