@@ -14,9 +14,7 @@ def test_is_null_followed_by_all() -> None:
         pl.col("val").is_null().all()
     )
 
-    assert (
-        r'[[(col("val").count()) == (col("val").null_count())]]' in result_lf.explain()
-    )
+    assert r'[[(col("val").len()) == (col("val").null_count())]]' in result_lf.explain()
     assert "is_null" not in result_lf
     assert_frame_equal(expected_df, result_lf.collect())
 
@@ -82,9 +80,7 @@ def test_is_not_null_followed_by_any() -> None:
         pl.col("val").is_not_null().any()
     )
 
-    assert (
-        r'[[(col("val").null_count()) < (col("val").count())]]' in result_lf.explain()
-    )
+    assert r'[[(col("val").null_count()) < (col("val").len())]]' in result_lf.explain()
     assert "is_not_null" not in result_lf.explain()
     assert_frame_equal(expected_df, result_lf.collect())
 
@@ -137,9 +133,7 @@ def test_is_not_null_followed_by_sum() -> None:
         pl.col("val").is_not_null().sum()
     )
 
-    assert (
-        r'[[(col("val").count()) - (col("val").null_count())]]' in result_lf.explain()
-    )
+    assert r'[[(col("val").len()) - (col("val").null_count())]]' in result_lf.explain()
     assert "is_not_null" not in result_lf.explain()
     assert_frame_equal(expected_df, result_lf.collect())
 
@@ -168,9 +162,7 @@ def test_drop_nulls_followed_by_len() -> None:
         pl.col("val").drop_nulls().len()
     )
 
-    assert (
-        r'[[(col("val").count()) - (col("val").null_count())]]' in result_lf.explain()
-    )
+    assert r'[[(col("val").len()) - (col("val").null_count())]]' in result_lf.explain()
     assert "drop_nulls" not in result_lf.explain()
     assert_frame_equal(expected_df, result_lf.collect())
 
@@ -194,9 +186,7 @@ def test_drop_nulls_followed_by_count() -> None:
         pl.col("val").drop_nulls().count()
     )
 
-    assert (
-        r'[[(col("val").count()) - (col("val").null_count())]]' in result_lf.explain()
-    )
+    assert r'[[(col("val").len()) - (col("val").null_count())]]' in result_lf.explain()
     assert "drop_nulls" not in result_lf.explain()
     assert_frame_equal(expected_df, result_lf.collect())
 
@@ -221,7 +211,7 @@ def test_collapse_joins() -> None:
     assert "INNER JOIN" in e
     assert "FILTER" not in e
     assert_frame_equal(
-        inner_join.collect(collapse_joins=False),
+        inner_join.collect(optimizations=pl.QueryOptFlags(collapse_joins=False)),
         inner_join.collect(),
         check_row_order=False,
     )
@@ -231,7 +221,7 @@ def test_collapse_joins() -> None:
     assert "INNER JOIN" in e
     assert "FILTER" not in e
     assert_frame_equal(
-        inner_join.collect(collapse_joins=False),
+        inner_join.collect(optimizations=pl.QueryOptFlags(collapse_joins=False)),
         inner_join.collect(),
         check_row_order=False,
     )
@@ -241,7 +231,7 @@ def test_collapse_joins() -> None:
     assert "INNER JOIN" in e
     assert "FILTER" not in e
     assert_frame_equal(
-        double_inner_join.collect(collapse_joins=False),
+        double_inner_join.collect(optimizations=pl.QueryOptFlags(collapse_joins=False)),
         double_inner_join.collect(),
         check_row_order=False,
     )
@@ -251,20 +241,11 @@ def test_collapse_joins() -> None:
     assert "NESTED LOOP JOIN" in e
     assert "FILTER" not in e
     assert_frame_equal(
-        dont_mix.collect(collapse_joins=False),
+        dont_mix.collect(optimizations=pl.QueryOptFlags(collapse_joins=False)),
         dont_mix.collect(),
         check_row_order=False,
     )
 
-    no_literals = cross.filter(pl.col.x == 2)
-    e = no_literals.explain()
-    assert "NESTED LOOP JOIN" in e
-    assert_frame_equal(
-        no_literals.collect(collapse_joins=False),
-        no_literals.collect(),
-        check_row_order=False,
-    )
-
     iejoin = cross.filter(pl.col.x >= pl.col.a)
     e = iejoin.explain()
     assert "IEJOIN" in e
@@ -272,7 +253,7 @@ def test_collapse_joins() -> None:
     assert "CROSS JOIN" not in e
     assert "FILTER" not in e
     assert_frame_equal(
-        iejoin.collect(collapse_joins=False),
+        iejoin.collect(optimizations=pl.QueryOptFlags(collapse_joins=False)),
         iejoin.collect(),
         check_row_order=False,
     )
@@ -284,7 +265,9 @@ def test_collapse_joins() -> None:
     assert "NESTED LOOP JOIN" not in e
     assert "FILTER" not in e
     assert_frame_equal(
-        iejoin.collect(collapse_joins=False), iejoin.collect(), check_row_order=False
+        iejoin.collect(optimizations=pl.QueryOptFlags(collapse_joins=False)),
+        iejoin.collect(),
+        check_row_order=False,
     )
 
 
@@ -316,7 +299,9 @@ def test_collapse_joins_combinations() -> None:
                 # IE-join is unspecified. Therefore, this might not necessarily
                 # create the exact same dataframe.
                 optimized = cross.filter(e).sort(pl.all()).collect()
-                unoptimized = cross.filter(e).collect(collapse_joins=False)
+                unoptimized = cross.filter(e).collect(
+                    optimizations=pl.QueryOptFlags(collapse_joins=False)
+                )
 
                 try:
                     assert_frame_equal(optimized, unoptimized, check_row_order=False)
@@ -328,7 +313,11 @@ def test_collapse_joins_combinations() -> None:
                     print(optimized)
                     print()
                     print("Unoptimized")
-                    print(cross.filter(e).explain(collapse_joins=False))
+                    print(
+                        cross.filter(e).explain(
+                            optimizations=pl.QueryOptFlags(collapse_joins=False)
+                        )
+                    )
                     print(unoptimized)
                     print()
 
@@ -358,3 +347,41 @@ def test_order_observe_sort_before_unique_22485() -> None:
     assert "SORT BY" in plan[plan.index("UNIQUE") :]
 
     assert_frame_equal(q.collect(), expect)
+
+
+def test_order_observe_group_by() -> None:
+    q = (
+        pl.LazyFrame({"a": range(5)})
+        .group_by("a", maintain_order=True)
+        .agg(b=1)
+        .sort("b")
+    )
+
+    plan = q.explain()
+    assert "AGGREGATE[maintain_order: false]" in plan
+
+    q = (
+        pl.LazyFrame({"a": range(5)})
+        .group_by("a", maintain_order=True)
+        .agg(b=1)
+        .sort("b", maintain_order=True)
+    )
+
+    plan = q.explain()
+    assert "AGGREGATE[maintain_order: true]" in plan
+
+
+def test_fused_correct_name() -> None:
+    df = pl.DataFrame({"x": [1, 2, 3]})
+
+    lf = df.lazy().select(
+        (pl.col.x.alias("a") * pl.col.x.alias("b")) + pl.col.x.alias("c")
+    )
+
+    no_opts = lf.collect(optimizations=pl.QueryOptFlags.none())
+    opts = lf.collect()
+    assert_frame_equal(
+        no_opts,
+        opts,
+    )
+    assert_frame_equal(opts, pl.DataFrame({"a": [2, 6, 12]}))
