@@ -58,11 +58,11 @@ def test_empty_all_any_horizontally() -> None:
     df = pl.DataFrame({"x": [1, 2, 3]})
     assert_frame_equal(
         df.select(pl.any_horizontal(cs.string().is_null())),
-        pl.DataFrame({"literal": False}),
+        pl.DataFrame({"any_horizontal": False}),
     )
     assert_frame_equal(
         df.select(pl.all_horizontal(cs.string().is_null())),
-        pl.DataFrame({"literal": True}),
+        pl.DataFrame({"all_horizontal": True}),
     )
 
 
@@ -364,6 +364,9 @@ def test_cum_sum_horizontal() -> None:
     expected = pl.DataFrame({"cum_sum": [{"a": 1, "c": 6}, {"a": 2, "c": 8}]})
     assert_frame_equal(result, expected)
 
+    q = df.lazy().select(pl.cum_sum_horizontal("a", "c"))
+    assert q.collect_schema() == q.collect().schema
+
 
 def test_sum_dtype_12028() -> None:
     result = pl.select(
