@@ -874,3 +874,19 @@ def test_arithmetic_i128_nonint() -> None:
     s = pl.Series("a", [True], dtype=pl.Boolean)
     assert_series_equal(s + s128, pl.Series("a", [1], dtype=pl.Int128))
     assert_series_equal(s128 + s, pl.Series("a", [1], dtype=pl.Int128))
+
+
+def test_float_truediv_output_type() -> None:
+    lf = pl.LazyFrame(schema={"f32": pl.Float32, "f64": pl.Float64})
+    assert lf.select(x=pl.col("f32") / pl.col("f32")).collect_schema() == pl.Schema(
+        {"x": pl.Float32}
+    )
+    assert lf.select(x=pl.col("f32") / pl.col("f64")).collect_schema() == pl.Schema(
+        {"x": pl.Float64}
+    )
+    assert lf.select(x=pl.col("f64") / pl.col("f32")).collect_schema() == pl.Schema(
+        {"x": pl.Float64}
+    )
+    assert lf.select(x=pl.col("f64") / pl.col("f64")).collect_schema() == pl.Schema(
+        {"x": pl.Float64}
+    )
