@@ -176,6 +176,26 @@ TEST_CASES = [
         """lambda x: x.lstrip().startswith(('!','#','?',"'"))""",
         """pl.col("b").str.strip_chars_start().str.contains(r"^(!|\\#|\\?|')")""",
     ),
+    (
+        "b",
+        "lambda x: x.replace(':','')",
+        """pl.col("b").str.replace_all(':','',literal=True)""",
+    ),
+    (
+        "b",
+        "lambda x: x.replace(':','',2)",
+        """pl.col("b").str.replace(':','',n=2,literal=True)""",
+    ),
+    (
+        "b",
+        "lambda x: x.removeprefix('A').removesuffix('F')",
+        """pl.col("b").str.strip_prefix('A').str.strip_suffix('F')""",
+    ),
+    (
+        "b",
+        "lambda x: x.zfill(8)",
+        """pl.col("b").str.zfill(8)""",
+    ),
     # ---------------------------------------------
     # json expr: load/extract
     # ---------------------------------------------
@@ -278,9 +298,11 @@ def test_parse_invalid_function(func: str) -> None:
     TEST_CASES,
 )
 @pytest.mark.filterwarnings(
+    "ignore:.*:polars.exceptions.MapWithoutReturnDtypeWarning",
     "ignore:invalid value encountered:RuntimeWarning",
     "ignore:.*without specifying `return_dtype`:polars.exceptions.MapWithoutReturnDtypeWarning",
 )
+@pytest.mark.may_fail_auto_streaming  # dtype not set
 def test_parse_apply_functions(col: str, func: str, expr_repr: str) -> None:
     with pytest.warns(
         PolarsInefficientMapWarning,
@@ -321,9 +343,11 @@ def test_parse_apply_functions(col: str, func: str, expr_repr: str) -> None:
 
 
 @pytest.mark.filterwarnings(
+    "ignore:.*:polars.exceptions.MapWithoutReturnDtypeWarning",
     "ignore:invalid value encountered:RuntimeWarning",
     "ignore:.*without specifying `return_dtype`:polars.exceptions.MapWithoutReturnDtypeWarning",
 )
+@pytest.mark.may_fail_auto_streaming  # dtype is not set
 def test_parse_apply_raw_functions() -> None:
     lf = pl.LazyFrame({"a": [1.1, 2.0, 3.4]})
 
@@ -340,7 +364,9 @@ def test_parse_apply_raw_functions() -> None:
             PolarsInefficientMapWarning,
             match=rf"(?s)Expr\.map_elements.*Replace this expression.*np\.{func_name}",
         ):
-            df1 = lf.select(pl.col("a").map_elements(func)).collect()
+            df1 = lf.select(
+                pl.col("a").map_elements(func, return_dtype=pl.self_dtype())
+            ).collect()
             df2 = lf.select(getattr(pl.col("a"), func_name)()).collect()
             assert_frame_equal(df1, df2)
 
@@ -352,7 +378,16 @@ def test_parse_apply_raw_functions() -> None:
     ):
         for expr in (
             pl.col("value").str.json_decode(),
-            pl.col("value").map_elements(json.loads),
+            pl.col("value").map_elements(
+                json.loads,
+                return_dtype=pl.Struct(
+                    {
+                        "a": pl.Int64,
+                        "b": pl.Boolean,
+                        "c": pl.String,
+                    }
+                ),
+            ),
         ):
             result_frames.append(  # noqa: PERF401
                 pl.LazyFrame({"value": ['{"a":1, "b": true, "c": "xx"}', None]})
@@ -370,7 +405,9 @@ def test_parse_apply_raw_functions() -> None:
             match=rf'(?s)with this one instead.*pl\.col\("a"\)\.cast\(pl\.{pl_dtype.__name__}\)',
         ):
             assert_frame_equal(
-                lf.select(pl.col("a").map_elements(py_cast)).collect(),
+                lf.select(
+                    pl.col("a").map_elements(py_cast, return_dtype=pl_dtype)
+                ).collect(),
                 lf.select(pl.col("a").cast(pl_dtype)).collect(),
             )
 
@@ -465,6 +502,7 @@ def test_parse_apply_series(
         assert_series_equal(expected_series, result_series)
 
 
+@pytest.mark.may_fail_auto_streaming
 def test_expr_exact_warning_message() -> None:
     red, green, end_escape = (
         ("\x1b[31m", "\x1b[32m", "\x1b[0m")
