@@ -5,6 +5,6 @@ from pathlib import Path
 import pytest
 
 
-@pytest.fixture()
+@pytest.fixture
 def namespace_files_path() -> Path:
     return Path(__file__).parent / "files"
