@@ -15,6 +15,7 @@ from polars.exceptions import (
     SchemaError,
 )
 from polars.testing import assert_frame_equal, assert_series_equal
+from tests.unit.conftest import time_func
 
 if TYPE_CHECKING:
     from polars._typing import PolarsDataType
@@ -385,12 +386,12 @@ def test_list_drop_nulls() -> None:
 def test_list_sample() -> None:
     s = pl.Series("values", [[1, 2, 3, None], [None, None], [1, 2], None])
 
-    expected_sample_n = pl.Series("values", [[3, 1], [None], [2], None])
+    expected_sample_n = pl.Series("values", [[None, 3], [None], [2], None])
     assert_series_equal(
         s.list.sample(n=pl.Series([2, 1, 1, 1]), seed=1), expected_sample_n
     )
 
-    expected_sample_frac = pl.Series("values", [[3, 1], [None], [1, 2], None])
+    expected_sample_frac = pl.Series("values", [[None, 3], [None], [1, 2], None])
     assert_series_equal(
         s.list.sample(fraction=pl.Series([0.5, 0.5, 1.0, 0.3]), seed=1),
         expected_sample_frac,
@@ -408,7 +409,10 @@ def test_list_sample() -> None:
         sample_frac=pl.col("values").list.sample(fraction=pl.col("frac"), seed=1),
     )
     expected_df = pl.DataFrame(
-        {"sample_n": [[3, 1], [None], [3, 4]], "sample_frac": [[3, 1], [None], [3, 4]]}
+        {
+            "sample_n": [[None, 3], [None], [3, 4]],
+            "sample_frac": [[None, 3], [None], [3, 4]],
+        }
     )
     assert_frame_equal(df, expected_df)
 
@@ -1080,3 +1084,159 @@ def test_list_shift_unequal_lengths_22018() -> None:
 
 def test_list_shift_self_broadcast() -> None:
     assert pl.Series("a", [[1, 2]]).list.shift(pl.Series([1, 2, 1])).len() == 3
+
+
+def test_list_filter_simple() -> None:
+    assert pl.Series(
+        [
+            [1, 2, 3, 4, 5],
+            [1, 3, 7, 8],
+            [6, 1, 4, 5],
+        ]
+    ).list.filter(pl.element() < 5).to_list() == [
+        [1, 2, 3, 4],
+        [1, 3],
+        [1, 4],
+    ]
+
+
+def test_list_filter_result_empty() -> None:
+    assert pl.Series(
+        [
+            ["a"],
+            ["b", "c"],
+        ]
+    ).list.filter(pl.element() == "d").to_list() == [
+        [],
+        [],
+    ]
+
+
+def test_list_filter_null() -> None:
+    assert pl.Series(
+        [
+            [None, 1, 2],
+            [None, None],
+            [1, 2],
+        ]
+    ).list.filter(pl.element().is_not_null()).to_list() == [
+        [1, 2],
+        [],
+        [1, 2],
+    ]
+
+
+@pytest.mark.slow
+def test_list_struct_field_perf() -> None:
+    base_df = pl.concat(100 * [pl.DataFrame({"a": [[{"fld": 1}]]})]).rechunk()
+    df = base_df
+
+    q = df.lazy().select(pl.col("a").list.eval(pl.element().struct.field("fld")))
+
+    t0 = time_func(q.collect, iterations=5)
+
+    # Note: Rechunk is important here to force single threaded
+    df = pl.concat(10_000 * [base_df]).rechunk()
+
+    q = df.lazy().select(pl.col("a").list.eval(pl.element().struct.field("fld")))
+
+    t1 = time_func(q.collect, iterations=5)
+
+    slowdown = t1 / t0
+
+    # Timings (Apple M3 Pro 11-core)
+    # * Debug build w/ elementwise: 1x
+    # * Release pypi 1.29.0: 80x
+    threshold = 3
+
+    if slowdown > threshold:
+        msg = f"slowdown ({slowdown}) > {threshold}x ({t0 = }, {t1 = })"
+        raise ValueError(msg)
+
+
+def test_list_elementwise_eval_logical_output_type() -> None:
+    out = pl.DataFrame({"a": [["2025-01-01"], ["2025-01-01"]]}).select(
+        pl.col("a").list.eval(pl.element().str.strptime(pl.Datetime, format="%Y-%m-%d"))
+    )
+
+    assert_series_equal(
+        out.to_series(),
+        pl.Series(
+            "a",
+            [[datetime(2025, 1, 1)], [datetime(2025, 1, 1)]],
+            dtype=pl.List(pl.Datetime),
+        ),
+    )
+
+
+def test_list_elementwise_eval_fallible_masked_sliced() -> None:
+    # Baseline - fails on invalid data
+    with pytest.raises(
+        InvalidOperationError, match=r"conversion from `str` to `datetime\[μs\]` failed"
+    ):
+        pl.DataFrame({"a": [["AAA"], ["2025-01-01"]]}).select(
+            pl.col("a").list.eval(
+                pl.element().str.strptime(pl.Datetime, format="%Y-%m-%d")
+            )
+        )
+
+    # Ensure fallible expressions do not cause failures on masked-out data.
+    out = (
+        pl.DataFrame({"a": [["AAA"], ["2025-01-01"]]})
+        .with_columns(pl.when(pl.Series([False, True])).then(pl.col("a")).alias("a"))
+        .select(
+            pl.col("a").list.eval(
+                pl.element().str.strptime(pl.Datetime, format="%Y-%m-%d")
+            )
+        )
+    )
+
+    assert_series_equal(
+        out.to_series(),
+        pl.Series("a", [None, [datetime(2025, 1, 1)]], dtype=pl.List(pl.Datetime)),
+    )
+
+    out = (
+        pl.DataFrame({"a": [["AAA"], ["2025-01-01"], ["2025-01-01"]]})
+        .slice(1)
+        .select(
+            pl.col("a").list.eval(
+                pl.element().str.strptime(pl.Datetime, format="%Y-%m-%d")
+            )
+        )
+    )
+
+    assert_series_equal(
+        out.to_series(),
+        pl.Series(
+            "a",
+            [[datetime(2025, 1, 1)], [datetime(2025, 1, 1)]],
+            dtype=pl.List(pl.Datetime),
+        ),
+    )
+
+
+def test_list_contains() -> None:
+    s = pl.Series([[1, 2, None], [None], None])
+
+    assert_series_equal(
+        s.list.contains(None, nulls_equal=False),
+        pl.Series([None, None, None], dtype=pl.Boolean),
+    )
+    assert_series_equal(
+        s.list.contains(None, nulls_equal=True),
+        pl.Series([True, True, None], dtype=pl.Boolean),
+    )
+    assert_series_equal(
+        s.list.contains(1, nulls_equal=False),
+        pl.Series([True, False, None], dtype=pl.Boolean),
+    )
+    assert_series_equal(
+        s.list.contains(1, nulls_equal=True),
+        pl.Series([True, False, None], dtype=pl.Boolean),
+    )
+
+
+def test_list_diff_invalid_type() -> None:
+    with pytest.raises(pl.exceptions.InvalidOperationError):
+        pl.Series([1, 2, 3]).list.diff()
