@@ -1,12 +1,13 @@
 from __future__ import annotations
 
+from io import BytesIO
+
 import pytest
 
 import polars as pl
 from polars.testing import assert_frame_equal, assert_series_equal
 
 
-@pytest.mark.usefixtures("test_global_and_local")
 def test_categorical_lexical_sort() -> None:
     df = pl.DataFrame(
         {"cats": ["z", "z", "k", "a", "b"], "vals": [3, 1, 2, 2, 3]}
@@ -42,7 +43,6 @@ def test_categorical_lexical_sort() -> None:
     ]
 
 
-@pytest.mark.usefixtures("test_global_and_local")
 def test_categorical_lexical_ordering_after_concat() -> None:
     ldf1 = (
         pl.DataFrame([pl.Series("key1", [8, 5]), pl.Series("key2", ["fox", "baz"])])
@@ -63,19 +63,6 @@ def test_categorical_lexical_ordering_after_concat() -> None:
     }
 
 
-@pytest.mark.usefixtures("test_global_and_local")
-@pytest.mark.may_fail_auto_streaming
-def test_sort_categoricals_6014_internal() -> None:
-    # create basic categorical
-    df = pl.DataFrame({"key": ["bbb", "aaa", "ccc"]}).with_columns(
-        pl.col("key").cast(pl.Categorical)
-    )
-
-    out = df.sort("key")
-    assert out.to_dict(as_series=False) == {"key": ["bbb", "aaa", "ccc"]}
-
-
-@pytest.mark.usefixtures("test_global_and_local")
 def test_sort_categoricals_6014_lexical() -> None:
     # create lexically-ordered categorical
     df = pl.DataFrame({"key": ["bbb", "aaa", "ccc"]}).with_columns(
@@ -86,76 +73,28 @@ def test_sort_categoricals_6014_lexical() -> None:
     assert out.to_dict(as_series=False) == {"key": ["aaa", "bbb", "ccc"]}
 
 
-@pytest.mark.usefixtures("test_global_and_local")
 def test_categorical_get_categories() -> None:
-    assert pl.Series(
-        "cats", ["foo", "bar", "foo", "foo", "ham"], dtype=pl.Categorical
-    ).cat.get_categories().to_list() == ["foo", "bar", "ham"]
+    s = pl.Series("cats", ["foo", "bar", "foo", "foo", "ham"], dtype=pl.Categorical)
+    assert set(s.cat.get_categories().to_list()) >= {"foo", "bar", "ham"}
 
 
 def test_cat_to_local() -> None:
-    with pl.StringCache():
-        s1 = pl.Series(["a", "b", "a"], dtype=pl.Categorical)
-        s2 = pl.Series(["c", "b", "d"], dtype=pl.Categorical)
-
-    # s2 physical starts after s1
-    assert s1.to_physical().to_list() == [0, 1, 0]
-    assert s2.to_physical().to_list() == [2, 1, 3]
-
-    out = s2.cat.to_local()
-
-    # Physical has changed and now starts at 0, string values are the same
-    assert out.cat.is_local()
-    assert out.to_physical().to_list() == [0, 1, 2]
-    assert out.to_list() == s2.to_list()
-
-    # s2 should be unchanged after the operation
-    assert not s2.cat.is_local()
-    assert s2.to_physical().to_list() == [2, 1, 3]
-    assert s2.to_list() == ["c", "b", "d"]
+    s = pl.Series(["a", "b", "a"], dtype=pl.Categorical)
+    assert_series_equal(s, s.cat.to_local())
 
 
-def test_cat_to_local_missing_values() -> None:
-    with pl.StringCache():
-        _ = pl.Series(["a", "b"], dtype=pl.Categorical)
-        s = pl.Series(["c", "b", None, "d"], dtype=pl.Categorical)
-
-    out = s.cat.to_local()
-    assert out.to_physical().to_list() == [0, 1, None, 2]
-
-
-def test_cat_to_local_already_local() -> None:
-    s = pl.Series(["a", "c", "a", "b"], dtype=pl.Categorical)
-
-    assert s.cat.is_local()
-    out = s.cat.to_local()
-
-    assert out.to_physical().to_list() == [0, 1, 0, 2]
-    assert out.to_list() == ["a", "c", "a", "b"]
-
-
-def test_cat_is_local() -> None:
-    s = pl.Series(["a", "c", "a", "b"], dtype=pl.Categorical)
-    assert s.cat.is_local()
-
-    with pl.StringCache():
-        s2 = pl.Series(["a", "b", "a"], dtype=pl.Categorical)
-    assert not s2.cat.is_local()
-
-
-@pytest.mark.usefixtures("test_global_and_local")
 def test_cat_uses_lexical_ordering() -> None:
     s = pl.Series(["a", "b", None, "b"]).cast(pl.Categorical)
-    assert s.cat.uses_lexical_ordering() is False
+    assert s.cat.uses_lexical_ordering()
 
     s = s.cast(pl.Categorical("lexical"))
-    assert s.cat.uses_lexical_ordering() is True
+    assert s.cat.uses_lexical_ordering()
 
-    s = s.cast(pl.Categorical("physical"))
-    assert s.cat.uses_lexical_ordering() is False
+    with pytest.warns(DeprecationWarning):
+        s = s.cast(pl.Categorical("physical"))  # Deprecated.
+        assert s.cat.uses_lexical_ordering()
 
 
-@pytest.mark.usefixtures("test_global_and_local")
 def test_cat_len_bytes() -> None:
     # test Series
     s = pl.Series("a", ["Café", None, "Café", "345", "東京"], dtype=pl.Categorical)
@@ -192,7 +131,6 @@ def test_cat_len_bytes() -> None:
     assert_frame_equal(result_df, expected_df)
 
 
-@pytest.mark.usefixtures("test_global_and_local")
 def test_cat_len_chars() -> None:
     # test Series
     s = pl.Series("a", ["Café", None, "Café", "345", "東京"], dtype=pl.Categorical)
@@ -229,7 +167,6 @@ def test_cat_len_chars() -> None:
     assert_frame_equal(result_df, expected_df)
 
 
-@pytest.mark.usefixtures("test_global_and_local")
 def test_starts_ends_with() -> None:
     s = pl.Series(
         "a",
@@ -301,3 +238,17 @@ def test_cat_slice() -> None:
         "",
         None,
     ]
+
+
+def test_cat_order_flag_csv_read_23823() -> None:
+    data = BytesIO(b"colx,coly\nabc,123\n#not_a_row\nxyz,456")
+    lf = pl.scan_csv(
+        source=data,
+        comment_prefix="#",
+        schema_overrides={"colx": pl.Categorical},
+    )
+    expected = pl.DataFrame(
+        {"colx": ["abc", "xyz"], "coly": [123, 456]},
+        schema_overrides={"colx": pl.Categorical},
+    )
+    assert_frame_equal(expected, lf.sort("colx", descending=False).collect())
