@@ -4,7 +4,6 @@ import pytest
 
 import polars as pl
 from polars.testing import assert_frame_equal, assert_series_equal
-from tests.unit.conftest import with_string_cache_if_auto_streaming
 
 inf = float("inf")
 
@@ -88,34 +87,28 @@ def test_cut_bin_name_in_agg_context() -> None:
         qcut=pl.col("a").qcut([1], include_breaks=True).over(1),
         qcut_uniform=pl.col("a").qcut(1, include_breaks=True).over(1),
     )
-    schema = pl.Struct(
-        {"breakpoint": pl.Float64, "category": pl.Categorical("physical")}
-    )
+    schema = pl.Struct({"breakpoint": pl.Float64, "category": pl.Categorical()})
     assert df.schema == {"cut": schema, "qcut": schema, "qcut_uniform": schema}
 
 
 @pytest.mark.parametrize(
-    ("breaks", "expected_labels", "expected_physical", "expected_unique"),
+    ("breaks", "expected_labels", "expected_unique"),
     [
         (
             [2, 4],
             pl.Series("x", ["(-inf, 2]", "(-inf, 2]", "(2, 4]", "(2, 4]", "(4, inf]"]),
-            pl.Series("x", [0, 0, 1, 1, 2], dtype=pl.UInt32),
             3,
         ),
         (
             [99, 101],
             pl.Series("x", 5 * ["(-inf, 99]"]),
-            pl.Series("x", 5 * [0], dtype=pl.UInt32),
             1,
         ),
     ],
 )
-@with_string_cache_if_auto_streaming
 def test_cut_fast_unique_15981(
     breaks: list[int],
     expected_labels: pl.Series,
-    expected_physical: pl.Series,
     expected_unique: int,
 ) -> None:
     s = pl.Series("x", [1, 2, 3, 4, 5])
@@ -124,7 +117,6 @@ def test_cut_fast_unique_15981(
     s_cut = s.cut(breaks, include_breaks=include_breaks)
 
     assert_series_equal(s_cut.cast(pl.String), expected_labels)
-    assert_series_equal(s_cut.to_physical(), expected_physical)
     assert s_cut.n_unique() == s_cut.to_physical().n_unique() == expected_unique
     s_cut.to_frame().group_by(s.name).len()
 
@@ -134,6 +126,5 @@ def test_cut_fast_unique_15981(
     )
 
     assert_series_equal(s_cut.cast(pl.String), expected_labels)
-    assert_series_equal(s_cut.to_physical(), expected_physical)
     assert s_cut.n_unique() == s_cut.to_physical().n_unique() == expected_unique
     s_cut.to_frame().group_by(s.name).len()
