@@ -89,7 +89,7 @@ def test_explode_correct_for_slice() -> None:
                 how="cross",
             )
         )
-        .sort("group")
+        .sort("group", maintain_order=True)
         .with_row_index()
     )
     expected = pl.DataFrame(
@@ -123,20 +123,26 @@ def test_sliced_null_explode() -> None:
     assert s.slice(2, 4).list.explode().to_list() == [True, False, None, True]
 
 
-def test_explode_in_agg_context() -> None:
+@pytest.mark.parametrize("maintain_order", [False, True])
+def test_explode_in_agg_context(maintain_order: bool) -> None:
     df = pl.DataFrame(
         {"idxs": [[0], [1], [0, 2]], "array": [[0.0, 3.5], [4.6, 0.0], [0.0, 7.8, 0.0]]}
     )
 
-    assert (
+    assert_frame_equal(
         df.with_row_index()
         .explode("idxs")
-        .group_by("index")
-        .agg(pl.col("array").flatten())
-    ).to_dict(as_series=False) == {
-        "index": [0, 1, 2],
-        "array": [[0.0, 3.5], [4.6, 0.0], [0.0, 7.8, 0.0, 0.0, 7.8, 0.0]],
-    }
+        .group_by("index", maintain_order=maintain_order)
+        .agg(pl.col("array").flatten()),
+        pl.DataFrame(
+            {
+                "index": [0, 1, 2],
+                "array": [[0.0, 3.5], [4.6, 0.0], [0.0, 7.8, 0.0, 0.0, 7.8, 0.0]],
+            },
+            schema_overrides={"index": pl.get_index_type()},
+        ),
+        check_row_order=maintain_order,
+    )
 
 
 def test_explode_inner_lists_3985() -> None:
@@ -289,7 +295,7 @@ def test_df_explode_with_array() -> None:
             "val": ["x", "x", "y", "y", "z", "q", "q"],
         }
     )
-    assert_frame_equal(df.explode(pl.col("arr")), expected_by_arr)
+    assert_frame_equal(df.explode("arr"), expected_by_arr)
 
     expected_by_list = pl.DataFrame(
         {
@@ -303,7 +309,7 @@ def test_df_explode_with_array() -> None:
             "val": pl.String,
         },
     )
-    assert_frame_equal(df.explode(pl.col("list")), expected_by_list)
+    assert_frame_equal(df.explode("list"), expected_by_list)
 
     df = pl.DataFrame(
         {
