@@ -17,10 +17,10 @@ if TYPE_CHECKING:
 def test_simplify_expression_lit_true_4376() -> None:
     df = pl.DataFrame([[1, 4, 7], [2, 5, 8], [3, 6, 9]])
     assert df.lazy().filter(pl.lit(True) | (pl.col("column_0") == 1)).collect(
-        simplify_expression=True
+        optimizations=pl.QueryOptFlags(simplify_expression=True),
     ).rows() == [(1, 2, 3), (4, 5, 6), (7, 8, 9)]
     assert df.lazy().filter((pl.col("column_0") == 1) | pl.lit(True)).collect(
-        simplify_expression=True
+        optimizations=pl.QueryOptFlags(simplify_expression=True),
     ).rows() == [(1, 2, 3), (4, 5, 6), (7, 8, 9)]
 
 
@@ -73,7 +73,8 @@ def test_group_by_filter_all_true() -> None:
     assert out.to_dict(as_series=False) == {"n_unique": [1, 1]}
 
 
-def test_filter_is_in_4572() -> None:
+@pytest.mark.parametrize("maintain_order", [False, True])
+def test_filter_is_in_4572(maintain_order: bool) -> None:
     df = pl.DataFrame({"id": [1, 2, 1, 2], "k": ["a"] * 2 + ["b"] * 2})
     expected = df.group_by("id").agg(pl.col("k").filter(k="a").implode()).sort("id")
     result = (
@@ -84,14 +85,24 @@ def test_filter_is_in_4572() -> None:
     assert_frame_equal(result, expected)
     result = (
         df.sort("id")
-        .group_by("id")
+        .group_by("id", maintain_order=maintain_order)
         .agg(pl.col("k").filter(pl.col("k").is_in(["a"])).implode())
     )
-    assert_frame_equal(result, expected)
+    assert_frame_equal(result, expected, check_row_order=maintain_order)
 
 
 @pytest.mark.parametrize(
-    "dtype", [pl.Int32, pl.Boolean, pl.String, pl.Binary, pl.List(pl.Int64), pl.Object]
+    "dtype",
+    [
+        pl.Int32,
+        pl.Boolean,
+        pl.String,
+        pl.Binary,
+        pl.List(pl.Int64),
+        pytest.param(
+            pl.Object, marks=pytest.mark.may_fail_cloud
+        ),  # reason: object dtype
+    ],
 )
 def test_filter_on_empty(dtype: PolarsDataType) -> None:
     df = pl.DataFrame({"a": []}, schema={"a": dtype})
@@ -153,7 +164,6 @@ def test_binary_simplification_5971() -> None:
     ]
 
 
-@pytest.mark.usefixtures("test_global_and_local")
 def test_categorical_string_comparison_6283() -> None:
     scores = pl.DataFrame(
         {
@@ -212,7 +222,7 @@ def test_agg_function_of_filter_10565() -> None:
     ) == {"a": []}
 
     assert df_str.lazy().filter(pl.col("a").n_unique().over("a") == 1).collect(
-        predicate_pushdown=False
+        optimizations=pl.QueryOptFlags(predicate_pushdown=False)
     ).to_dict(as_series=False) == {"a": []}
 
 
