@@ -2,7 +2,6 @@ import pytest
 
 import polars as pl
 import polars.selectors as cs
-from polars import StringCache
 from polars.testing import assert_frame_equal
 
 
@@ -97,8 +96,7 @@ def test_unpivot_empty_18170() -> None:
     )
 
 
-@StringCache()
-def test_unpivot_categorical_global() -> None:
+def test_unpivot_categorical() -> None:
     df = pl.DataFrame(
         {
             "index": [0, 1],
@@ -107,14 +105,9 @@ def test_unpivot_categorical_global() -> None:
         }
     )
     out = df.unpivot(["1", "2"], index="index")
-    assert out.dtypes == [pl.Int64, pl.String, pl.Categorical(ordering="physical")]
+    assert out.dtypes == [pl.Int64, pl.String, pl.Categorical(ordering="lexical")]
     assert out.to_dict(as_series=False) == {
         "index": [0, 1, 0, 1],
         "variable": ["1", "1", "2", "2"],
         "value": ["a", "b", "b", "c"],
     }
-
-
-def test_unpivot_categorical_raise_19770() -> None:
-    with pytest.raises(pl.exceptions.ComputeError):
-        (pl.DataFrame({"x": ["foo"]}).cast(pl.Categorical).unpivot())
