@@ -137,19 +137,19 @@ def test_join_lazy_on_df() -> None:
 
     with pytest.raises(
         TypeError,
-        match="expected `other` .* to be a LazyFrame.* not 'DataFrame'",
+        match="expected `other` .*to be a 'LazyFrame'.* not 'DataFrame'",
     ):
         df_left.lazy().join(df_right, on="Id")  # type: ignore[arg-type]
 
     with pytest.raises(
         TypeError,
-        match="expected `other` .* to be a LazyFrame.* not 'DataFrame'",
+        match="expected `other` .*to be a 'LazyFrame'.* not 'DataFrame'",
     ):
         df_left.lazy().join_asof(df_right, on="Id")  # type: ignore[arg-type]
 
     with pytest.raises(
         TypeError,
-        match="expected `other` .* to be a LazyFrame.* not 'pandas.core.frame.DataFrame'",
+        match="expected `other` .*to be a 'LazyFrame'.* not 'pandas.core.frame.DataFrame'",
     ):
         df_left.lazy().join_asof(df_right.to_pandas(), on="Id")  # type: ignore[arg-type]
 
@@ -254,7 +254,7 @@ def test_window_expression_different_group_length() -> None:
         pl.DataFrame({"groups": ["a", "a", "b", "a", "b"]}).select(
             pl.col("groups").map_elements(lambda _: pl.Series([1, 2])).over("groups")
         )
-    except ComputeError as exc:
+    except ShapeError as exc:
         msg = str(exc)
         assert (
             "the length of the window expression did not match that of the group" in msg
@@ -343,9 +343,7 @@ def test_invalid_dtype() -> None:
 def test_arr_eval_named_cols() -> None:
     df = pl.DataFrame({"A": ["a", "b"], "B": [["a", "b"], ["c", "d"]]})
 
-    with pytest.raises(
-        ComputeError,
-    ):
+    with pytest.raises(ComputeError):
         df.select(pl.col("B").list.eval(pl.element().append(pl.col("A"))))
 
 
@@ -427,21 +425,6 @@ def test_date_string_comparison(e: pl.Expr) -> None:
         df.select(e)
 
 
-def test_err_on_multiple_column_expansion() -> None:
-    # this would be a great feature :)
-    with pytest.raises(
-        ComputeError, match=r"expanding more than one `col` is not allowed"
-    ):
-        pl.DataFrame(
-            {
-                "a": [1],
-                "b": [2],
-                "c": [3],
-                "d": [4],
-            }
-        ).select([pl.col(["a", "b"]) + pl.col(["c", "d"])])
-
-
 def test_compare_different_len() -> None:
     df = pl.DataFrame(
         {
@@ -739,5 +722,7 @@ def test_raise_on_different_results_20104() -> None:
 
     with pytest.raises(pl.exceptions.SchemaError):
         df.rolling("x", period="3i").agg(
-            result=pl.col("x").gather_every(2, offset=1).map_batches(pl.Series.min)
+            result=pl.col("x")
+            .gather_every(2, offset=1)
+            .map_batches(pl.Series.min, return_dtype=pl.Float64)
         )
